'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var path = require('path');
var path__default = _interopDefault(path);
var fs = require('fs');
var sourcemapCodec = require('sourcemap-codec');
var MagicString = require('magic-string');
var MagicString__default = _interopDefault(MagicString);
var acorn = require('acorn');
var locateCharacter = require('locate-character');
var Immutable = _interopDefault(require('immutable'));
var isReference = _interopDefault(require('is-reference'));
var sourceMap = require('source-map');
var EventEmitter = _interopDefault(require('events'));
var createFilter = _interopDefault(require('rollup-pluginutils/src/createFilter.js'));
var relative$1 = _interopDefault(require('require-relative'));

var DEBUG = false;
var map = new Map();

var timeStartHelper = void 0;
var timeEndHelper = void 0;

if (typeof process === 'undefined' || typeof process.hrtime === 'undefined') {
	timeStartHelper = function timeStartHelper() {
		return window.performance.now();
	};

	timeEndHelper = function timeEndHelper(previous) {
		return window.performance.now() - previous;
	};
} else {
	timeStartHelper = function timeStartHelper() {
		return process.hrtime();
	};

	timeEndHelper = function timeEndHelper(previous) {
		var hrtime = process.hrtime(previous);
		return hrtime[0] * 1e3 + Math.floor(hrtime[1] / 1e6);
	};
}

function timeStart(label) {
	if (!map.has(label)) {
		map.set(label, {
			time: 0
		});
	}
	map.get(label).start = timeStartHelper();
}

function timeEnd(label) {
	if (map.has(label)) {
		var item = map.get(label);
		item.time += timeEndHelper(item.start);
	}
}

function flushTime() {
	var log = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : defaultLog;
	var _iteratorNormalCompletion = true;
	var _didIteratorError = false;
	var _iteratorError = undefined;

	try {
		for (var _iterator = map.entries()[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
			var item = _step.value;

			log(item[0], item[1].time);
		}
	} catch (err) {
		_didIteratorError = true;
		_iteratorError = err;
	} finally {
		try {
			if (!_iteratorNormalCompletion && _iterator.return) {
				_iterator.return();
			}
		} finally {
			if (_didIteratorError) {
				throw _iteratorError;
			}
		}
	}

	map.clear();
}

function defaultLog(label, time) {
	if (DEBUG) {
		/* eslint-disable no-console */
		console.info('%dms: %s', time, label);
		/* eslint-enable no-console */
	}
}

var absolutePath = /^(?:\/|(?:[A-Za-z]:)?[\\|/])/;
var relativePath = /^\.?\.\//;

function isAbsolute(path$$1) {
	return absolutePath.test(path$$1);
}

function isRelative(path$$1) {
	return relativePath.test(path$$1);
}

function normalize(path$$1) {
	return path$$1.replace(/\\/g, '/');
}

function mkdirpath(path$$1) {
	var dir = path.dirname(path$$1);
	try {
		fs.readdirSync(dir);
	} catch (err) {
		mkdirpath(dir);
		try {
			fs.mkdirSync(dir);
		} catch (err2) {
			if (err2.code !== 'EEXIST') {
				throw err2;
			}
		}
	}
}

function writeFile$1(dest, data) {
	return new Promise(function (fulfil, reject) {
		mkdirpath(dest);

		fs.writeFile(dest, data, function (err) {
			if (err) {
				reject(err);
			} else {
				fulfil();
			}
		});
	});
}

var keys = Object.keys;
function blank() {
	return Object.create(null);
}

function forOwn(object, func) {
	Object.keys(object).forEach(function (key) {
		return func(object[key], key);
	});
}

function assign(target) {
	for (var _len = arguments.length, sources = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
		sources[_key - 1] = arguments[_key];
	}

	sources.forEach(function (source) {
		for (var key in source) {
			if (source.hasOwnProperty(key)) target[key] = source[key];
		}
	});

	return target;
}

function mapSequence(array, fn) {
	var results = [];
	var promise = Promise.resolve();

	function next(member, i) {
		return Promise.resolve(fn(member)).then(function (value) {
			return results[i] = value;
		});
	}

	var _loop = function _loop(i) {
		promise = promise.then(function () {
			return next(array[i], i);
		});
	};

	for (var i = 0; i < array.length; i += 1) {
		_loop(i);
	}

	return promise.then(function () {
		return results;
	});
}

function runSequence(array) {
	return mapSequence(array, function (i) {
		return i;
	});
}

function error(props) {
	// use the same constructor as props (if it's an error object)
	// so that err.name is preserved etc
	// (Object.keys below does not update these values because they
	// are properties on the prototype chain)
	// basically if props is a SyntaxError it will not be overriden as a generic Error
	var constructor = props instanceof Error ? props.constructor : Error;
	var err = new constructor(props.message);

	Object.keys(props).forEach(function (key) {
		err[key] = props[key];
	});

	throw err;
}

// this looks ridiculous, but it prevents sourcemap tooling from mistaking
// this for an actual sourceMappingURL
var SOURCEMAPPING_URL = 'sourceMa';
SOURCEMAPPING_URL += 'ppingURL';

var SOURCEMAPPING_URL_RE = new RegExp('^#\\s+' + SOURCEMAPPING_URL + '=.+\\n?');

function ensureArray(thing) {
	if (Array.isArray(thing)) return thing;
	if (thing == undefined) return [];
	return [thing];
}

// second argument is required because rollup's Node API
// passes inputOptions first and then output options
// unlike the config file which passes whole options in one go
function deprecateOptions(options, deprecateConfig) {
	var deprecations = [];
	if (deprecateConfig.input) deprecateInputOptions();
	if (deprecateConfig.output) deprecateOutputOptions();

	return deprecations;

	function deprecateInputOptions() {
		if (!options.input && options.entry) deprecate('entry', 'input');
		if (options.moduleName) deprecate('moduleName', 'output.name', true);
		if (options.name) deprecate('name', 'output.name', true);
		if (options.extend) deprecate('extend', 'output.extend', true);
		if (options.globals) deprecate('globals', 'output.globals', true);
		if (options.indent) deprecate('indent', 'output.indent', true);
		if (options.noConflict) deprecate('noConflict', 'output.noConflict', true);
		if (options.paths) deprecate('paths', 'output.paths', true);
		if (options.sourcemap) deprecate('sourcemap', 'output.sourcemap', true);
		if (options.sourceMap) deprecate('sourceMap', 'output.sourcemap', true);
		if (options.sourceMapFile) deprecate('sourceMapFile', 'output.sourcemapFile', true);
		if (options.useStrict) deprecate('useStrict', 'output.strict', true);
		if (options.format) deprecate('format', 'output.format', true);

		if (options.targets) {
			deprecations.push({ old: 'targets', new: 'output' });

			// as targets is an array and we need to merge other output options
			// like sourcemap etc.
			options.output = options.targets.map(function (t) {
				return Object.assign({}, t, options.output);
			});
			delete options.targets;

			var deprecatedDest = false;
			options.output.forEach(function (output) {
				if (output.dest) {
					if (!deprecatedDest) {
						deprecations.push({ old: 'targets.dest', new: 'output.file' });
						deprecatedDest = true;
					}
					output.file = output.dest;
					delete output.dest;
				}
			});
		} else if (options.dest) {
			deprecations.push({ old: 'dest', new: 'output.file' });
			options.output = {
				file: options.dest,
				format: options.format
			};
			delete options.dest;
		}

		if (options.pureExternalModules) {
			deprecations.push({ old: 'pureExternalModules', new: 'treeshake.pureExternalModules' });
			if (options.treeshake === undefined) {
				options.treeshake = {};
			}
			if (options.treeshake) {
				options.treeshake.pureExternalModules = options.pureExternalModules;
			}
			delete options.pureExternalModules;
		}
	}

	function deprecateOutputOptions() {
		if (options.output && options.output.moduleId) {
			options.output.amd = { id: options.moduleId };
			deprecations.push({ old: 'moduleId', new: 'amd' });
			delete options.output.moduleId;
		}
	}

	// a utility function to add deprecations for straightforward options
	function deprecate(oldOption, newOption, shouldDelete) {
		deprecations.push({ new: newOption, old: oldOption });

		if (newOption.indexOf('output') > -1) {
			options.output = options.output || {};
			options.output[newOption.replace(/output\./, '')] = options[oldOption];
		} else {
			options[newOption] = options[oldOption];
		}

		if (shouldDelete) delete options[oldOption];
	}
}

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function normalizeObjectOptionValue(optionValue) {
	if (!optionValue) {
		return optionValue;
	}
	if ((typeof optionValue === 'undefined' ? 'undefined' : _typeof(optionValue)) !== 'object') {
		return {};
	}
	return optionValue;
}

function mergeOptions(_ref) {
	var config = _ref.config,
	    _ref$command = _ref.command,
	    command = _ref$command === undefined ? {} : _ref$command,
	    deprecateConfig = _ref.deprecateConfig,
	    _ref$defaultOnWarn = _ref.defaultOnWarn,
	    defaultOnWarn = _ref$defaultOnWarn === undefined ? function (warning) {
		return console.warn(warning.message);
	} : _ref$defaultOnWarn;

	var deprecations = deprecate(config, command, deprecateConfig);

	var getOption = function getOption(config) {
		return function (name) {
			return command[name] !== undefined ? command[name] : config[name];
		};
	};

	var getInputOption = getOption(config);
	var getOutputOption = getOption(config.output || {});

	function getObjectOption(name) {
		var commandOption = normalizeObjectOptionValue(command[name]);
		var configOption = normalizeObjectOptionValue(config[name]);
		if (commandOption !== undefined) {
			return commandOption && configOption ? Object.assign({}, configOption, commandOption) : commandOption;
		}
		return configOption;
	}

	var onwarn = config.onwarn;
	var warn = void 0;

	if (onwarn) {
		warn = function warn(warning) {
			onwarn(warning, defaultOnWarn);
		};
	} else {
		warn = defaultOnWarn;
	}

	var inputOptions = {
		input: getInputOption('input'),
		legacy: getInputOption('legacy'),
		treeshake: getObjectOption('treeshake'),
		acorn: config.acorn,
		context: config.context,
		moduleContext: config.moduleContext,
		plugins: config.plugins,
		onwarn: warn,
		watch: config.watch,
		cache: getInputOption('cache'),
		preferConst: getInputOption('preferConst'),
		experimentalDynamicImport: getInputOption('experimentalDynamicImport')
	};

	// legacy, to ensure e.g. commonjs plugin still works
	inputOptions.entry = inputOptions.input;

	var commandExternal = (command.external || '').split(',');
	var configExternal = config.external;

	if (command.globals) {
		var globals = Object.create(null);

		command.globals.split(',').forEach(function (str) {
			var names = str.split(':');
			globals[names[0]] = names[1];

			// Add missing Module IDs to external.
			if (commandExternal.indexOf(names[0]) === -1) {
				commandExternal.push(names[0]);
			}
		});

		command.globals = globals;
	}

	if (typeof configExternal === 'function') {
		inputOptions.external = function (id) {
			for (var _len = arguments.length, rest = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
				rest[_key - 1] = arguments[_key];
			}

			return configExternal.apply(undefined, [id].concat(rest)) || ~commandExternal.indexOf(id);
		};
	} else {
		inputOptions.external = (configExternal || []).concat(commandExternal);
	}

	if (command.silent) {
		inputOptions.onwarn = function () {};
	}

	var baseOutputOptions = {
		extend: getOutputOption('extend'),
		amd: Object.assign({}, config.amd, command.amd),
		banner: getOutputOption('banner'),
		footer: getOutputOption('footer'),
		intro: getOutputOption('intro'),
		format: getOutputOption('format'),
		outro: getOutputOption('outro'),
		sourcemap: getOutputOption('sourcemap'),
		sourcemapFile: getOutputOption('sourcemapFile'),
		name: getOutputOption('name'),
		globals: getOutputOption('globals'),
		interop: getOutputOption('interop'),
		legacy: getOutputOption('legacy'),
		freeze: getOutputOption('freeze'),
		indent: getOutputOption('indent'),
		strict: getOutputOption('strict'),
		noConflict: getOutputOption('noConflict'),
		paths: getOutputOption('paths'),
		exports: getOutputOption('exports'),
		file: getOutputOption('file')
	};

	var mergedOutputOptions = void 0;
	if (Array.isArray(config.output)) {
		mergedOutputOptions = config.output.map(function (output) {
			return Object.assign({}, output, command.output);
		});
	} else if (config.output && command.output) {
		mergedOutputOptions = [Object.assign({}, config.output, command.output)];
	} else {
		mergedOutputOptions = command.output || config.output ? ensureArray(command.output || config.output) : [{
			file: command.output ? command.output.file : null,
			format: command.output ? command.output.format : null
		}];
	}

	var outputOptions = mergedOutputOptions.map(function (output) {
		return Object.assign({}, baseOutputOptions, output);
	});

	// check for errors
	var validKeys = [].concat(_toConsumableArray(Object.keys(inputOptions)), _toConsumableArray(Object.keys(baseOutputOptions)), ['pureExternalModules' // (backward compatibility) till everyone moves to treeshake.pureExternalModules
	]);
	var errors = [].concat(_toConsumableArray(Object.keys(config || {})), _toConsumableArray(Object.keys(config.output || {}))).filter(function (k) {
		return k !== 'output' && validKeys.indexOf(k) === -1;
	});

	return {
		inputOptions: inputOptions,
		outputOptions: outputOptions,
		deprecations: deprecations,
		optionError: errors.length ? 'Unknown option found: ' + errors.join(', ') + '. Allowed keys: ' + validKeys.join(', ') : null
	};
}

function deprecate(config) {
	var command = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
	var deprecateConfig = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : { input: true, output: true };

	var deprecations = [];

	// CLI
	if (command.id) {
		deprecations.push({
			old: '-u/--id',
			new: '--amd.id'
		});
		(command.amd || (command.amd = {})).id = command.id;
	}

	if (typeof command.output === 'string') {
		deprecations.push({
			old: '--output',
			new: '--output.file'
		});
		command.output = { file: command.output };
	}

	if (command.format) {
		deprecations.push({
			old: '--format',
			new: '--output.format'
		});
		(command.output || (command.output = {})).format = command.format;
	}

	// config file
	deprecations.push.apply(deprecations, _toConsumableArray(deprecateOptions(config, deprecateConfig)));
	return deprecations;
}

// Return the first non-falsy result from an array of
// maybe-sync, maybe-promise-returning functions
function first(candidates) {
	return function () {
		for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
			args[_key] = arguments[_key];
		}

		return candidates.reduce(function (promise, candidate) {
			return promise.then(function (result) {
				return result != null ? result : Promise.resolve(candidate.apply(undefined, args));
			});
		}, Promise.resolve());
	};
}

function find(array, fn) {
	for (var i = 0; i < array.length; i += 1) {
		if (fn(array[i], i)) return array[i];
	}

	return null;
}

// Dynamic Import support for acorn
function wrapDynamicImportPlugin(acorn$$1) {
  var moduleDynamicImportsReturnBinding = void 0;
  acorn$$1.tokTypes._import.startsExpr = true;
  acorn$$1.plugins.dynamicImport = function (instance) {
    instance.extend('parseStatement', function (nextMethod) {
      return function parseStatement() {
        var node = this.startNode();
        if (this.type === acorn$$1.tokTypes._import) {
          var nextToken = this.input[this.pos];
          if (nextToken === acorn$$1.tokTypes.parenL.label) {
            var expr = this.parseExpression();
            return this.parseExpressionStatement(node, expr);
          }
        }

        for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
          args[_key] = arguments[_key];
        }

        return nextMethod.apply(this, args);
      };
    });

    instance.extend('parseExprAtom', function (nextMethod) {
      return function parseExprAtom(refDestructuringErrors) {
        if (this.type === acorn$$1.tokTypes._import) {
          var node = this.startNode();
          this.next();
          if (this.type !== acorn$$1.tokTypes.parenL) {
            this.unexpected();
          }
          if (moduleDynamicImportsReturnBinding) {
            moduleDynamicImportsReturnBinding.push(node);
          }
          return this.finishNode(node, 'Import');
        }
        return nextMethod.call(this, refDestructuringErrors);
      };
    });
  };

  // returns a function to set the dynamicImport array for getting these nodes during parsing
  return function setModuleDynamicImportsReturnBinding(_moduleDynamicImportsReturnBinding) {
    moduleDynamicImportsReturnBinding = _moduleDynamicImportsReturnBinding;
  };
}

var reservedWords = 'break case class catch const continue debugger default delete do else export extends finally for function if import in instanceof let new return super switch this throw try typeof var void while with yield enum await implements package protected static interface private public'.split(' ');
var builtins = 'Infinity NaN undefined null true false eval uneval isFinite isNaN parseFloat parseInt decodeURI decodeURIComponent encodeURI encodeURIComponent escape unescape Object Function Boolean Symbol Error EvalError InternalError RangeError ReferenceError SyntaxError TypeError URIError Number Math Date String RegExp Array Int8Array Uint8Array Uint8ClampedArray Int16Array Uint16Array Int32Array Uint32Array Float32Array Float64Array Map Set WeakMap WeakSet SIMD ArrayBuffer DataView JSON Promise Generator GeneratorFunction Reflect Proxy Intl'.split(' ');

var blacklisted = blank();
reservedWords.concat(builtins).forEach(function (word) {
	return blacklisted[word] = true;
});

var illegalCharacters = /[^$_a-zA-Z0-9]/g;

var startsWithDigit = function startsWithDigit(str) {
	return (/\d/.test(str[0])
	);
};

function isLegal(str) {
	if (startsWithDigit(str) || blacklisted[str]) {
		return false;
	}
	if (illegalCharacters.test(str)) {
		return false;
	}
	return true;
}

function makeLegal(str) {
	str = str.replace(/-(\w)/g, function (_, letter) {
		return letter.toUpperCase();
	}).replace(illegalCharacters, '_');

	if (startsWithDigit(str) || blacklisted[str]) str = '_' + str;

	return str;
}

function spaces(i) {
	var result = '';
	while (i--) {
		result += ' ';
	}return result;
}

function tabsToSpaces(str) {
	return str.replace(/^\t+/, function (match) {
		return match.split('\t').join('  ');
	});
}

function getCodeFrame(source, line, column) {
	var lines = source.split('\n');

	var frameStart = Math.max(0, line - 3);
	var frameEnd = Math.min(line + 2, lines.length);

	lines = lines.slice(frameStart, frameEnd);
	while (!/\S/.test(lines[lines.length - 1])) {
		lines.pop();
		frameEnd -= 1;
	}

	var digits = String(frameEnd).length;

	return lines.map(function (str, i) {
		var isErrorLine = frameStart + i + 1 === line;

		var lineNum = String(i + frameStart + 1);
		while (lineNum.length < digits) {
			lineNum = ' ' + lineNum;
		}if (isErrorLine) {
			var indicator = spaces(digits + 2 + tabsToSpaces(str.slice(0, column)).length) + '^';
			return lineNum + ': ' + tabsToSpaces(str) + '\n' + indicator;
		}

		return lineNum + ': ' + tabsToSpaces(str);
	}).join('\n');
}

function relativeId(id) {
	if (typeof process === 'undefined' || !isAbsolute(id)) return id;
	return path.relative(process.cwd(), id);
}

var UNKNOWN_VALUE = { toString: function toString() {
		return '[[UNKNOWN]]';
	} };

var UNKNOWN_ASSIGNMENT = {
	type: 'UNKNOWN',
	reassignPath: function reassignPath() {},
	forEachReturnExpressionWhenCalledAtPath: function forEachReturnExpressionWhenCalledAtPath() {},
	hasEffectsWhenAccessedAtPath: function hasEffectsWhenAccessedAtPath(path$$1) {
		return path$$1.length > 0;
	},
	hasEffectsWhenAssignedAtPath: function hasEffectsWhenAssignedAtPath(path$$1) {
		return path$$1.length > 0;
	},
	hasEffectsWhenCalledAtPath: function hasEffectsWhenCalledAtPath() {
		return true;
	},
	someReturnExpressionWhenCalledAtPath: function someReturnExpressionWhenCalledAtPath() {
		return true;
	},
	toString: function toString() {
		return '[[UNKNOWN]]';
	}
};

var UNDEFINED_ASSIGNMENT = {
	type: 'UNDEFINED',
	reassignPath: function reassignPath() {},
	forEachReturnExpressionWhenCalledAtPath: function forEachReturnExpressionWhenCalledAtPath() {},
	hasEffectsWhenAccessedAtPath: function hasEffectsWhenAccessedAtPath(path$$1) {
		return path$$1.length > 0;
	},
	hasEffectsWhenAssignedAtPath: function hasEffectsWhenAssignedAtPath(path$$1) {
		return path$$1.length > 0;
	},
	hasEffectsWhenCalledAtPath: function hasEffectsWhenCalledAtPath() {
		return true;
	},
	someReturnExpressionWhenCalledAtPath: function someReturnExpressionWhenCalledAtPath() {
		return true;
	},
	toString: function toString() {
		return '[[UNDEFINED]]';
	}
};

var _createClass$3 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$3(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/* eslint-disable no-unused-vars */

var Variable = function () {
	function Variable(name) {
		_classCallCheck$3(this, Variable);

		this.name = name;
	}

	/**
  * Binds identifiers that reference this variable to this variable.
  * Necessary to be able to change variable names.
  * @param {Identifier} identifier
  */


	_createClass$3(Variable, [{
		key: 'addReference',
		value: function addReference(identifier) {}

		/**
   * @param {String[]} path
   * @param {ExecutionPathOptions} options
   */

	}, {
		key: 'reassignPath',
		value: function reassignPath(path$$1, options) {}

		/**
   * @param {String[]} path
   * @param {CallOptions} callOptions
   * @param {Function} callback
   * @param {ExecutionPathOptions} options
   */

	}, {
		key: 'forEachReturnExpressionWhenCalledAtPath',
		value: function forEachReturnExpressionWhenCalledAtPath(path$$1, callOptions, callback, options) {}

		/**
   * @returns {String}
   */

	}, {
		key: 'getName',
		value: function getName() {
			return this.name;
		}

		/**
   * @param {String[]} path
   * @param {ExecutionPathOptions} options
   * @return {boolean}
   */

	}, {
		key: 'hasEffectsWhenAccessedAtPath',
		value: function hasEffectsWhenAccessedAtPath(path$$1, options) {
			return path$$1.length > 0;
		}

		/**
   * @param {String[]} path
   * @param {ExecutionPathOptions} options
   * @return {boolean}
   */

	}, {
		key: 'hasEffectsWhenAssignedAtPath',
		value: function hasEffectsWhenAssignedAtPath(path$$1, options) {
			return true;
		}

		/**
   * @param {String[]} path
   * @param {CallOptions} callOptions
   * @param {ExecutionPathOptions} options
   * @return {boolean}
   */

	}, {
		key: 'hasEffectsWhenCalledAtPath',
		value: function hasEffectsWhenCalledAtPath(path$$1, callOptions, options) {
			return true;
		}

		/**
   * Marks this variable as being part of the bundle, which is usually the case when one of
   * its identifiers becomes part of the bundle. Returns true if it has not been included
   * previously.
   * Once a variable is included, it should take care all its declarations are included.
   * @returns {boolean}
   */

	}, {
		key: 'includeVariable',
		value: function includeVariable() {
			if (this.included) {
				return false;
			}
			this.included = true;
			return true;
		}

		/**
   * @param {String[]} path
   * @param {CallOptions} callOptions
   * @param {Function} predicateFunction
   * @param {ExecutionPathOptions} options
   * @returns {boolean}
   */

	}, {
		key: 'someReturnExpressionWhenCalledAtPath',
		value: function someReturnExpressionWhenCalledAtPath(path$$1, callOptions, predicateFunction, options) {
			return predicateFunction(options)(UNKNOWN_ASSIGNMENT);
		}
	}]);

	return Variable;
}();

var _createClass$2 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$2(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var NamespaceVariable = function (_Variable) {
	_inherits(NamespaceVariable, _Variable);

	function NamespaceVariable(module) {
		_classCallCheck$2(this, NamespaceVariable);

		var _this = _possibleConstructorReturn(this, (NamespaceVariable.__proto__ || Object.getPrototypeOf(NamespaceVariable)).call(this, module.basename()));

		_this.isNamespace = true;
		_this.module = module;
		_this.needsNamespaceBlock = false;

		_this.originals = blank();
		module.getExports().concat(module.getReexports()).forEach(function (name) {
			_this.originals[name] = module.traceExport(name);
		});
		return _this;
	}

	_createClass$2(NamespaceVariable, [{
		key: 'addReference',
		value: function addReference(identifier) {
			this.name = identifier.name;
		}
	}, {
		key: 'includeVariable',
		value: function includeVariable() {
			if (!_get(NamespaceVariable.prototype.__proto__ || Object.getPrototypeOf(NamespaceVariable.prototype), 'includeVariable', this).call(this)) {
				return false;
			}
			this.needsNamespaceBlock = true;
			forOwn(this.originals, function (original) {
				return original.includeVariable();
			});
			return true;
		}
	}, {
		key: 'renderBlock',
		value: function renderBlock(es, legacy, freeze, indentString) {
			var _this2 = this;

			var members = keys(this.originals).map(function (name) {
				var original = _this2.originals[name];

				if (original.isReassigned && !legacy) {
					return indentString + 'get ' + name + ' () { return ' + original.getName(es) + '; }';
				}

				if (legacy && ~reservedWords.indexOf(name)) name = '\'' + name + '\'';
				return '' + indentString + name + ': ' + original.getName(es);
			});

			var callee = freeze ? legacy ? '(Object.freeze || Object)' : 'Object.freeze' : '';
			return this.module.bundle.varOrConst + ' ' + this.getName(es) + ' = ' + callee + '({\n' + members.join(',\n') + '\n});\n\n';
		}
	}]);

	return NamespaceVariable;
}(Variable);

function extractNames(param) {
	var names = [];
	extractors[param.type](names, param);
	return names;
}

var extractors = {
	Identifier: function Identifier(names, param) {
		names.push(param.name);
	},
	ObjectPattern: function ObjectPattern(names, param) {
		param.properties.forEach(function (prop) {
			extractors[prop.value.type](names, prop.value);
		});
	},
	ArrayPattern: function ArrayPattern(names, param) {
		param.elements.forEach(function (element) {
			if (element) extractors[element.type](names, element);
		});
	},
	RestElement: function RestElement(names, param) {
		extractors[param.argument.type](names, param.argument);
	},
	AssignmentPattern: function AssignmentPattern(names, param) {
		extractors[param.left.type](names, param.left);
	}
};

var _createClass$6 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _toConsumableArray$1(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck$6(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var OPTION_IGNORED_LABELS = 'IGNORED_LABELS';
var OPTION_ACCESSED_NODES = 'ACCESSED_NODES';
var OPTION_ARGUMENTS_VARIABLES = 'ARGUMENTS_VARIABLES';
var OPTION_ASSIGNED_NODES = 'ASSIGNED_NODES';
var OPTION_IGNORE_BREAK_STATEMENTS = 'IGNORE_BREAK_STATEMENTS';
var OPTION_IGNORE_RETURN_AWAIT_YIELD = 'IGNORE_RETURN_AWAIT_YIELD';
var OPTION_NODES_CALLED_AT_PATH_WITH_OPTIONS = 'NODES_CALLED_AT_PATH_WITH_OPTIONS';
var OPTION_REPLACED_VARIABLE_INITS = 'REPLACED_VARIABLE_INITS';
var OPTION_RETURN_EXPRESSIONS_ACCESSED_AT_PATH = 'RETURN_EXPRESSIONS_ACCESSED_AT_PATH';
var OPTION_RETURN_EXPRESSIONS_ASSIGNED_AT_PATH = 'RETURN_EXPRESSIONS_ASSIGNED_AT_PATH';
var OPTION_RETURN_EXPRESSIONS_CALLED_AT_PATH = 'RETURN_EXPRESSIONS_CALLED_AT_PATH';

var RESULT_KEY = {};

/** Wrapper to ensure immutability */

var ExecutionPathOptions = function () {
	_createClass$6(ExecutionPathOptions, null, [{
		key: 'create',

		/**
   * @returns {ExecutionPathOptions}
   */
		value: function create() {
			return new this(Immutable.Map());
		}
	}]);

	function ExecutionPathOptions(optionValues) {
		_classCallCheck$6(this, ExecutionPathOptions);

		this._optionValues = optionValues;
	}

	/**
  * @param {string} option - The name of an option
  * @returns {*} Its value
  */


	_createClass$6(ExecutionPathOptions, [{
		key: 'get',
		value: function get(option) {
			return this._optionValues.get(option);
		}

		/**
   * Returns a new ExecutionPathOptions instance with the given option removed.
   * Does not mutate the current instance. Also works in sub-classes.
   * @param {string} option - The name of an option
   * @returns {*} Its value
   */

	}, {
		key: 'remove',
		value: function remove(option) {
			return new this.constructor(this._optionValues.remove(option));
		}

		/**
   * Returns a new ExecutionPathOptions instance with the given option set to a new value.
   * Does not mutate the current instance. Also works in sub-classes.
   * @param {string} option - The name of an option
   * @param {*} value - The new value of the option
   * @returns {ExecutionPathOptions} A new options instance
   */

	}, {
		key: 'set',
		value: function set(option, value) {
			return new this.constructor(this._optionValues.set(option, value));
		}
	}, {
		key: 'setIn',
		value: function setIn(optionPath, value) {
			return new this.constructor(this._optionValues.setIn(optionPath, value));
		}

		/**
   * @param {String[]} path
   * @param {Node} node
   * @return {ExecutionPathOptions}
   */

	}, {
		key: 'addAccessedNodeAtPath',
		value: function addAccessedNodeAtPath(path$$1, node) {
			return this.setIn([OPTION_ACCESSED_NODES, node].concat(_toConsumableArray$1(path$$1), [RESULT_KEY]), true);
		}

		/**
   * @param {String[]} path
   * @param {CallExpression|Property} callExpression
   * @return {ExecutionPathOptions}
   */

	}, {
		key: 'addAccessedReturnExpressionAtPath',
		value: function addAccessedReturnExpressionAtPath(path$$1, callExpression) {
			return this.setIn([OPTION_RETURN_EXPRESSIONS_ACCESSED_AT_PATH, callExpression].concat(_toConsumableArray$1(path$$1), [RESULT_KEY]), true);
		}

		/**
   * @param {String[]} path
   * @param {Node} node
   * @return {ExecutionPathOptions}
   */

	}, {
		key: 'addAssignedNodeAtPath',
		value: function addAssignedNodeAtPath(path$$1, node) {
			return this.setIn([OPTION_ASSIGNED_NODES, node].concat(_toConsumableArray$1(path$$1), [RESULT_KEY]), true);
		}

		/**
   * @param {String[]} path
   * @param {CallExpression|Property} callExpression
   * @return {ExecutionPathOptions}
   */

	}, {
		key: 'addAssignedReturnExpressionAtPath',
		value: function addAssignedReturnExpressionAtPath(path$$1, callExpression) {
			return this.setIn([OPTION_RETURN_EXPRESSIONS_ASSIGNED_AT_PATH, callExpression].concat(_toConsumableArray$1(path$$1), [RESULT_KEY]), true);
		}

		/**
   * @param {String[]} path
   * @param {Node} node
   * @param {CallOptions} callOptions
   * @return {ExecutionPathOptions}
   */

	}, {
		key: 'addCalledNodeAtPathWithOptions',
		value: function addCalledNodeAtPathWithOptions(path$$1, node, callOptions) {
			return this.setIn([OPTION_NODES_CALLED_AT_PATH_WITH_OPTIONS, node].concat(_toConsumableArray$1(path$$1), [RESULT_KEY, callOptions]), true);
		}

		/**
   * @param {String[]} path
   * @param {CallExpression|Property} callExpression
   * @return {ExecutionPathOptions}
   */

	}, {
		key: 'addCalledReturnExpressionAtPath',
		value: function addCalledReturnExpressionAtPath(path$$1, callExpression) {
			return this.setIn([OPTION_RETURN_EXPRESSIONS_CALLED_AT_PATH, callExpression].concat(_toConsumableArray$1(path$$1), [RESULT_KEY]), true);
		}

		/**
   * @return {ParameterVariable[]}
   */

	}, {
		key: 'getArgumentsVariables',
		value: function getArgumentsVariables() {
			return this.get(OPTION_ARGUMENTS_VARIABLES) || [];
		}

		/**
   * @return {ExecutionPathOptions}
   */

	}, {
		key: 'getHasEffectsWhenCalledOptions',
		value: function getHasEffectsWhenCalledOptions() {
			return this.setIgnoreReturnAwaitYield().setIgnoreBreakStatements(false).setIgnoreNoLabels();
		}

		/**
   * @param {ThisVariable|ParameterVariable} variable
   * @returns {Node}
   */

	}, {
		key: 'getReplacedVariableInit',
		value: function getReplacedVariableInit(variable) {
			return this._optionValues.getIn([OPTION_REPLACED_VARIABLE_INITS, variable]);
		}

		/**
   * @param {String[]} path
   * @param {Node} node
   * @return {boolean}
   */

	}, {
		key: 'hasNodeBeenAccessedAtPath',
		value: function hasNodeBeenAccessedAtPath(path$$1, node) {
			return this._optionValues.getIn([OPTION_ACCESSED_NODES, node].concat(_toConsumableArray$1(path$$1), [RESULT_KEY]));
		}

		/**
   * @param {String[]} path
   * @param {Node} node
   * @return {boolean}
   */

	}, {
		key: 'hasNodeBeenAssignedAtPath',
		value: function hasNodeBeenAssignedAtPath(path$$1, node) {
			return this._optionValues.getIn([OPTION_ASSIGNED_NODES, node].concat(_toConsumableArray$1(path$$1), [RESULT_KEY]));
		}

		/**
   * @param {String[]} path
   * @param {Node} node
   * @param {CallOptions} callOptions
   * @return {boolean}
   */

	}, {
		key: 'hasNodeBeenCalledAtPathWithOptions',
		value: function hasNodeBeenCalledAtPathWithOptions(path$$1, node, callOptions) {
			var previousCallOptions = this._optionValues.getIn([OPTION_NODES_CALLED_AT_PATH_WITH_OPTIONS, node].concat(_toConsumableArray$1(path$$1), [RESULT_KEY]));
			return previousCallOptions && previousCallOptions.find(function (_, otherCallOptions) {
				return otherCallOptions.equals(callOptions);
			});
		}

		/**
   * @param {String[]} path
   * @param {CallExpression|Property} callExpression
   * @return {boolean}
   */

	}, {
		key: 'hasReturnExpressionBeenAccessedAtPath',
		value: function hasReturnExpressionBeenAccessedAtPath(path$$1, callExpression) {
			return this._optionValues.getIn([OPTION_RETURN_EXPRESSIONS_ACCESSED_AT_PATH, callExpression].concat(_toConsumableArray$1(path$$1), [RESULT_KEY]));
		}

		/**
   * @param {String[]} path
   * @param {CallExpression|Property} callExpression
   * @return {boolean}
   */

	}, {
		key: 'hasReturnExpressionBeenAssignedAtPath',
		value: function hasReturnExpressionBeenAssignedAtPath(path$$1, callExpression) {
			return this._optionValues.getIn([OPTION_RETURN_EXPRESSIONS_ASSIGNED_AT_PATH, callExpression].concat(_toConsumableArray$1(path$$1), [RESULT_KEY]));
		}

		/**
   * @param {String[]} path
   * @param {CallExpression|Property} callExpression
   * @return {boolean}
   */

	}, {
		key: 'hasReturnExpressionBeenCalledAtPath',
		value: function hasReturnExpressionBeenCalledAtPath(path$$1, callExpression) {
			return this._optionValues.getIn([OPTION_RETURN_EXPRESSIONS_CALLED_AT_PATH, callExpression].concat(_toConsumableArray$1(path$$1), [RESULT_KEY]));
		}

		/**
   * @return {boolean}
   */

	}, {
		key: 'ignoreBreakStatements',
		value: function ignoreBreakStatements() {
			return this.get(OPTION_IGNORE_BREAK_STATEMENTS);
		}

		/**
   * @param {string} labelName
   * @return {boolean}
   */

	}, {
		key: 'ignoreLabel',
		value: function ignoreLabel(labelName) {
			return this._optionValues.getIn([OPTION_IGNORED_LABELS, labelName]);
		}

		/**
   * @return {boolean}
   */

	}, {
		key: 'ignoreReturnAwaitYield',
		value: function ignoreReturnAwaitYield() {
			return this.get(OPTION_IGNORE_RETURN_AWAIT_YIELD);
		}

		/**
   * @param {ThisVariable|ParameterVariable} variable
   * @param {Node} init
   * @return {ExecutionPathOptions}
   */

	}, {
		key: 'replaceVariableInit',
		value: function replaceVariableInit(variable, init) {
			return this.setIn([OPTION_REPLACED_VARIABLE_INITS, variable], init);
		}

		/**
   * @param {ParameterVariable[]} variables
   * @return {ExecutionPathOptions}
   */

	}, {
		key: 'setArgumentsVariables',
		value: function setArgumentsVariables(variables) {
			return this.set(OPTION_ARGUMENTS_VARIABLES, variables);
		}

		/**
   * @param {boolean} [value=true]
   * @return {ExecutionPathOptions}
   */

	}, {
		key: 'setIgnoreBreakStatements',
		value: function setIgnoreBreakStatements() {
			var value = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : true;

			return this.set(OPTION_IGNORE_BREAK_STATEMENTS, value);
		}

		/**
   * @param {string} labelName
   * @return {ExecutionPathOptions}
   */

	}, {
		key: 'setIgnoreLabel',
		value: function setIgnoreLabel(labelName) {
			return this.setIn([OPTION_IGNORED_LABELS, labelName], true);
		}

		/**
   * @return {ExecutionPathOptions}
   */

	}, {
		key: 'setIgnoreNoLabels',
		value: function setIgnoreNoLabels() {
			return this.remove(OPTION_IGNORED_LABELS);
		}

		/**
   * @param {boolean} [value=true]
   * @return {ExecutionPathOptions}
   */

	}, {
		key: 'setIgnoreReturnAwaitYield',
		value: function setIgnoreReturnAwaitYield() {
			var value = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : true;

			return this.set(OPTION_IGNORE_RETURN_AWAIT_YIELD, value);
		}
	}]);

	return ExecutionPathOptions;
}();

var _createClass$5 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$5(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/* eslint-disable no-unused-vars */

var Node = function () {
	function Node() {
		_classCallCheck$5(this, Node);

		this.keys = [];
	}

	/**
  * Called once all nodes have been initialised and the scopes have been populated.
  * Usually one should not override this function but override bindNode and/or
  * bindChildren instead.
  */


	_createClass$5(Node, [{
		key: 'bind',
		value: function bind() {
			this.bindChildren();
			this.bindNode();
		}

		/**
   * Reassign a given path of an object.
   * E.g., node.reassignPath(['x', 'y']) is called when something
   * is assigned to node.x.y.
   * The default noop implementation is ok as long as hasEffectsWhenAssignedAtPath
   * always returns true for this node. Otherwise it should be overridden.
   * @param {String[]} path
   * @param {ExecutionPathOptions} options
   */

	}, {
		key: 'reassignPath',
		value: function reassignPath(path$$1, options) {}

		/**
   * Override to control on which children "bind" is called.
   */

	}, {
		key: 'bindChildren',
		value: function bindChildren() {
			this.eachChild(function (child) {
				return child.bind();
			});
		}

		/**
   * Override this to bind assignments to variables and do any initialisations that
   * require the scopes to be populated with variables.
   */

	}, {
		key: 'bindNode',
		value: function bindNode() {}
	}, {
		key: 'eachChild',
		value: function eachChild(callback) {
			var _this = this;

			this.keys.forEach(function (key) {
				var value = _this[key];
				if (!value) return;

				if (Array.isArray(value)) {
					value.forEach(function (child) {
						return child && callback(child);
					});
				} else {
					callback(value);
				}
			});
		}

		/**
   * Executes the callback on each possible return expression when calling this node.
   * @param {String[]} path
   * @param {CallOptions} callOptions
   * @param {Function} callback
   * @param {ExecutionPathOptions} options
   */

	}, {
		key: 'forEachReturnExpressionWhenCalledAtPath',
		value: function forEachReturnExpressionWhenCalledAtPath(path$$1, callOptions, callback, options) {}
	}, {
		key: 'getValue',
		value: function getValue() {
			return UNKNOWN_VALUE;
		}

		/**
   * Determine if this Node would have an effect on the bundle.
   * This is usually true for already included nodes. Exceptions are e.g. break statements
   * which only have an effect if their surrounding loop or switch statement is included.
   * The options pass on information like this about the current execution path.
   * @param {ExecutionPathOptions} options
   * @return {boolean}
   */

	}, {
		key: 'hasEffects',
		value: function hasEffects(options) {
			return this.someChild(function (child) {
				return child.hasEffects(options);
			});
		}

		/**
   * @param {String[]} path
   * @param {ExecutionPathOptions} options
   * @return {boolean}
   */

	}, {
		key: 'hasEffectsWhenAccessedAtPath',
		value: function hasEffectsWhenAccessedAtPath(path$$1, options) {
			return path$$1.length > 0;
		}

		/**
   * @param {String[]} path
   * @param {ExecutionPathOptions} options
   * @return {boolean}
   */

	}, {
		key: 'hasEffectsWhenAssignedAtPath',
		value: function hasEffectsWhenAssignedAtPath(path$$1, options) {
			return true;
		}

		/**
   * @param {String[]} path
   * @param {CallOptions} callOptions
   * @param {ExecutionPathOptions} options
   * @return {boolean}
   */

	}, {
		key: 'hasEffectsWhenCalledAtPath',
		value: function hasEffectsWhenCalledAtPath(path$$1, callOptions, options) {
			return true;
		}

		/**
   * Returns true if this node or any of its children is included.
   * @return {boolean}
   */

	}, {
		key: 'hasIncludedChild',
		value: function hasIncludedChild() {
			return this.included || this.someChild(function (child) {
				return child.hasIncludedChild();
			});
		}

		/**
   * Includes the node in the bundle. Children are usually included if they are
   * necessary for this node (e.g. a function body) or if they have effects.
   * Necessary variables need to be included as well. Should return true if any
   * nodes or variables have been added that were missing before.
   * @return {boolean}
   */

	}, {
		key: 'includeInBundle',
		value: function includeInBundle() {
			var addedNewNodes = !this.included;
			this.included = true;
			this.eachChild(function (childNode) {
				if (childNode.includeInBundle()) {
					addedNewNodes = true;
				}
			});
			return addedNewNodes;
		}

		/**
   * Alternative version of includeInBundle to override the default behaviour of
   * declarations to only include nodes for declarators that have an effect. Necessary
   * for for-loops that do not use a declared loop variable.
   * @return {boolean}
   */

	}, {
		key: 'includeWithAllDeclarations',
		value: function includeWithAllDeclarations() {
			return this.includeInBundle();
		}

		/**
   * Assign a scope to this node and make sure all children have the right scopes.
   * Perform any additional initialisation that does not depend on the scope being
   * populated with variables.
   * Usually one should not override this function but override initialiseScope,
   * initialiseNode and/or initialiseChildren instead. BlockScopes have a special
   * alternative initialisation initialiseAndReplaceScope.
   * @param {Scope} parentScope
   */

	}, {
		key: 'initialise',
		value: function initialise(parentScope) {
			this.initialiseScope(parentScope);
			this.initialiseNode(parentScope);
			this.initialiseChildren(parentScope);
		}

		/**
   * Override to change how and with what scopes children are initialised
   * @param {Scope} parentScope
   */

	}, {
		key: 'initialiseChildren',
		value: function initialiseChildren(parentScope) {
			var _this2 = this;

			this.eachChild(function (child) {
				return child.initialise(_this2.scope);
			});
		}

		/**
   * Override to perform special initialisation steps after the scope is initialised
   * @param {Scope} parentScope
   */

	}, {
		key: 'initialiseNode',
		value: function initialiseNode(parentScope) {}

		/**
   * Override if this scope should receive a different scope than the parent scope.
   * @param {Scope} parentScope
   */

	}, {
		key: 'initialiseScope',
		value: function initialiseScope(parentScope) {
			this.scope = parentScope;
		}
	}, {
		key: 'insertSemicolon',
		value: function insertSemicolon(code) {
			if (code.original[this.end - 1] !== ';') {
				code.appendLeft(this.end, ';');
			}
		}
	}, {
		key: 'locate',
		value: function locate$$1() {
			// useful for debugging
			var location = locateCharacter.locate(this.module.code, this.start, { offsetLine: 1 });
			location.file = this.module.id;
			location.toString = function () {
				return JSON.stringify(location);
			};

			return location;
		}
	}, {
		key: 'render',
		value: function render(code, es) {
			this.eachChild(function (child) {
				return child.render(code, es);
			});
		}

		/**
   * Start a new execution path to determine if this node has an effect on the bundle and
   * should therefore be included. Included nodes should always be included again in subsequent
   * visits as the inclusion of additional variables may require the inclusion of more child
   * nodes in e.g. block statements.
   * @return {boolean}
   */

	}, {
		key: 'shouldBeIncluded',
		value: function shouldBeIncluded() {
			return this.included || this.hasEffects(ExecutionPathOptions.create()) || this.hasIncludedChild();
		}
	}, {
		key: 'someChild',
		value: function someChild(callback) {
			var _this3 = this;

			return this.keys.some(function (key) {
				var value = _this3[key];
				if (!value) return false;

				if (Array.isArray(value)) {
					return value.some(function (child) {
						return child && callback(child);
					});
				}
				return callback(value);
			});
		}

		/**
   * Returns true if some possible return expression when called at the given
   * path returns true. predicateFunction receives a `node` as parameter.
   * @param {String[]} path
   * @param {CallOptions} callOptions
   * @param {Function} predicateFunction
   * @param {ExecutionPathOptions} options
   * @returns {boolean}
   */

	}, {
		key: 'someReturnExpressionWhenCalledAtPath',
		value: function someReturnExpressionWhenCalledAtPath(path$$1, callOptions, predicateFunction, options) {
			return predicateFunction(options)(UNKNOWN_ASSIGNMENT);
		}
	}, {
		key: 'toString',
		value: function toString() {
			return this.module.code.slice(this.start, this.end);
		}
	}]);

	return Node;
}();

var _createClass$4 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$4(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$1(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$1(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ArrayExpression = function (_Node) {
	_inherits$1(ArrayExpression, _Node);

	function ArrayExpression() {
		_classCallCheck$4(this, ArrayExpression);

		return _possibleConstructorReturn$1(this, (ArrayExpression.__proto__ || Object.getPrototypeOf(ArrayExpression)).apply(this, arguments));
	}

	_createClass$4(ArrayExpression, [{
		key: 'hasEffectsWhenAccessedAtPath',
		value: function hasEffectsWhenAccessedAtPath(path$$1) {
			return path$$1.length > 1;
		}
	}]);

	return ArrayExpression;
}(Node);

var _createClass$7 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$7(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$2(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$2(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ArrayPattern = function (_Node) {
	_inherits$2(ArrayPattern, _Node);

	function ArrayPattern() {
		_classCallCheck$7(this, ArrayPattern);

		return _possibleConstructorReturn$2(this, (ArrayPattern.__proto__ || Object.getPrototypeOf(ArrayPattern)).apply(this, arguments));
	}

	_createClass$7(ArrayPattern, [{
		key: 'reassignPath',
		value: function reassignPath(path$$1, options) {
			path$$1.length === 0 && this.eachChild(function (child) {
				return child.reassignPath([], options);
			});
		}
	}, {
		key: 'hasEffectsWhenAssignedAtPath',
		value: function hasEffectsWhenAssignedAtPath(path$$1, options) {
			return path$$1.length > 0 || this.someChild(function (child) {
				return child.hasEffectsWhenAssignedAtPath([], options);
			});
		}
	}, {
		key: 'initialiseAndDeclare',
		value: function initialiseAndDeclare(parentScope, kind) {
			this.initialiseScope(parentScope);
			this.eachChild(function (child) {
				return child.initialiseAndDeclare(parentScope, kind, UNKNOWN_ASSIGNMENT);
			});
		}
	}]);

	return ArrayPattern;
}(Node);

var _createClass$11 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _toArray(arr) { return Array.isArray(arr) ? arr : Array.from(arr); }

function _classCallCheck$11(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var UNKNOWN_KEY = { type: 'UNKNOWN_KEY' };

var ReassignedPathTracker = function () {
	function ReassignedPathTracker() {
		_classCallCheck$11(this, ReassignedPathTracker);

		this._reassigned = false;
		this._unknownReassignedSubPath = false;
		this._subPaths = new Map();
	}

	_createClass$11(ReassignedPathTracker, [{
		key: 'isReassigned',
		value: function isReassigned(path$$1) {
			if (path$$1.length === 0) {
				return this._reassigned;
			}

			var _path = _toArray(path$$1),
			    subPath = _path[0],
			    remainingPath = _path.slice(1);

			return this._unknownReassignedSubPath || this._subPaths.has(subPath) && this._subPaths.get(subPath).isReassigned(remainingPath);
		}
	}, {
		key: 'reassignPath',
		value: function reassignPath(path$$1) {
			if (this._reassigned) return;
			if (path$$1.length === 0) {
				this._reassigned = true;
			} else {
				this._reassignSubPath(path$$1);
			}
		}
	}, {
		key: '_reassignSubPath',
		value: function _reassignSubPath(path$$1) {
			if (this._unknownReassignedSubPath) return;

			var _path2 = _toArray(path$$1),
			    subPath = _path2[0],
			    remainingPath = _path2.slice(1);

			if (subPath === UNKNOWN_KEY) {
				this._unknownReassignedSubPath = true;
			} else {
				if (!this._subPaths.has(subPath)) {
					this._subPaths.set(subPath, new ReassignedPathTracker());
				}
				this._subPaths.get(subPath).reassignPath(remainingPath);
			}
		}
	}, {
		key: 'someReassignedPath',
		value: function someReassignedPath(path$$1, callback) {
			return this._reassigned ? callback(path$$1, UNKNOWN_ASSIGNMENT) : path$$1.length >= 1 && this._onSubPathIfReassigned(path$$1, callback);
		}
	}, {
		key: '_onSubPathIfReassigned',
		value: function _onSubPathIfReassigned(path$$1, callback) {
			var _path3 = _toArray(path$$1),
			    subPath = _path3[0],
			    remainingPath = _path3.slice(1);

			return this._unknownReassignedSubPath || subPath === UNKNOWN_KEY ? callback(remainingPath, UNKNOWN_ASSIGNMENT) : this._subPaths.has(subPath) && this._subPaths.get(subPath).someReassignedPath(remainingPath, callback);
		}
	}]);

	return ReassignedPathTracker;
}();

var VariableReassignmentTracker = function () {
	function VariableReassignmentTracker(initialExpression) {
		_classCallCheck$11(this, VariableReassignmentTracker);

		this._initialExpression = initialExpression;
		this._reassignedPathTracker = new ReassignedPathTracker();
	}

	_createClass$11(VariableReassignmentTracker, [{
		key: 'reassignPath',
		value: function reassignPath(path$$1, options) {
			if (path$$1.length > 0) {
				this._initialExpression && this._initialExpression.reassignPath(path$$1, options);
			}
			this._reassignedPathTracker.reassignPath(path$$1, options);
		}
	}, {
		key: 'forEachAtPath',
		value: function forEachAtPath(path$$1, callback) {
			this._initialExpression && callback(path$$1, this._initialExpression);
		}
	}, {
		key: 'someAtPath',
		value: function someAtPath(path$$1, predicateFunction) {
			return this._reassignedPathTracker.someReassignedPath(path$$1, predicateFunction) || this._initialExpression && predicateFunction(path$$1, this._initialExpression);
		}
	}]);

	return VariableReassignmentTracker;
}();

var _createClass$10 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$1 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$10(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$4(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$4(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

// To avoid infinite recursions
var MAX_PATH_DEPTH = 7;

var LocalVariable = function (_Variable) {
	_inherits$4(LocalVariable, _Variable);

	function LocalVariable(name, declarator, init) {
		_classCallCheck$10(this, LocalVariable);

		var _this = _possibleConstructorReturn$4(this, (LocalVariable.__proto__ || Object.getPrototypeOf(LocalVariable)).call(this, name));

		_this.isReassigned = false;
		_this.exportName = null;
		_this.declarations = new Set(declarator ? [declarator] : null);
		_this.boundExpressions = new VariableReassignmentTracker(init);
		return _this;
	}

	_createClass$10(LocalVariable, [{
		key: 'addDeclaration',
		value: function addDeclaration(identifier) {
			this.declarations.add(identifier);
		}
	}, {
		key: 'reassignPath',
		value: function reassignPath(path$$1, options) {
			if (path$$1.length > MAX_PATH_DEPTH) return;
			if (path$$1.length === 0) {
				this.isReassigned = true;
			}
			if (!options.hasNodeBeenAssignedAtPath(path$$1, this)) {
				this.boundExpressions.reassignPath(path$$1, options.addAssignedNodeAtPath(path$$1, this));
			}
		}
	}, {
		key: 'forEachReturnExpressionWhenCalledAtPath',
		value: function forEachReturnExpressionWhenCalledAtPath(path$$1, callOptions, callback, options) {
			if (path$$1.length > MAX_PATH_DEPTH) return;
			this.boundExpressions.forEachAtPath(path$$1, function (relativePath, node) {
				return !options.hasNodeBeenCalledAtPathWithOptions(relativePath, node, callOptions) && node.forEachReturnExpressionWhenCalledAtPath(relativePath, callOptions, callback, options.addCalledNodeAtPathWithOptions(relativePath, node, callOptions));
			});
		}
	}, {
		key: 'getName',
		value: function getName(es) {
			if (es) return this.name;
			if (!this.isReassigned || !this.exportName) return this.name;

			return 'exports.' + this.exportName;
		}
	}, {
		key: 'hasEffectsWhenAccessedAtPath',
		value: function hasEffectsWhenAccessedAtPath(path$$1, options) {
			return path$$1.length > MAX_PATH_DEPTH || this.boundExpressions.someAtPath(path$$1, function (relativePath, node) {
				return relativePath.length > 0 && !options.hasNodeBeenAccessedAtPath(relativePath, node) && node.hasEffectsWhenAccessedAtPath(relativePath, options.addAccessedNodeAtPath(relativePath, node));
			});
		}
	}, {
		key: 'hasEffectsWhenAssignedAtPath',
		value: function hasEffectsWhenAssignedAtPath(path$$1, options) {
			return this.included || path$$1.length > MAX_PATH_DEPTH || this.boundExpressions.someAtPath(path$$1, function (relativePath, node) {
				return relativePath.length > 0 && !options.hasNodeBeenAssignedAtPath(relativePath, node) && node.hasEffectsWhenAssignedAtPath(relativePath, options.addAssignedNodeAtPath(relativePath, node));
			});
		}
	}, {
		key: 'hasEffectsWhenCalledAtPath',
		value: function hasEffectsWhenCalledAtPath(path$$1, callOptions, options) {
			return path$$1.length > MAX_PATH_DEPTH || this.included && path$$1.length > 0 || this.boundExpressions.someAtPath(path$$1, function (relativePath, node) {
				return !options.hasNodeBeenCalledAtPathWithOptions(relativePath, node, callOptions) && node.hasEffectsWhenCalledAtPath(relativePath, callOptions, options.addCalledNodeAtPathWithOptions(relativePath, node, callOptions));
			});
		}
	}, {
		key: 'includeVariable',
		value: function includeVariable() {
			if (!_get$1(LocalVariable.prototype.__proto__ || Object.getPrototypeOf(LocalVariable.prototype), 'includeVariable', this).call(this)) return false;
			this.declarations.forEach(function (identifier) {
				return identifier.includeInBundle();
			});
			return true;
		}
	}, {
		key: 'someReturnExpressionWhenCalledAtPath',
		value: function someReturnExpressionWhenCalledAtPath(path$$1, callOptions, predicateFunction, options) {
			return path$$1.length > MAX_PATH_DEPTH || this.included && path$$1.length > 0 || this.boundExpressions.someAtPath(path$$1, function (relativePath, node) {
				return !options.hasNodeBeenCalledAtPathWithOptions(relativePath, node, callOptions) && node.someReturnExpressionWhenCalledAtPath(relativePath, callOptions, predicateFunction, options.addCalledNodeAtPathWithOptions(relativePath, node, callOptions));
			});
		}
	}, {
		key: 'toString',
		value: function toString() {
			return this.name;
		}
	}]);

	return LocalVariable;
}(Variable);

var _createClass$12 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$2 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$12(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$5(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$5(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ExportDefaultVariable = function (_LocalVariable) {
	_inherits$5(ExportDefaultVariable, _LocalVariable);

	function ExportDefaultVariable(name, exportDefaultDeclaration) {
		_classCallCheck$12(this, ExportDefaultVariable);

		var _this = _possibleConstructorReturn$5(this, (ExportDefaultVariable.__proto__ || Object.getPrototypeOf(ExportDefaultVariable)).call(this, name, exportDefaultDeclaration, exportDefaultDeclaration.declaration));

		_this.isDefault = true;
		_this.hasId = !!exportDefaultDeclaration.declaration.id;
		return _this;
	}

	_createClass$12(ExportDefaultVariable, [{
		key: 'addReference',
		value: function addReference(identifier) {
			this.name = identifier.name;
			if (this._original) {
				this._original.addReference(identifier);
			}
		}
	}, {
		key: 'getName',
		value: function getName(es) {
			if (this._original && !this._original.isReassigned) {
				return this._original.getName(es);
			}
			return this.name;
		}
	}, {
		key: 'getOriginalVariableName',
		value: function getOriginalVariableName(es) {
			return this._original && this._original.getName(es);
		}
	}, {
		key: 'includeVariable',
		value: function includeVariable() {
			if (!_get$2(ExportDefaultVariable.prototype.__proto__ || Object.getPrototypeOf(ExportDefaultVariable.prototype), 'includeVariable', this).call(this)) {
				return false;
			}
			this.declarations.forEach(function (declaration) {
				return declaration.includeDefaultExport();
			});
			return true;
		}
	}, {
		key: 'setOriginalVariable',
		value: function setOriginalVariable(original) {
			this._original = original;
		}
	}]);

	return ExportDefaultVariable;
}(LocalVariable);

var _createClass$9 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$9(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Scope = function () {
	function Scope() {
		var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

		_classCallCheck$9(this, Scope);

		this.parent = options.parent;
		this.isModuleScope = !!options.isModuleScope;

		this.children = [];
		if (this.parent) this.parent.children.push(this);

		this.variables = blank();
	}

	/**
  * @param identifier
  * @param {Object} [options] - valid options are
  *        {(Node|null)} init
  *        {boolean} isHoisted
  * @return {Variable}
  */


	_createClass$9(Scope, [{
		key: 'addDeclaration',
		value: function addDeclaration(identifier) {
			var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

			var name = identifier.name;
			if (this.variables[name]) {
				var variable = this.variables[name];
				variable.addDeclaration(identifier);
				variable.reassignPath([], ExecutionPathOptions.create());
			} else {
				this.variables[name] = new LocalVariable(identifier.name, identifier, options.init || UNDEFINED_ASSIGNMENT);
			}
			return this.variables[name];
		}
	}, {
		key: 'addExportDefaultDeclaration',
		value: function addExportDefaultDeclaration(name, exportDefaultDeclaration) {
			this.variables.default = new ExportDefaultVariable(name, exportDefaultDeclaration);
			return this.variables.default;
		}
	}, {
		key: 'addReturnExpression',
		value: function addReturnExpression(expression) {
			this.parent && this.parent.addReturnExpression(expression);
		}
	}, {
		key: 'contains',
		value: function contains(name) {
			return !!this.variables[name] || (this.parent ? this.parent.contains(name) : false);
		}
	}, {
		key: 'deshadow',
		value: function deshadow(names) {
			var _this = this;

			keys(this.variables).forEach(function (key) {
				var declaration = _this.variables[key];

				// we can disregard exports.foo etc
				if (declaration.exportName && declaration.isReassigned) return;

				var name = declaration.getName(true);
				var deshadowed = name;

				var i = 1;

				while (names.has(deshadowed)) {
					deshadowed = name + '$$' + i++;
				}

				declaration.name = deshadowed;
			});

			this.children.forEach(function (scope) {
				return scope.deshadow(names);
			});
		}
	}, {
		key: 'findLexicalBoundary',
		value: function findLexicalBoundary() {
			return this.parent.findLexicalBoundary();
		}
	}, {
		key: 'findVariable',
		value: function findVariable(name) {
			return this.variables[name] || this.parent && this.parent.findVariable(name);
		}
	}]);

	return Scope;
}();

var _createClass$16 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$3 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$16(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$9(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$9(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ReplaceableInitializationVariable = function (_LocalVariable) {
	_inherits$9(ReplaceableInitializationVariable, _LocalVariable);

	function ReplaceableInitializationVariable(name, declarator) {
		_classCallCheck$16(this, ReplaceableInitializationVariable);

		return _possibleConstructorReturn$9(this, (ReplaceableInitializationVariable.__proto__ || Object.getPrototypeOf(ReplaceableInitializationVariable)).call(this, name, declarator, null));
	}

	_createClass$16(ReplaceableInitializationVariable, [{
		key: 'getName',
		value: function getName() {
			return this.name;
		}
	}, {
		key: 'hasEffectsWhenAccessedAtPath',
		value: function hasEffectsWhenAccessedAtPath(path$$1, options) {
			return this._getInit(options).hasEffectsWhenAccessedAtPath(path$$1, options) || _get$3(ReplaceableInitializationVariable.prototype.__proto__ || Object.getPrototypeOf(ReplaceableInitializationVariable.prototype), 'hasEffectsWhenAccessedAtPath', this).call(this, path$$1, options);
		}
	}, {
		key: 'hasEffectsWhenAssignedAtPath',
		value: function hasEffectsWhenAssignedAtPath(path$$1, options) {
			return this._getInit(options).hasEffectsWhenAssignedAtPath(path$$1, options) || _get$3(ReplaceableInitializationVariable.prototype.__proto__ || Object.getPrototypeOf(ReplaceableInitializationVariable.prototype), 'hasEffectsWhenAssignedAtPath', this).call(this, path$$1, options);
		}
	}, {
		key: 'hasEffectsWhenCalledAtPath',
		value: function hasEffectsWhenCalledAtPath(path$$1, callOptions, options) {
			return this._getInit(options).hasEffectsWhenCalledAtPath(path$$1, callOptions, options) || _get$3(ReplaceableInitializationVariable.prototype.__proto__ || Object.getPrototypeOf(ReplaceableInitializationVariable.prototype), 'hasEffectsWhenCalledAtPath', this).call(this, path$$1, callOptions, options);
		}
	}, {
		key: 'someReturnExpressionWhenCalledAtPath',
		value: function someReturnExpressionWhenCalledAtPath(path$$1, callOptions, predicateFunction, options) {
			return this._getInit(options).someReturnExpressionWhenCalledAtPath(path$$1, callOptions, predicateFunction, options) || _get$3(ReplaceableInitializationVariable.prototype.__proto__ || Object.getPrototypeOf(ReplaceableInitializationVariable.prototype), 'someReturnExpressionWhenCalledAtPath', this).call(this, path$$1, callOptions, predicateFunction, options);
		}
	}, {
		key: '_getInit',
		value: function _getInit(options) {
			return options.getReplacedVariableInit(this) || UNKNOWN_ASSIGNMENT;
		}
	}]);

	return ReplaceableInitializationVariable;
}(LocalVariable);

var _createClass$15 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$15(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$8(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$8(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ParameterVariable = function (_ReplaceableInitializ) {
	_inherits$8(ParameterVariable, _ReplaceableInitializ);

	function ParameterVariable(identifier) {
		_classCallCheck$15(this, ParameterVariable);

		return _possibleConstructorReturn$8(this, (ParameterVariable.__proto__ || Object.getPrototypeOf(ParameterVariable)).call(this, identifier.name, identifier, null));
	}

	_createClass$15(ParameterVariable, [{
		key: 'getName',
		value: function getName() {
			return this.name;
		}
	}]);

	return ParameterVariable;
}(ReplaceableInitializationVariable);

var _createClass$14 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$14(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$7(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$7(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ParameterScope = function (_Scope) {
	_inherits$7(ParameterScope, _Scope);

	function ParameterScope() {
		var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

		_classCallCheck$14(this, ParameterScope);

		var _this = _possibleConstructorReturn$7(this, (ParameterScope.__proto__ || Object.getPrototypeOf(ParameterScope)).call(this, options));

		_this._parameters = [];
		return _this;
	}

	/**
  * Adds a parameter to this scope. Parameters must be added in the correct
  * order, e.g. from left to right.
  * @param {Identifier} identifier
  * @returns {Variable}
  */


	_createClass$14(ParameterScope, [{
		key: 'addParameterDeclaration',
		value: function addParameterDeclaration(identifier) {
			var variable = new ParameterVariable(identifier);
			this.variables[identifier.name] = variable;
			this._parameters.push(variable);
			return variable;
		}
	}, {
		key: 'getParameterVariables',
		value: function getParameterVariables() {
			return this._parameters;
		}
	}]);

	return ParameterScope;
}(Scope);

var _createClass$13 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$13(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$6(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$6(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ReturnValueScope = function (_ParameterScope) {
	_inherits$6(ReturnValueScope, _ParameterScope);

	function ReturnValueScope() {
		var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

		_classCallCheck$13(this, ReturnValueScope);

		var _this = _possibleConstructorReturn$6(this, (ReturnValueScope.__proto__ || Object.getPrototypeOf(ReturnValueScope)).call(this, options));

		_this._returnExpressions = new Set();
		return _this;
	}

	_createClass$13(ReturnValueScope, [{
		key: 'addReturnExpression',
		value: function addReturnExpression(expression) {
			this._returnExpressions.add(expression);
		}
	}, {
		key: 'forEachReturnExpressionWhenCalled',
		value: function forEachReturnExpressionWhenCalled(callOptions, callback, options) {
			this._returnExpressions.forEach(callback(options));
		}
	}, {
		key: 'someReturnExpressionWhenCalled',
		value: function someReturnExpressionWhenCalled(callOptions, predicateFunction, options) {
			return Array.from(this._returnExpressions).some(predicateFunction(options));
		}
	}]);

	return ReturnValueScope;
}(ParameterScope);

var _createClass$8 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$8(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$3(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$3(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ArrowFunctionExpression = function (_Node) {
	_inherits$3(ArrowFunctionExpression, _Node);

	function ArrowFunctionExpression() {
		_classCallCheck$8(this, ArrowFunctionExpression);

		return _possibleConstructorReturn$3(this, (ArrowFunctionExpression.__proto__ || Object.getPrototypeOf(ArrowFunctionExpression)).apply(this, arguments));
	}

	_createClass$8(ArrowFunctionExpression, [{
		key: 'bindNode',
		value: function bindNode() {
			this.body.bindImplicitReturnExpressionToScope ? this.body.bindImplicitReturnExpressionToScope() : this.scope.addReturnExpression(this.body);
		}
	}, {
		key: 'forEachReturnExpressionWhenCalledAtPath',
		value: function forEachReturnExpressionWhenCalledAtPath(path$$1, callOptions, callback, options) {
			path$$1.length === 0 && this.scope.forEachReturnExpressionWhenCalled(callOptions, callback, options);
		}
	}, {
		key: 'hasEffects',
		value: function hasEffects() {
			return false;
		}
	}, {
		key: 'hasEffectsWhenAccessedAtPath',
		value: function hasEffectsWhenAccessedAtPath(path$$1) {
			return path$$1.length > 1;
		}
	}, {
		key: 'hasEffectsWhenAssignedAtPath',
		value: function hasEffectsWhenAssignedAtPath(path$$1) {
			return path$$1.length > 1;
		}
	}, {
		key: 'hasEffectsWhenCalledAtPath',
		value: function hasEffectsWhenCalledAtPath(path$$1, callOptions, options) {
			if (path$$1.length > 0) {
				return true;
			}
			return this.params.some(function (param) {
				return param.hasEffects(options);
			}) || this.body.hasEffects(options);
		}
	}, {
		key: 'initialiseChildren',
		value: function initialiseChildren() {
			var _this2 = this;

			this.params.forEach(function (param) {
				return param.initialiseAndDeclare(_this2.scope, 'parameter');
			});
			if (this.body.initialiseAndReplaceScope) {
				this.body.initialiseAndReplaceScope(new Scope({ parent: this.scope }));
			} else {
				this.body.initialise(this.scope);
			}
		}
	}, {
		key: 'initialiseScope',
		value: function initialiseScope(parentScope) {
			this.scope = new ReturnValueScope({ parent: parentScope });
		}
	}, {
		key: 'someReturnExpressionWhenCalledAtPath',
		value: function someReturnExpressionWhenCalledAtPath(path$$1, callOptions, predicateFunction, options) {
			return path$$1.length > 0 || this.scope.someReturnExpressionWhenCalled(callOptions, predicateFunction, options);
		}
	}]);

	return ArrowFunctionExpression;
}(Node);

// TODO tidy this up a bit (e.g. they can both use node.module.imports)
function disallowIllegalReassignment(scope, node) {
	if (node.type === 'MemberExpression' && node.object.type === 'Identifier') {
		var variable = scope.findVariable(node.object.name);
		if (variable.isNamespace) {
			node.module.error({
				code: 'ILLEGAL_NAMESPACE_REASSIGNMENT',
				message: 'Illegal reassignment to import \'' + node.object.name + '\''
			}, node.start);
		}
	} else if (node.type === 'Identifier') {
		if (node.module.imports[node.name] && !scope.contains(node.name)) {
			node.module.error({
				code: 'ILLEGAL_REASSIGNMENT',
				message: 'Illegal reassignment to import \'' + node.name + '\''
			}, node.start);
		}
	}
}

var _createClass$17 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$4 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$17(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$10(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$10(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var AssignmentExpression = function (_Node) {
	_inherits$10(AssignmentExpression, _Node);

	function AssignmentExpression() {
		_classCallCheck$17(this, AssignmentExpression);

		return _possibleConstructorReturn$10(this, (AssignmentExpression.__proto__ || Object.getPrototypeOf(AssignmentExpression)).apply(this, arguments));
	}

	_createClass$17(AssignmentExpression, [{
		key: 'bindNode',
		value: function bindNode() {
			disallowIllegalReassignment(this.scope, this.left);
			this.left.reassignPath([], ExecutionPathOptions.create());
		}
	}, {
		key: 'hasEffects',
		value: function hasEffects(options) {
			return _get$4(AssignmentExpression.prototype.__proto__ || Object.getPrototypeOf(AssignmentExpression.prototype), 'hasEffects', this).call(this, options) || this.left.hasEffectsWhenAssignedAtPath([], options);
		}
	}, {
		key: 'hasEffectsWhenAccessedAtPath',
		value: function hasEffectsWhenAccessedAtPath(path$$1, options) {
			return path$$1.length > 0 && this.right.hasEffectsWhenAccessedAtPath(path$$1, options);
		}
	}]);

	return AssignmentExpression;
}(Node);

var _createClass$18 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$18(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$11(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$11(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var AssignmentPattern = function (_Node) {
	_inherits$11(AssignmentPattern, _Node);

	function AssignmentPattern() {
		_classCallCheck$18(this, AssignmentPattern);

		return _possibleConstructorReturn$11(this, (AssignmentPattern.__proto__ || Object.getPrototypeOf(AssignmentPattern)).apply(this, arguments));
	}

	_createClass$18(AssignmentPattern, [{
		key: 'bindNode',
		value: function bindNode() {
			this.left.reassignPath([], ExecutionPathOptions.create());
		}
	}, {
		key: 'reassignPath',
		value: function reassignPath(path$$1, options) {
			path$$1.length === 0 && this.left.reassignPath(path$$1, options);
		}
	}, {
		key: 'hasEffectsWhenAssignedAtPath',
		value: function hasEffectsWhenAssignedAtPath(path$$1, options) {
			return path$$1.length > 0 || this.left.hasEffectsWhenAssignedAtPath([], options);
		}
	}, {
		key: 'initialiseAndDeclare',
		value: function initialiseAndDeclare(parentScope, kind, init) {
			this.initialiseScope(parentScope);
			this.right.initialise(parentScope);
			this.left.initialiseAndDeclare(parentScope, kind, init);
		}
	}]);

	return AssignmentPattern;
}(Node);

var _createClass$19 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$5 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$19(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$12(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$12(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var AwaitExpression = function (_Node) {
	_inherits$12(AwaitExpression, _Node);

	function AwaitExpression() {
		_classCallCheck$19(this, AwaitExpression);

		return _possibleConstructorReturn$12(this, (AwaitExpression.__proto__ || Object.getPrototypeOf(AwaitExpression)).apply(this, arguments));
	}

	_createClass$19(AwaitExpression, [{
		key: 'hasEffects',
		value: function hasEffects(options) {
			return _get$5(AwaitExpression.prototype.__proto__ || Object.getPrototypeOf(AwaitExpression.prototype), 'hasEffects', this).call(this, options) || !options.ignoreReturnAwaitYield();
		}
	}]);

	return AwaitExpression;
}(Node);

var _createClass$20 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$20(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$13(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$13(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var operators = {
	'==': function _(left, right) {
		return left == right;
	},
	'!=': function _(left, right) {
		return left != right;
	},
	'===': function _(left, right) {
		return left === right;
	},
	'!==': function _(left, right) {
		return left !== right;
	},
	'<': function _(left, right) {
		return left < right;
	},
	'<=': function _(left, right) {
		return left <= right;
	},
	'>': function _(left, right) {
		return left > right;
	},
	'>=': function _(left, right) {
		return left >= right;
	},
	'<<': function _(left, right) {
		return left << right;
	},
	'>>': function _(left, right) {
		return left >> right;
	},
	'>>>': function _(left, right) {
		return left >>> right;
	},
	'+': function _(left, right) {
		return left + right;
	},
	'-': function _(left, right) {
		return left - right;
	},
	'*': function _(left, right) {
		return left * right;
	},
	'/': function _(left, right) {
		return left / right;
	},
	'%': function _(left, right) {
		return left % right;
	},
	'|': function _(left, right) {
		return left | right;
	},
	'^': function _(left, right) {
		return left ^ right;
	},
	'&': function _(left, right) {
		return left & right;
	},
	'**': function _(left, right) {
		return Math.pow(left, right);
	},
	in: function _in(left, right) {
		return left in right;
	},
	instanceof: function _instanceof(left, right) {
		return left instanceof right;
	}
};

var BinaryExpression = function (_Node) {
	_inherits$13(BinaryExpression, _Node);

	function BinaryExpression() {
		_classCallCheck$20(this, BinaryExpression);

		return _possibleConstructorReturn$13(this, (BinaryExpression.__proto__ || Object.getPrototypeOf(BinaryExpression)).apply(this, arguments));
	}

	_createClass$20(BinaryExpression, [{
		key: 'getValue',
		value: function getValue() {
			var leftValue = this.left.getValue();
			if (leftValue === UNKNOWN_VALUE) return UNKNOWN_VALUE;

			var rightValue = this.right.getValue();
			if (rightValue === UNKNOWN_VALUE) return UNKNOWN_VALUE;

			if (!operators[this.operator]) return UNKNOWN_VALUE;

			return operators[this.operator](leftValue, rightValue);
		}
	}, {
		key: 'hasEffectsWhenAccessedAtPath',
		value: function hasEffectsWhenAccessedAtPath(path$$1) {
			return path$$1.length > 1;
		}
	}]);

	return BinaryExpression;
}(Node);

var _createClass$22 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$6 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$22(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$15(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$15(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var Statement = function (_Node) {
	_inherits$15(Statement, _Node);

	function Statement() {
		_classCallCheck$22(this, Statement);

		return _possibleConstructorReturn$15(this, (Statement.__proto__ || Object.getPrototypeOf(Statement)).apply(this, arguments));
	}

	_createClass$22(Statement, [{
		key: 'render',
		value: function render(code, es) {
			if (!this.module.bundle.treeshake || this.included) {
				_get$6(Statement.prototype.__proto__ || Object.getPrototypeOf(Statement.prototype), 'render', this).call(this, code, es);
			} else {
				code.remove(this.leadingCommentStart || this.start, this.next || this.end);
			}
		}
	}]);

	return Statement;
}(Node);

var _createClass$23 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$7 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$23(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$16(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$16(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var BlockScope = function (_Scope) {
	_inherits$16(BlockScope, _Scope);

	function BlockScope() {
		_classCallCheck$23(this, BlockScope);

		return _possibleConstructorReturn$16(this, (BlockScope.__proto__ || Object.getPrototypeOf(BlockScope)).apply(this, arguments));
	}

	_createClass$23(BlockScope, [{
		key: 'addDeclaration',
		value: function addDeclaration(identifier) {
			var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

			if (options.isHoisted) {
				return this.parent.addDeclaration(identifier, options);
			} else {
				return _get$7(BlockScope.prototype.__proto__ || Object.getPrototypeOf(BlockScope.prototype), 'addDeclaration', this).call(this, identifier, options);
			}
		}
	}]);

	return BlockScope;
}(Scope);

var _createClass$21 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$21(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$14(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$14(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var BlockStatement = function (_Statement) {
	_inherits$14(BlockStatement, _Statement);

	function BlockStatement() {
		_classCallCheck$21(this, BlockStatement);

		return _possibleConstructorReturn$14(this, (BlockStatement.__proto__ || Object.getPrototypeOf(BlockStatement)).apply(this, arguments));
	}

	_createClass$21(BlockStatement, [{
		key: 'bindImplicitReturnExpressionToScope',
		value: function bindImplicitReturnExpressionToScope() {
			var lastStatement = this.body[this.body.length - 1];
			if (!lastStatement || lastStatement.type !== 'ReturnStatement') {
				this.scope.addReturnExpression(UNDEFINED_ASSIGNMENT);
			}
		}
	}, {
		key: 'hasEffects',
		value: function hasEffects(options) {
			return this.body.some(function (child) {
				return child.hasEffects(options);
			});
		}
	}, {
		key: 'includeInBundle',
		value: function includeInBundle() {
			var addedNewNodes = !this.included;
			this.included = true;
			this.body.forEach(function (node) {
				if (node.shouldBeIncluded()) {
					if (node.includeInBundle()) {
						addedNewNodes = true;
					}
				}
			});
			return addedNewNodes;
		}
	}, {
		key: 'initialiseAndReplaceScope',
		value: function initialiseAndReplaceScope(scope) {
			this.scope = scope;
			this.initialiseNode();
			this.initialiseChildren(scope);
		}
	}, {
		key: 'initialiseChildren',
		value: function initialiseChildren() {
			var lastNode = void 0;
			var _iteratorNormalCompletion = true;
			var _didIteratorError = false;
			var _iteratorError = undefined;

			try {
				for (var _iterator = this.body[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
					var node = _step.value;

					node.initialise(this.scope);

					if (lastNode) lastNode.next = node.start;
					lastNode = node;
				}
			} catch (err) {
				_didIteratorError = true;
				_iteratorError = err;
			} finally {
				try {
					if (!_iteratorNormalCompletion && _iterator.return) {
						_iterator.return();
					}
				} finally {
					if (_didIteratorError) {
						throw _iteratorError;
					}
				}
			}
		}
	}, {
		key: 'initialiseScope',
		value: function initialiseScope(parentScope) {
			this.scope = new BlockScope({ parent: parentScope });
		}
	}, {
		key: 'render',
		value: function render(code, es) {
			if (this.body.length) {
				var _iteratorNormalCompletion2 = true;
				var _didIteratorError2 = false;
				var _iteratorError2 = undefined;

				try {
					for (var _iterator2 = this.body[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
						var node = _step2.value;

						node.render(code, es);
					}
				} catch (err) {
					_didIteratorError2 = true;
					_iteratorError2 = err;
				} finally {
					try {
						if (!_iteratorNormalCompletion2 && _iterator2.return) {
							_iterator2.return();
						}
					} finally {
						if (_didIteratorError2) {
							throw _iteratorError2;
						}
					}
				}
			} else {
				Statement.prototype.render.call(this, code, es);
			}
		}
	}]);

	return BlockStatement;
}(Statement);

var _createClass$24 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$8 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$24(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$17(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$17(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var BreakStatement = function (_Node) {
	_inherits$17(BreakStatement, _Node);

	function BreakStatement() {
		_classCallCheck$24(this, BreakStatement);

		return _possibleConstructorReturn$17(this, (BreakStatement.__proto__ || Object.getPrototypeOf(BreakStatement)).apply(this, arguments));
	}

	_createClass$24(BreakStatement, [{
		key: 'hasEffects',
		value: function hasEffects(options) {
			return _get$8(BreakStatement.prototype.__proto__ || Object.getPrototypeOf(BreakStatement.prototype), 'hasEffects', this).call(this, options) || !options.ignoreBreakStatements() || this.label && !options.ignoreLabel(this.label.name);
		}
	}]);

	return BreakStatement;
}(Node);

var _createClass$26 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$26(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var CallOptions = function () {
	_createClass$26(CallOptions, null, [{
		key: "create",
		value: function create(callOptions) {
			return new this(callOptions);
		}
	}]);

	function CallOptions() {
		var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
		    _ref$withNew = _ref.withNew,
		    withNew = _ref$withNew === undefined ? false : _ref$withNew,
		    _ref$args = _ref.args,
		    args = _ref$args === undefined ? [] : _ref$args,
		    caller = _ref.caller;

		_classCallCheck$26(this, CallOptions);

		this.withNew = withNew;
		this.args = args;
		this.caller = caller;
	}

	_createClass$26(CallOptions, [{
		key: "equals",
		value: function equals(callOptions) {
			return callOptions && this.caller === callOptions.caller;
		}
	}]);

	return CallOptions;
}();

var _createClass$25 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$25(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$18(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$18(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var CallExpression = function (_Node) {
	_inherits$18(CallExpression, _Node);

	function CallExpression() {
		_classCallCheck$25(this, CallExpression);

		return _possibleConstructorReturn$18(this, (CallExpression.__proto__ || Object.getPrototypeOf(CallExpression)).apply(this, arguments));
	}

	_createClass$25(CallExpression, [{
		key: 'reassignPath',
		value: function reassignPath(path$$1, options) {
			var _this2 = this;

			!options.hasReturnExpressionBeenAssignedAtPath(path$$1, this) && this.callee.forEachReturnExpressionWhenCalledAtPath([], this._callOptions, function (innerOptions) {
				return function (node) {
					return node.reassignPath(path$$1, innerOptions.addAssignedReturnExpressionAtPath(path$$1, _this2));
				};
			}, options);
		}
	}, {
		key: 'bindNode',
		value: function bindNode() {
			if (this.callee.type === 'Identifier') {
				var variable = this.scope.findVariable(this.callee.name);

				if (variable.isNamespace) {
					this.module.error({
						code: 'CANNOT_CALL_NAMESPACE',
						message: 'Cannot call a namespace (\'' + this.callee.name + '\')'
					}, this.start);
				}

				if (this.callee.name === 'eval' && variable.isGlobal) {
					this.module.warn({
						code: 'EVAL',
						message: 'Use of eval is strongly discouraged, as it poses security risks and may cause issues with minification',
						url: 'https://github.com/rollup/rollup/wiki/Troubleshooting#avoiding-eval'
					}, this.start);
				}
			}
		}
	}, {
		key: 'forEachReturnExpressionWhenCalledAtPath',
		value: function forEachReturnExpressionWhenCalledAtPath(path$$1, callOptions, callback, options) {
			this.callee.forEachReturnExpressionWhenCalledAtPath([], this._callOptions, function (innerOptions) {
				return function (node) {
					return node.forEachReturnExpressionWhenCalledAtPath(path$$1, callOptions, callback, innerOptions);
				};
			}, options);
		}
	}, {
		key: 'hasEffects',
		value: function hasEffects(options) {
			return this.arguments.some(function (child) {
				return child.hasEffects(options);
			}) || this.callee.hasEffectsWhenCalledAtPath([], this._callOptions, options.getHasEffectsWhenCalledOptions());
		}
	}, {
		key: 'hasEffectsWhenAccessedAtPath',
		value: function hasEffectsWhenAccessedAtPath(path$$1, options) {
			var _this3 = this;

			return path$$1.length > 0 && !options.hasReturnExpressionBeenAccessedAtPath(path$$1, this) && this.callee.someReturnExpressionWhenCalledAtPath([], this._callOptions, function (innerOptions) {
				return function (node) {
					return node.hasEffectsWhenAccessedAtPath(path$$1, innerOptions.addAccessedReturnExpressionAtPath(path$$1, _this3));
				};
			}, options);
		}
	}, {
		key: 'hasEffectsWhenAssignedAtPath',
		value: function hasEffectsWhenAssignedAtPath(path$$1, options) {
			var _this4 = this;

			return !options.hasReturnExpressionBeenAssignedAtPath(path$$1, this) && this.callee.someReturnExpressionWhenCalledAtPath([], this._callOptions, function (innerOptions) {
				return function (node) {
					return node.hasEffectsWhenAssignedAtPath(path$$1, innerOptions.addAssignedReturnExpressionAtPath(path$$1, _this4));
				};
			}, options);
		}
	}, {
		key: 'hasEffectsWhenCalledAtPath',
		value: function hasEffectsWhenCalledAtPath(path$$1, callOptions, options) {
			var _this5 = this;

			return !options.hasReturnExpressionBeenCalledAtPath(path$$1, this) && this.callee.someReturnExpressionWhenCalledAtPath([], this._callOptions, function (innerOptions) {
				return function (node) {
					return node.hasEffectsWhenCalledAtPath(path$$1, callOptions, innerOptions.addCalledReturnExpressionAtPath(path$$1, _this5));
				};
			}, options);
		}
	}, {
		key: 'initialiseNode',
		value: function initialiseNode() {
			this._callOptions = CallOptions.create({ withNew: false, args: this.arguments, caller: this });
		}
	}, {
		key: 'someReturnExpressionWhenCalledAtPath',
		value: function someReturnExpressionWhenCalledAtPath(path$$1, callOptions, predicateFunction, options) {
			return this.callee.someReturnExpressionWhenCalledAtPath([], this._callOptions, function (innerOptions) {
				return function (node) {
					return node.someReturnExpressionWhenCalledAtPath(path$$1, callOptions, predicateFunction, innerOptions);
				};
			}, options);
		}
	}]);

	return CallExpression;
}(Node);

var _createClass$28 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$9 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$28(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$20(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$20(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var CatchScope = function (_ParameterScope) {
	_inherits$20(CatchScope, _ParameterScope);

	function CatchScope() {
		_classCallCheck$28(this, CatchScope);

		return _possibleConstructorReturn$20(this, (CatchScope.__proto__ || Object.getPrototypeOf(CatchScope)).apply(this, arguments));
	}

	_createClass$28(CatchScope, [{
		key: 'addDeclaration',
		value: function addDeclaration(identifier) {
			var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

			if (options.isHoisted) {
				return this.parent.addDeclaration(identifier, options);
			} else {
				return _get$9(CatchScope.prototype.__proto__ || Object.getPrototypeOf(CatchScope.prototype), 'addDeclaration', this).call(this, identifier, options);
			}
		}
	}]);

	return CatchScope;
}(ParameterScope);

var _createClass$27 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$27(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$19(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$19(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var CatchClause = function (_Node) {
	_inherits$19(CatchClause, _Node);

	function CatchClause() {
		_classCallCheck$27(this, CatchClause);

		return _possibleConstructorReturn$19(this, (CatchClause.__proto__ || Object.getPrototypeOf(CatchClause)).apply(this, arguments));
	}

	_createClass$27(CatchClause, [{
		key: 'initialiseChildren',
		value: function initialiseChildren() {
			this.param && this.param.initialiseAndDeclare(this.scope, 'parameter');
			this.body.initialiseAndReplaceScope(this.scope);
		}
	}, {
		key: 'initialiseScope',
		value: function initialiseScope(parentScope) {
			this.scope = new CatchScope({ parent: parentScope });
		}
	}]);

	return CatchClause;
}(Node);

var _createClass$29 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$29(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$21(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$21(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ClassBody = function (_Node) {
	_inherits$21(ClassBody, _Node);

	function ClassBody() {
		_classCallCheck$29(this, ClassBody);

		return _possibleConstructorReturn$21(this, (ClassBody.__proto__ || Object.getPrototypeOf(ClassBody)).apply(this, arguments));
	}

	_createClass$29(ClassBody, [{
		key: 'hasEffectsWhenCalledAtPath',
		value: function hasEffectsWhenCalledAtPath(path$$1, callOptions, options) {
			if (path$$1.length > 0) {
				return true;
			}
			return this.classConstructor && this.classConstructor.hasEffectsWhenCalledAtPath([], callOptions, options);
		}
	}, {
		key: 'initialiseNode',
		value: function initialiseNode() {
			this.classConstructor = this.body.find(function (method) {
				return method.kind === 'constructor';
			});
		}
	}]);

	return ClassBody;
}(Node);

var _createClass$31 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$31(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$23(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$23(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ClassNode = function (_Node) {
	_inherits$23(ClassNode, _Node);

	function ClassNode() {
		_classCallCheck$31(this, ClassNode);

		return _possibleConstructorReturn$23(this, (ClassNode.__proto__ || Object.getPrototypeOf(ClassNode)).apply(this, arguments));
	}

	_createClass$31(ClassNode, [{
		key: 'hasEffectsWhenAccessedAtPath',
		value: function hasEffectsWhenAccessedAtPath(path$$1) {
			return path$$1.length > 1;
		}
	}, {
		key: 'hasEffectsWhenAssignedAtPath',
		value: function hasEffectsWhenAssignedAtPath(path$$1) {
			return path$$1.length > 1;
		}
	}, {
		key: 'hasEffectsWhenCalledAtPath',
		value: function hasEffectsWhenCalledAtPath(path$$1, callOptions, options) {
			return this.body.hasEffectsWhenCalledAtPath(path$$1, callOptions, options) || this.superClass && this.superClass.hasEffectsWhenCalledAtPath(path$$1, callOptions, options);
		}
	}, {
		key: 'initialiseChildren',
		value: function initialiseChildren() {
			if (this.superClass) {
				this.superClass.initialise(this.scope);
			}
			this.body.initialise(this.scope);
		}
	}, {
		key: 'initialiseScope',
		value: function initialiseScope(parentScope) {
			this.scope = new Scope({ parent: parentScope });
		}
	}]);

	return ClassNode;
}(Node);

var _createClass$30 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$10 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$30(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$22(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$22(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ClassDeclaration = function (_ClassNode) {
	_inherits$22(ClassDeclaration, _ClassNode);

	function ClassDeclaration() {
		_classCallCheck$30(this, ClassDeclaration);

		return _possibleConstructorReturn$22(this, (ClassDeclaration.__proto__ || Object.getPrototypeOf(ClassDeclaration)).apply(this, arguments));
	}

	_createClass$30(ClassDeclaration, [{
		key: 'initialiseChildren',
		value: function initialiseChildren(parentScope) {
			// Class declarations are like let declarations: Not hoisted, can be reassigned, cannot be redeclared
			this.id && this.id.initialiseAndDeclare(parentScope, 'class', this);
			_get$10(ClassDeclaration.prototype.__proto__ || Object.getPrototypeOf(ClassDeclaration.prototype), 'initialiseChildren', this).call(this, parentScope);
		}
	}, {
		key: 'render',
		value: function render(code, es) {
			if (!this.module.bundle.treeshake || this.included) {
				_get$10(ClassDeclaration.prototype.__proto__ || Object.getPrototypeOf(ClassDeclaration.prototype), 'render', this).call(this, code, es);
			} else {
				code.remove(this.leadingCommentStart || this.start, this.next || this.end);
			}
		}
	}]);

	return ClassDeclaration;
}(ClassNode);

var _createClass$32 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$11 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$32(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$24(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$24(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ClassExpression = function (_ClassNode) {
	_inherits$24(ClassExpression, _ClassNode);

	function ClassExpression() {
		_classCallCheck$32(this, ClassExpression);

		return _possibleConstructorReturn$24(this, (ClassExpression.__proto__ || Object.getPrototypeOf(ClassExpression)).apply(this, arguments));
	}

	_createClass$32(ClassExpression, [{
		key: 'initialiseChildren',
		value: function initialiseChildren(parentScope) {
			this.id && this.id.initialiseAndDeclare(this.scope, 'class', this);
			_get$11(ClassExpression.prototype.__proto__ || Object.getPrototypeOf(ClassExpression.prototype), 'initialiseChildren', this).call(this, parentScope);
		}
	}]);

	return ClassExpression;
}(ClassNode);

var _createClass$33 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$12 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$33(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$25(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$25(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ConditionalExpression = function (_Node) {
	_inherits$25(ConditionalExpression, _Node);

	function ConditionalExpression() {
		_classCallCheck$33(this, ConditionalExpression);

		return _possibleConstructorReturn$25(this, (ConditionalExpression.__proto__ || Object.getPrototypeOf(ConditionalExpression)).apply(this, arguments));
	}

	_createClass$33(ConditionalExpression, [{
		key: 'reassignPath',
		value: function reassignPath(path$$1, options) {
			path$$1.length > 0 && this._forEachRelevantBranch(function (node) {
				return node.reassignPath(path$$1, options);
			});
		}
	}, {
		key: 'forEachReturnExpressionWhenCalledAtPath',
		value: function forEachReturnExpressionWhenCalledAtPath(path$$1, callOptions, callback, options) {
			this._forEachRelevantBranch(function (node) {
				return node.forEachReturnExpressionWhenCalledAtPath(path$$1, callOptions, callback, options);
			});
		}
	}, {
		key: 'getValue',
		value: function getValue() {
			var testValue = this.test.getValue();
			if (testValue === UNKNOWN_VALUE) return UNKNOWN_VALUE;

			return testValue ? this.consequent.getValue() : this.alternate.getValue();
		}
	}, {
		key: 'hasEffects',
		value: function hasEffects(options) {
			return this.test.hasEffects(options) || this._someRelevantBranch(function (node) {
				return node.hasEffects(options);
			});
		}
	}, {
		key: 'hasEffectsWhenAccessedAtPath',
		value: function hasEffectsWhenAccessedAtPath(path$$1, options) {
			return path$$1.length > 0 && this._someRelevantBranch(function (node) {
				return node.hasEffectsWhenAccessedAtPath(path$$1, options);
			});
		}
	}, {
		key: 'hasEffectsWhenAssignedAtPath',
		value: function hasEffectsWhenAssignedAtPath(path$$1, options) {
			return path$$1.length === 0 || this._someRelevantBranch(function (node) {
				return node.hasEffectsWhenAssignedAtPath(path$$1, options);
			});
		}
	}, {
		key: 'hasEffectsWhenCalledAtPath',
		value: function hasEffectsWhenCalledAtPath(path$$1, callOptions, options) {
			return this._someRelevantBranch(function (node) {
				return node.hasEffectsWhenCalledAtPath(path$$1, callOptions, options);
			});
		}
	}, {
		key: 'initialiseChildren',
		value: function initialiseChildren(parentScope) {
			_get$12(ConditionalExpression.prototype.__proto__ || Object.getPrototypeOf(ConditionalExpression.prototype), 'initialiseChildren', this).call(this, parentScope);
			if (this.module.bundle.treeshake) {
				this.testValue = this.test.getValue();

				if (this.testValue === UNKNOWN_VALUE) {
					return;
				} else if (this.testValue) {
					this.alternate = null;
				} else if (this.alternate) {
					this.consequent = null;
				}
			}
		}
	}, {
		key: 'render',
		value: function render(code, es) {
			if (!this.module.bundle.treeshake) {
				_get$12(ConditionalExpression.prototype.__proto__ || Object.getPrototypeOf(ConditionalExpression.prototype), 'render', this).call(this, code, es);
			} else {
				if (this.testValue === UNKNOWN_VALUE) {
					_get$12(ConditionalExpression.prototype.__proto__ || Object.getPrototypeOf(ConditionalExpression.prototype), 'render', this).call(this, code, es);
				} else {
					var branchToRetain = this.testValue ? this.consequent : this.alternate;

					code.remove(this.start, branchToRetain.start);
					code.remove(branchToRetain.end, this.end);
					if (branchToRetain.type === 'SequenceExpression') {
						code.prependLeft(branchToRetain.start, '(');
						code.appendRight(branchToRetain.end, ')');
					}
					branchToRetain.render(code, es);
				}
			}
		}
	}, {
		key: 'someReturnExpressionWhenCalledAtPath',
		value: function someReturnExpressionWhenCalledAtPath(path$$1, callOptions, predicateFunction, options) {
			return this._someRelevantBranch(function (node) {
				return node.someReturnExpressionWhenCalledAtPath(path$$1, callOptions, predicateFunction, options);
			});
		}
	}, {
		key: '_forEachRelevantBranch',
		value: function _forEachRelevantBranch(callback) {
			if (this.testValue === UNKNOWN_VALUE) {
				callback(this.consequent);
				callback(this.alternate);
			} else {
				this.testValue ? callback(this.consequent) : callback(this.alternate);
			}
		}
	}, {
		key: '_someRelevantBranch',
		value: function _someRelevantBranch(predicateFunction) {
			return this.testValue === UNKNOWN_VALUE ? predicateFunction(this.consequent) || predicateFunction(this.alternate) : this.testValue ? predicateFunction(this.consequent) : predicateFunction(this.alternate);
		}
	}]);

	return ConditionalExpression;
}(Node);

var _createClass$34 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$34(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$26(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$26(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var DoWhileStatement = function (_Statement) {
	_inherits$26(DoWhileStatement, _Statement);

	function DoWhileStatement() {
		_classCallCheck$34(this, DoWhileStatement);

		return _possibleConstructorReturn$26(this, (DoWhileStatement.__proto__ || Object.getPrototypeOf(DoWhileStatement)).apply(this, arguments));
	}

	_createClass$34(DoWhileStatement, [{
		key: 'hasEffects',
		value: function hasEffects(options) {
			return this.test.hasEffects(options) || this.body.hasEffects(options.setIgnoreBreakStatements());
		}
	}]);

	return DoWhileStatement;
}(Statement);

var _createClass$35 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$35(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$27(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$27(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var EmptyStatement = function (_Statement) {
	_inherits$27(EmptyStatement, _Statement);

	function EmptyStatement() {
		_classCallCheck$35(this, EmptyStatement);

		return _possibleConstructorReturn$27(this, (EmptyStatement.__proto__ || Object.getPrototypeOf(EmptyStatement)).apply(this, arguments));
	}

	_createClass$35(EmptyStatement, [{
		key: 'render',
		value: function render(code) {
			if (this.parent.type === 'BlockStatement' || this.parent.type === 'Program') {
				code.remove(this.start, this.end);
			}
		}
	}]);

	return EmptyStatement;
}(Statement);

var _createClass$36 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$36(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$28(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$28(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ExportAllDeclaration = function (_Node) {
	_inherits$28(ExportAllDeclaration, _Node);

	function ExportAllDeclaration() {
		_classCallCheck$36(this, ExportAllDeclaration);

		return _possibleConstructorReturn$28(this, (ExportAllDeclaration.__proto__ || Object.getPrototypeOf(ExportAllDeclaration)).apply(this, arguments));
	}

	_createClass$36(ExportAllDeclaration, [{
		key: 'initialiseNode',
		value: function initialiseNode() {
			this.isExportDeclaration = true;
		}
	}, {
		key: 'render',
		value: function render(code) {
			code.remove(this.leadingCommentStart || this.start, this.next || this.end);
		}
	}]);

	return ExportAllDeclaration;
}(Node);

var _createClass$37 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$13 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$37(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$29(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$29(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var functionOrClassDeclaration = /^(?:Function|Class)Declaration/;

function buildRegexWithSpaces(re) {
	var spaceOrComment = "(?:" + [/\s/.source, // Space
	/\/\/.*[\n\r]/.source, // Single line comment
	/\/\*[^]*?\*\//.source].join("|") + ")";
	return new RegExp(re.source.replace(/\s|\\s/g, spaceOrComment), re.flags);
}

var sourceRE = {
	exportDefault: buildRegexWithSpaces(/^ *export +default */),
	declarationHeader: buildRegexWithSpaces(/^ *export +default +(?:(?:async +)?function(?: *\*)?|class)/)
};

var ExportDefaultDeclaration = function (_Node) {
	_inherits$29(ExportDefaultDeclaration, _Node);

	function ExportDefaultDeclaration() {
		_classCallCheck$37(this, ExportDefaultDeclaration);

		return _possibleConstructorReturn$29(this, (ExportDefaultDeclaration.__proto__ || Object.getPrototypeOf(ExportDefaultDeclaration)).apply(this, arguments));
	}

	_createClass$37(ExportDefaultDeclaration, [{
		key: 'bindNode',
		value: function bindNode() {
			if (this._declarationName) {
				this.variable.setOriginalVariable(this.scope.findVariable(this._declarationName));
			}
		}
	}, {
		key: 'includeDefaultExport',
		value: function includeDefaultExport() {
			this.included = true;
			this.declaration.includeInBundle();
		}
	}, {
		key: 'includeInBundle',
		value: function includeInBundle() {
			if (this.declaration.shouldBeIncluded()) {
				return this.declaration.includeInBundle();
			}
			return false;
		}
	}, {
		key: 'initialiseNode',
		value: function initialiseNode() {
			this.isExportDeclaration = true;
			this._declarationName = this.declaration.id && this.declaration.id.name || this.declaration.name;
			this.variable = this.scope.addExportDefaultDeclaration(this._declarationName || this.module.basename(), this);
		}
	}, {
		key: 'render',
		value: function render(code, es) {
			var _this2 = this;

			var remove = function remove() {
				code.remove(_this2.leadingCommentStart || _this2.start, _this2.next || _this2.end);
			};
			var removeExportDefault = function removeExportDefault() {
				code.remove(_this2.start, declaration_start);
			};

			var treeshakeable = this.module.bundle.treeshake && !this.included && !this.declaration.included;
			var name = this.variable.getName(es);
			var statementStr = code.original.slice(this.start, this.end);

			// paren workaround: find first non-whitespace character position after `export default`
			var declaration_start = this.start + statementStr.match(sourceRE.exportDefault)[0].length;

			if (functionOrClassDeclaration.test(this.declaration.type)) {
				if (treeshakeable) {
					return remove();
				}

				// Add the id to anonymous declarations
				if (!this.declaration.id) {
					var id_insertPos = this.start + statementStr.match(sourceRE.declarationHeader)[0].length;
					code.appendLeft(id_insertPos, ' ' + name);
				}

				removeExportDefault();
			} else {
				if (treeshakeable) {
					var hasEffects = this.declaration.hasEffects(ExecutionPathOptions.create());
					return hasEffects ? removeExportDefault() : remove();
				}

				// Prevent `var foo = foo`
				if (this.variable.getOriginalVariableName(es) === name) {
					return remove();
				}

				// Only output `var foo =` if `foo` is used
				if (this.included) {
					code.overwrite(this.start, declaration_start, this.module.bundle.varOrConst + ' ' + name + ' = ');
				} else {
					removeExportDefault();
				}
			}
			_get$13(ExportDefaultDeclaration.prototype.__proto__ || Object.getPrototypeOf(ExportDefaultDeclaration.prototype), 'render', this).call(this, code, es);
		}
	}]);

	return ExportDefaultDeclaration;
}(Node);

var _createClass$38 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$38(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$30(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$30(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ExportNamedDeclaration = function (_Node) {
	_inherits$30(ExportNamedDeclaration, _Node);

	function ExportNamedDeclaration() {
		_classCallCheck$38(this, ExportNamedDeclaration);

		return _possibleConstructorReturn$30(this, (ExportNamedDeclaration.__proto__ || Object.getPrototypeOf(ExportNamedDeclaration)).apply(this, arguments));
	}

	_createClass$38(ExportNamedDeclaration, [{
		key: 'bindChildren',
		value: function bindChildren() {
			// Do not bind specifiers
			if (this.declaration) this.declaration.bind();
		}
	}, {
		key: 'hasEffects',
		value: function hasEffects(options) {
			return this.declaration && this.declaration.hasEffects(options);
		}
	}, {
		key: 'initialiseNode',
		value: function initialiseNode() {
			this.isExportDeclaration = true;
		}
	}, {
		key: 'render',
		value: function render(code, es) {
			if (this.declaration) {
				code.remove(this.start, this.declaration.start);
				this.declaration.render(code, es);
			} else {
				var start = this.leadingCommentStart || this.start;
				var end = this.next || this.end;

				if (this.defaultExport) {
					var name = this.defaultExport.getName(es);
					var originalName = this.defaultExport.original.getName(es);

					if (name !== originalName) {
						code.overwrite(start, end, 'var ' + name + ' = ' + originalName + ';');
						return;
					}
				}

				code.remove(start, end);
			}
		}
	}]);

	return ExportNamedDeclaration;
}(Node);

var _createClass$39 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$14 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$39(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$31(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$31(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ExpressionStatement = function (_Statement) {
	_inherits$31(ExpressionStatement, _Statement);

	function ExpressionStatement() {
		_classCallCheck$39(this, ExpressionStatement);

		return _possibleConstructorReturn$31(this, (ExpressionStatement.__proto__ || Object.getPrototypeOf(ExpressionStatement)).apply(this, arguments));
	}

	_createClass$39(ExpressionStatement, [{
		key: 'render',
		value: function render(code, es) {
			_get$14(ExpressionStatement.prototype.__proto__ || Object.getPrototypeOf(ExpressionStatement.prototype), 'render', this).call(this, code, es);
			if (this.included) this.insertSemicolon(code);
		}
	}]);

	return ExpressionStatement;
}(Statement);

var _createClass$40 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$40(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$32(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$32(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ForStatement = function (_Statement) {
	_inherits$32(ForStatement, _Statement);

	function ForStatement() {
		_classCallCheck$40(this, ForStatement);

		return _possibleConstructorReturn$32(this, (ForStatement.__proto__ || Object.getPrototypeOf(ForStatement)).apply(this, arguments));
	}

	_createClass$40(ForStatement, [{
		key: 'hasEffects',
		value: function hasEffects(options) {
			return this.init && this.init.hasEffects(options) || this.test && this.test.hasEffects(options) || this.update && this.update.hasEffects(options) || this.body.hasEffects(options.setIgnoreBreakStatements());
		}
	}, {
		key: 'initialiseChildren',
		value: function initialiseChildren() {
			if (this.init) this.init.initialise(this.scope);
			if (this.test) this.test.initialise(this.scope);
			if (this.update) this.update.initialise(this.scope);

			if (this.body.type === 'BlockStatement') {
				this.body.initialiseScope(this.scope);
				this.body.initialiseChildren();
			} else {
				this.body.initialise(this.scope);
			}
		}
	}, {
		key: 'initialiseScope',
		value: function initialiseScope(parentScope) {
			this.scope = new BlockScope({ parent: parentScope });
		}
	}]);

	return ForStatement;
}(Statement);

var _createClass$41 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$15 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$41(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$33(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$33(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ForInStatement = function (_Statement) {
	_inherits$33(ForInStatement, _Statement);

	function ForInStatement() {
		_classCallCheck$41(this, ForInStatement);

		return _possibleConstructorReturn$33(this, (ForInStatement.__proto__ || Object.getPrototypeOf(ForInStatement)).apply(this, arguments));
	}

	_createClass$41(ForInStatement, [{
		key: 'hasEffects',
		value: function hasEffects(options) {
			return this.left && (this.left.hasEffects(options) || this.left.hasEffectsWhenAssignedAtPath([], options)) || this.right && this.right.hasEffects(options) || this.body.hasEffects(options.setIgnoreBreakStatements());
		}
	}, {
		key: 'initialiseChildren',
		value: function initialiseChildren() {
			this.left.initialise(this.scope);
			this.right.initialise(this.scope.parent);
			this.body.initialiseAndReplaceScope ? this.body.initialiseAndReplaceScope(this.scope) : this.body.initialise(this.scope);
		}
	}, {
		key: 'includeInBundle',
		value: function includeInBundle() {
			var addedNewNodes = _get$15(ForInStatement.prototype.__proto__ || Object.getPrototypeOf(ForInStatement.prototype), 'includeInBundle', this).call(this);
			if (this.left.includeWithAllDeclarations()) {
				addedNewNodes = true;
			}
			return addedNewNodes;
		}
	}, {
		key: 'initialiseScope',
		value: function initialiseScope(parentScope) {
			this.scope = new BlockScope({ parent: parentScope });
		}
	}]);

	return ForInStatement;
}(Statement);

var _createClass$42 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$16 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$42(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$34(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$34(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ForOfStatement = function (_Statement) {
	_inherits$34(ForOfStatement, _Statement);

	function ForOfStatement() {
		_classCallCheck$42(this, ForOfStatement);

		return _possibleConstructorReturn$34(this, (ForOfStatement.__proto__ || Object.getPrototypeOf(ForOfStatement)).apply(this, arguments));
	}

	_createClass$42(ForOfStatement, [{
		key: 'bindNode',
		value: function bindNode() {
			this.left.reassignPath([], ExecutionPathOptions.create());
		}
	}, {
		key: 'hasEffects',
		value: function hasEffects(options) {
			return this.left && (this.left.hasEffects(options) || this.left.hasEffectsWhenAssignedAtPath([], options)) || this.right && this.right.hasEffects(options) || this.body.hasEffects(options.setIgnoreBreakStatements());
		}
	}, {
		key: 'includeInBundle',
		value: function includeInBundle() {
			var addedNewNodes = _get$16(ForOfStatement.prototype.__proto__ || Object.getPrototypeOf(ForOfStatement.prototype), 'includeInBundle', this).call(this);
			if (this.left.includeWithAllDeclarations()) {
				addedNewNodes = true;
			}
			return addedNewNodes;
		}
	}, {
		key: 'initialiseChildren',
		value: function initialiseChildren() {
			this.left.initialise(this.scope);
			this.right.initialise(this.scope.parent);
			this.body.initialiseAndReplaceScope ? this.body.initialiseAndReplaceScope(this.scope) : this.body.initialise(this.scope);
		}
	}, {
		key: 'initialiseScope',
		value: function initialiseScope(parentScope) {
			this.scope = new BlockScope({ parent: parentScope });
		}
	}]);

	return ForOfStatement;
}(Statement);

var _createClass$46 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$46(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$38(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$38(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var getParameterVariable = function getParameterVariable(path$$1, options) {
	return path$$1[0] < options.getArgumentsVariables().length && options.getArgumentsVariables()[path$$1[0]] || UNDEFINED_ASSIGNMENT;
};

var ArgumentsVariable = function (_LocalVariable) {
	_inherits$38(ArgumentsVariable, _LocalVariable);

	function ArgumentsVariable(parameters) {
		_classCallCheck$46(this, ArgumentsVariable);

		var _this = _possibleConstructorReturn$38(this, (ArgumentsVariable.__proto__ || Object.getPrototypeOf(ArgumentsVariable)).call(this, 'arguments', null, UNKNOWN_ASSIGNMENT));

		_this._parameters = parameters;
		return _this;
	}

	_createClass$46(ArgumentsVariable, [{
		key: 'reassignPath',
		value: function reassignPath(path$$1, options) {
			if (path$$1.length > 0) {
				if (path$$1[0] >= 0 && this._parameters[path$$1[0]]) {
					this._parameters[path$$1[0]].reassignPath(path$$1.slice(1), options);
				}
			}
		}
	}, {
		key: 'hasEffectsWhenAccessedAtPath',
		value: function hasEffectsWhenAccessedAtPath(path$$1, options) {
			return path$$1.length > 1 && getParameterVariable(path$$1, options).hasEffectsWhenAccessedAtPath(path$$1.slice(1), options);
		}
	}, {
		key: 'hasEffectsWhenAssignedAtPath',
		value: function hasEffectsWhenAssignedAtPath(path$$1, options) {
			return path$$1.length === 0 || this.included || getParameterVariable(path$$1, options).hasEffectsWhenAssignedAtPath(path$$1.slice(1), options);
		}
	}, {
		key: 'hasEffectsWhenCalledAtPath',
		value: function hasEffectsWhenCalledAtPath(path$$1, callOptions, options) {
			if (path$$1.length === 0) {
				return true;
			}
			return getParameterVariable(path$$1, options).hasEffectsWhenCalledAtPath(path$$1.slice(1), callOptions, options);
		}
	}, {
		key: 'someReturnExpressionWhenCalledAtPath',
		value: function someReturnExpressionWhenCalledAtPath(path$$1, callOptions, predicateFunction, options) {
			if (path$$1.length === 0) {
				return true;
			}
			return getParameterVariable(path$$1, options).someReturnExpressionWhenCalledAtPath(path$$1.slice(1), callOptions, predicateFunction, options);
		}
	}]);

	return ArgumentsVariable;
}(LocalVariable);

function _classCallCheck$47(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$39(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$39(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ThisVariable = function (_ReplaceableInitializ) {
	_inherits$39(ThisVariable, _ReplaceableInitializ);

	function ThisVariable() {
		_classCallCheck$47(this, ThisVariable);

		return _possibleConstructorReturn$39(this, (ThisVariable.__proto__ || Object.getPrototypeOf(ThisVariable)).call(this, 'this', null));
	}

	return ThisVariable;
}(ReplaceableInitializationVariable);

var _createClass$47 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$48(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$40(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$40(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var VirtualObjectExpression = function (_Node) {
	_inherits$40(VirtualObjectExpression, _Node);

	function VirtualObjectExpression() {
		_classCallCheck$48(this, VirtualObjectExpression);

		return _possibleConstructorReturn$40(this, (VirtualObjectExpression.__proto__ || Object.getPrototypeOf(VirtualObjectExpression)).apply(this, arguments));
	}

	_createClass$47(VirtualObjectExpression, [{
		key: 'hasEffectsWhenAccessedAtPath',
		value: function hasEffectsWhenAccessedAtPath(path$$1) {
			return path$$1.length > 1;
		}
	}, {
		key: 'hasEffectsWhenAssignedAtPath',
		value: function hasEffectsWhenAssignedAtPath(path$$1) {
			return path$$1.length > 1;
		}
	}, {
		key: 'toString',
		value: function toString() {
			return '[[VIRTUAL OBJECT]]';
		}
	}]);

	return VirtualObjectExpression;
}(Node);

var _createClass$45 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$19 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$45(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$37(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$37(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var FunctionScope = function (_ReturnValueScope) {
	_inherits$37(FunctionScope, _ReturnValueScope);

	function FunctionScope() {
		var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

		_classCallCheck$45(this, FunctionScope);

		var _this = _possibleConstructorReturn$37(this, (FunctionScope.__proto__ || Object.getPrototypeOf(FunctionScope)).call(this, options));

		_this.variables.arguments = new ArgumentsVariable(_get$19(FunctionScope.prototype.__proto__ || Object.getPrototypeOf(FunctionScope.prototype), 'getParameterVariables', _this).call(_this));
		_this.variables.this = new ThisVariable();
		return _this;
	}

	_createClass$45(FunctionScope, [{
		key: 'findLexicalBoundary',
		value: function findLexicalBoundary() {
			return this;
		}
	}, {
		key: 'getOptionsWhenCalledWith',
		value: function getOptionsWhenCalledWith(_ref, options) {
			var _this2 = this;

			var args = _ref.args,
			    withNew = _ref.withNew;

			return options.replaceVariableInit(this.variables.this, withNew ? new VirtualObjectExpression() : UNKNOWN_ASSIGNMENT).setArgumentsVariables(args.map(function (parameter, index) {
				return _get$19(FunctionScope.prototype.__proto__ || Object.getPrototypeOf(FunctionScope.prototype), 'getParameterVariables', _this2).call(_this2)[index] || parameter;
			}));
		}
	}]);

	return FunctionScope;
}(ReturnValueScope);

var _createClass$44 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$18 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$44(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$36(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$36(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var FunctionNode = function (_Node) {
	_inherits$36(FunctionNode, _Node);

	function FunctionNode() {
		_classCallCheck$44(this, FunctionNode);

		return _possibleConstructorReturn$36(this, (FunctionNode.__proto__ || Object.getPrototypeOf(FunctionNode)).apply(this, arguments));
	}

	_createClass$44(FunctionNode, [{
		key: 'bindNode',
		value: function bindNode() {
			this.body.bindImplicitReturnExpressionToScope();
		}
	}, {
		key: 'forEachReturnExpressionWhenCalledAtPath',
		value: function forEachReturnExpressionWhenCalledAtPath(path$$1, callOptions, callback, options) {
			path$$1.length === 0 && this.scope.forEachReturnExpressionWhenCalled(callOptions, callback, options);
		}
	}, {
		key: 'hasEffects',
		value: function hasEffects(options) {
			return this.id && this.id.hasEffects(options);
		}
	}, {
		key: 'hasEffectsWhenAccessedAtPath',
		value: function hasEffectsWhenAccessedAtPath(path$$1, options) {
			if (path$$1.length <= 1) {
				return false;
			}
			if (path$$1[0] === 'prototype') {
				return this.prototypeObject.hasEffectsWhenAccessedAtPath(path$$1.slice(1), options);
			}
			return true;
		}
	}, {
		key: 'hasEffectsWhenAssignedAtPath',
		value: function hasEffectsWhenAssignedAtPath(path$$1, options) {
			if (path$$1.length <= 1) {
				return false;
			}
			if (path$$1[0] === 'prototype') {
				return this.prototypeObject.hasEffectsWhenAssignedAtPath(path$$1.slice(1), options);
			}
			return true;
		}
	}, {
		key: 'hasEffectsWhenCalledAtPath',
		value: function hasEffectsWhenCalledAtPath(path$$1, callOptions, options) {
			if (path$$1.length > 0) {
				return true;
			}
			var innerOptions = this.scope.getOptionsWhenCalledWith(callOptions, options);
			return this.params.some(function (param) {
				return param.hasEffects(innerOptions);
			}) || this.body.hasEffects(innerOptions);
		}
	}, {
		key: 'includeInBundle',
		value: function includeInBundle() {
			this.scope.variables.arguments.includeVariable();
			return _get$18(FunctionNode.prototype.__proto__ || Object.getPrototypeOf(FunctionNode.prototype), 'includeInBundle', this).call(this);
		}
	}, {
		key: 'initialiseNode',
		value: function initialiseNode() {
			this.prototypeObject = new VirtualObjectExpression();
		}
	}, {
		key: 'initialiseScope',
		value: function initialiseScope(parentScope) {
			this.scope = new FunctionScope({ parent: parentScope });
		}
	}, {
		key: 'someReturnExpressionWhenCalledAtPath',
		value: function someReturnExpressionWhenCalledAtPath(path$$1, callOptions, predicateFunction, options) {
			return path$$1.length > 0 || this.scope.someReturnExpressionWhenCalled(callOptions, predicateFunction, options);
		}
	}]);

	return FunctionNode;
}(Node);

var _createClass$43 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$17 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$43(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$35(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$35(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var FunctionDeclaration = function (_FunctionNode) {
	_inherits$35(FunctionDeclaration, _FunctionNode);

	function FunctionDeclaration() {
		_classCallCheck$43(this, FunctionDeclaration);

		return _possibleConstructorReturn$35(this, (FunctionDeclaration.__proto__ || Object.getPrototypeOf(FunctionDeclaration)).apply(this, arguments));
	}

	_createClass$43(FunctionDeclaration, [{
		key: 'initialiseChildren',
		value: function initialiseChildren(parentScope) {
			var _this2 = this;

			this.id && this.id.initialiseAndDeclare(parentScope, 'function', this);
			this.params.forEach(function (param) {
				return param.initialiseAndDeclare(_this2.scope, 'parameter');
			});
			this.body.initialiseAndReplaceScope(new Scope({ parent: this.scope }));
		}
	}, {
		key: 'render',
		value: function render(code, es) {
			if (!this.module.bundle.treeshake || this.included) {
				_get$17(FunctionDeclaration.prototype.__proto__ || Object.getPrototypeOf(FunctionDeclaration.prototype), 'render', this).call(this, code, es);
			} else {
				code.remove(this.leadingCommentStart || this.start, this.next || this.end);
			}
		}
	}]);

	return FunctionDeclaration;
}(FunctionNode);

var _createClass$48 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$49(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$41(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$41(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var FunctionExpression = function (_FunctionNode) {
	_inherits$41(FunctionExpression, _FunctionNode);

	function FunctionExpression() {
		_classCallCheck$49(this, FunctionExpression);

		return _possibleConstructorReturn$41(this, (FunctionExpression.__proto__ || Object.getPrototypeOf(FunctionExpression)).apply(this, arguments));
	}

	_createClass$48(FunctionExpression, [{
		key: 'initialiseChildren',
		value: function initialiseChildren() {
			var _this2 = this;

			this.id && this.id.initialiseAndDeclare(this.scope, 'function', this);
			this.params.forEach(function (param) {
				return param.initialiseAndDeclare(_this2.scope, 'parameter');
			});
			this.body.initialiseAndReplaceScope(new Scope({ parent: this.scope }));
		}
	}]);

	return FunctionExpression;
}(FunctionNode);

var _createClass$49 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$50(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$42(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$42(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var Identifier = function (_Node) {
	_inherits$42(Identifier, _Node);

	function Identifier() {
		_classCallCheck$50(this, Identifier);

		return _possibleConstructorReturn$42(this, (Identifier.__proto__ || Object.getPrototypeOf(Identifier)).apply(this, arguments));
	}

	_createClass$49(Identifier, [{
		key: 'reassignPath',
		value: function reassignPath(path$$1, options) {
			this._bindVariableIfMissing();
			this.variable && this.variable.reassignPath(path$$1, options);
		}
	}, {
		key: 'bindNode',
		value: function bindNode() {
			this._bindVariableIfMissing();
		}
	}, {
		key: '_bindVariableIfMissing',
		value: function _bindVariableIfMissing() {
			if (!this.variable && isReference(this, this.parent)) {
				this.variable = this.scope.findVariable(this.name);
				this.variable.addReference(this);
			}
		}
	}, {
		key: 'forEachReturnExpressionWhenCalledAtPath',
		value: function forEachReturnExpressionWhenCalledAtPath(path$$1, callOptions, callback, options) {
			this._bindVariableIfMissing();
			this.variable && this.variable.forEachReturnExpressionWhenCalledAtPath(path$$1, callOptions, callback, options);
		}
	}, {
		key: 'hasEffectsWhenAccessedAtPath',
		value: function hasEffectsWhenAccessedAtPath(path$$1, options) {
			return this.variable && this.variable.hasEffectsWhenAccessedAtPath(path$$1, options);
		}
	}, {
		key: 'hasEffectsWhenAssignedAtPath',
		value: function hasEffectsWhenAssignedAtPath(path$$1, options) {
			return !this.variable || this.variable.hasEffectsWhenAssignedAtPath(path$$1, options);
		}
	}, {
		key: 'hasEffectsWhenCalledAtPath',
		value: function hasEffectsWhenCalledAtPath(path$$1, callOptions, options) {
			return !this.variable || this.variable.hasEffectsWhenCalledAtPath(path$$1, callOptions, options);
		}
	}, {
		key: 'includeInBundle',
		value: function includeInBundle() {
			if (this.included) return false;
			this.included = true;
			this.variable && this.variable.includeVariable();
			return true;
		}
	}, {
		key: 'initialiseAndDeclare',
		value: function initialiseAndDeclare(parentScope, kind, init) {
			this.initialiseScope(parentScope);
			switch (kind) {
				case 'var':
				case 'function':
					this.variable = this.scope.addDeclaration(this, { isHoisted: true, init: init });
					break;
				case 'let':
				case 'const':
				case 'class':
					this.variable = this.scope.addDeclaration(this, { init: init });
					break;
				case 'parameter':
					this.variable = this.scope.addParameterDeclaration(this);
					break;
				default:
					throw new Error('Unexpected identifier kind', kind);
			}
		}
	}, {
		key: 'render',
		value: function render(code, es) {
			if (this.variable) {
				var name = this.variable.getName(es);
				if (name !== this.name) {
					code.overwrite(this.start, this.end, name, { storeName: true, contentOnly: false });

					// special case
					if (this.parent.type === 'Property' && this.parent.shorthand) {
						code.appendLeft(this.start, this.name + ': ');
					}
				}
			}
		}
	}, {
		key: 'someReturnExpressionWhenCalledAtPath',
		value: function someReturnExpressionWhenCalledAtPath(path$$1, callOptions, predicateFunction, options) {
			if (this.variable) {
				return this.variable.someReturnExpressionWhenCalledAtPath(path$$1, callOptions, predicateFunction, options);
			}
			return predicateFunction(options)(UNKNOWN_ASSIGNMENT);
		}
	}]);

	return Identifier;
}(Node);

var _createClass$50 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$20 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$51(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$43(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$43(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

// Statement types which may contain if-statements as direct children.
var statementsWithIfStatements = new Set(['DoWhileStatement', 'ForInStatement', 'ForOfStatement', 'ForStatement', 'IfStatement', 'WhileStatement']);

function getHoistedVars(node, scope) {
	var hoistedVars = [];

	function visit(node) {
		if (node.type === 'VariableDeclaration' && node.kind === 'var') {
			node.declarations.forEach(function (declarator) {
				declarator.init = null;
				declarator.initialise(scope);

				extractNames(declarator.id).forEach(function (name) {
					if (hoistedVars.indexOf(name) < 0) hoistedVars.push(name);
				});
			});
		} else if (!/Function/.test(node.type)) {
			node.eachChild(visit);
		}
	}

	visit(node);

	return hoistedVars;
}

var IfStatement = function (_Statement) {
	_inherits$43(IfStatement, _Statement);

	function IfStatement() {
		_classCallCheck$51(this, IfStatement);

		return _possibleConstructorReturn$43(this, (IfStatement.__proto__ || Object.getPrototypeOf(IfStatement)).apply(this, arguments));
	}

	_createClass$50(IfStatement, [{
		key: 'initialiseChildren',
		value: function initialiseChildren(parentScope) {
			_get$20(IfStatement.prototype.__proto__ || Object.getPrototypeOf(IfStatement.prototype), 'initialiseChildren', this).call(this, parentScope);
			if (this.module.bundle.treeshake) {
				this.testValue = this.test.getValue();

				if (this.testValue === UNKNOWN_VALUE) {
					return;
				}
				if (this.testValue) {
					if (this.alternate) {
						this.hoistedVars = getHoistedVars(this.alternate, this.scope);
						this.alternate = null;
					}
				} else {
					this.hoistedVars = getHoistedVars(this.consequent, this.scope);
					this.consequent = null;
				}
			}
		}
	}, {
		key: 'render',
		value: function render(code, es) {
			var _this2 = this;

			if (this.module.bundle.treeshake) {
				if (this.testValue === UNKNOWN_VALUE) {
					_get$20(IfStatement.prototype.__proto__ || Object.getPrototypeOf(IfStatement.prototype), 'render', this).call(this, code, es);
				} else {
					code.overwrite(this.test.start, this.test.end, JSON.stringify(this.testValue));

					// TODO if no block-scoped declarations, remove enclosing
					// curlies and dedent block (if there is a block)

					if (this.hoistedVars) {
						var names = this.hoistedVars.map(function (name) {
							var variable = _this2.scope.findVariable(name);
							return variable.included ? variable.getName() : null;
						}).filter(Boolean);

						if (names.length > 0) {
							code.appendLeft(this.start, 'var ' + names.join(', ') + ';\n\n');
						}
					}

					if (this.testValue) {
						code.remove(this.start, this.consequent.start);
						code.remove(this.consequent.end, this.end);
						this.consequent.render(code, es);
					} else {
						code.remove(this.start, this.alternate ? this.alternate.start : this.next || this.end);

						if (this.alternate) {
							this.alternate.render(code, es);
						} else if (statementsWithIfStatements.has(this.parent.type)) {
							code.prependRight(this.start, '{}');
						}
					}
				}
			} else {
				_get$20(IfStatement.prototype.__proto__ || Object.getPrototypeOf(IfStatement.prototype), 'render', this).call(this, code, es);
			}
		}
	}]);

	return IfStatement;
}(Statement);

var _createClass$51 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$52(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$44(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$44(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ImportDeclaration = function (_Node) {
	_inherits$44(ImportDeclaration, _Node);

	function ImportDeclaration() {
		_classCallCheck$52(this, ImportDeclaration);

		return _possibleConstructorReturn$44(this, (ImportDeclaration.__proto__ || Object.getPrototypeOf(ImportDeclaration)).apply(this, arguments));
	}

	_createClass$51(ImportDeclaration, [{
		key: 'bindChildren',
		value: function bindChildren() {}
	}, {
		key: 'initialiseNode',
		value: function initialiseNode() {
			this.isImportDeclaration = true;
		}
	}, {
		key: 'render',
		value: function render(code) {
			code.remove(this.start, this.next || this.end);
		}
	}]);

	return ImportDeclaration;
}(Node);

var _createClass$52 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$53(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$45(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$45(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var LabeledStatement = function (_Statement) {
	_inherits$45(LabeledStatement, _Statement);

	function LabeledStatement() {
		_classCallCheck$53(this, LabeledStatement);

		return _possibleConstructorReturn$45(this, (LabeledStatement.__proto__ || Object.getPrototypeOf(LabeledStatement)).apply(this, arguments));
	}

	_createClass$52(LabeledStatement, [{
		key: 'hasEffects',
		value: function hasEffects(options) {
			return this.body.hasEffects(options.setIgnoreLabel(this.label.name).setIgnoreBreakStatements());
		}
	}]);

	return LabeledStatement;
}(Statement);

var _createClass$53 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$54(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$46(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$46(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var Literal = function (_Node) {
	_inherits$46(Literal, _Node);

	function Literal() {
		_classCallCheck$54(this, Literal);

		return _possibleConstructorReturn$46(this, (Literal.__proto__ || Object.getPrototypeOf(Literal)).apply(this, arguments));
	}

	_createClass$53(Literal, [{
		key: 'getValue',
		value: function getValue() {
			return this.value;
		}
	}, {
		key: 'hasEffectsWhenAccessedAtPath',
		value: function hasEffectsWhenAccessedAtPath(path$$1) {
			if (this.value === null) {
				return path$$1.length > 0;
			}
			return path$$1.length > 1;
		}
	}, {
		key: 'hasEffectsWhenAssignedAtPath',
		value: function hasEffectsWhenAssignedAtPath(path$$1) {
			if (this.value === null) {
				return path$$1.length > 0;
			}
			return path$$1.length > 1;
		}
	}, {
		key: 'render',
		value: function render(code) {
			if (typeof this.value === 'string') {
				code.indentExclusionRanges.push([this.start + 1, this.end - 1]);
			}
		}
	}]);

	return Literal;
}(Node);

var _createClass$54 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$55(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$47(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$47(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var LogicalExpression = function (_Node) {
	_inherits$47(LogicalExpression, _Node);

	function LogicalExpression() {
		_classCallCheck$55(this, LogicalExpression);

		return _possibleConstructorReturn$47(this, (LogicalExpression.__proto__ || Object.getPrototypeOf(LogicalExpression)).apply(this, arguments));
	}

	_createClass$54(LogicalExpression, [{
		key: 'reassignPath',
		value: function reassignPath(path$$1, options) {
			path$$1.length > 0 && this._forEachRelevantBranch(function (node) {
				return node.reassignPath(path$$1, options);
			});
		}
	}, {
		key: 'forEachReturnExpressionWhenCalledAtPath',
		value: function forEachReturnExpressionWhenCalledAtPath(path$$1, callOptions, callback, options) {
			this._forEachRelevantBranch(function (node) {
				return node.forEachReturnExpressionWhenCalledAtPath(path$$1, callOptions, callback, options);
			});
		}
	}, {
		key: 'getValue',
		value: function getValue() {
			var leftValue = this.left.getValue();
			if (leftValue === UNKNOWN_VALUE) return UNKNOWN_VALUE;
			if (leftValue && this.operator === '||' || !leftValue && this.operator === '&&') {
				return leftValue;
			}
			return this.right.getValue();
		}
	}, {
		key: 'hasEffects',
		value: function hasEffects(options) {
			var leftValue = this.left.getValue();
			return this.left.hasEffects(options) || (leftValue === UNKNOWN_VALUE || !leftValue && this.operator === '||' || leftValue && this.operator === '&&') && this.right.hasEffects(options);
		}
	}, {
		key: 'hasEffectsWhenAccessedAtPath',
		value: function hasEffectsWhenAccessedAtPath(path$$1, options) {
			return path$$1.length > 0 && this._someRelevantBranch(function (node) {
				return node.hasEffectsWhenAccessedAtPath(path$$1, options);
			});
		}
	}, {
		key: 'hasEffectsWhenAssignedAtPath',
		value: function hasEffectsWhenAssignedAtPath(path$$1, options) {
			return path$$1.length === 0 || this._someRelevantBranch(function (node) {
				return node.hasEffectsWhenAssignedAtPath(path$$1, options);
			});
		}
	}, {
		key: 'hasEffectsWhenCalledAtPath',
		value: function hasEffectsWhenCalledAtPath(path$$1, callOptions, options) {
			return this._someRelevantBranch(function (node) {
				return node.hasEffectsWhenCalledAtPath(path$$1, callOptions, options);
			});
		}
	}, {
		key: 'someReturnExpressionWhenCalledAtPath',
		value: function someReturnExpressionWhenCalledAtPath(path$$1, callOptions, predicateFunction, options) {
			return this._someRelevantBranch(function (node) {
				return node.someReturnExpressionWhenCalledAtPath(path$$1, callOptions, predicateFunction, options);
			});
		}
	}, {
		key: '_forEachRelevantBranch',
		value: function _forEachRelevantBranch(callback) {
			var leftValue = this.left.getValue();
			if (leftValue === UNKNOWN_VALUE) {
				callback(this.left);
				callback(this.right);
			} else if (leftValue && this.operator === '||' || !leftValue && this.operator === '&&') {
				callback(this.left);
			} else {
				callback(this.right);
			}
		}
	}, {
		key: '_someRelevantBranch',
		value: function _someRelevantBranch(predicateFunction) {
			var leftValue = this.left.getValue();
			if (leftValue === UNKNOWN_VALUE) {
				return predicateFunction(this.left) || predicateFunction(this.right);
			}
			if (leftValue && this.operator === '||' || !leftValue && this.operator === '&&') {
				return predicateFunction(this.left);
			}
			return predicateFunction(this.right);
		}
	}]);

	return LogicalExpression;
}(Node);

var _createClass$55 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$21 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _toConsumableArray$2(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _possibleConstructorReturn$48(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$48(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _classCallCheck$56(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var validProp = /^[a-zA-Z_$][a-zA-Z_$0-9]*$/;

var Keypath = function Keypath(node) {
	_classCallCheck$56(this, Keypath);

	this.parts = [];

	while (node.type === 'MemberExpression') {
		var prop = node.property;

		if (node.computed) {
			if (prop.type !== 'Literal' || typeof prop.value !== 'string' || !validProp.test(prop.value)) {
				this.computed = true;
				return;
			}
		}

		this.parts.unshift(prop);
		node = node.object;
	}

	this.root = node;
};

var MemberExpression = function (_Node) {
	_inherits$48(MemberExpression, _Node);

	function MemberExpression() {
		_classCallCheck$56(this, MemberExpression);

		return _possibleConstructorReturn$48(this, (MemberExpression.__proto__ || Object.getPrototypeOf(MemberExpression)).apply(this, arguments));
	}

	_createClass$55(MemberExpression, [{
		key: 'bind',
		value: function bind() {
			// if this resolves to a namespaced declaration, prepare
			// to replace it
			// TODO this code is a bit inefficient
			this._bound = true;
			var keypath = new Keypath(this);

			if (!keypath.computed && keypath.root.type === 'Identifier') {
				var variable = this.scope.findVariable(keypath.root.name);

				while (variable.isNamespace && keypath.parts.length) {
					var exporterId = variable.module.id;

					var part = keypath.parts[0];
					variable = variable.module.traceExport(part.name || part.value);

					if (!variable) {
						this.module.warn({
							code: 'MISSING_EXPORT',
							missing: part.name || part.value,
							importer: relativeId(this.module.id),
							exporter: relativeId(exporterId),
							message: '\'' + (part.name || part.value) + '\' is not exported by \'' + relativeId(exporterId) + '\'',
							url: 'https://github.com/rollup/rollup/wiki/Troubleshooting#name-is-not-exported-by-module'
						}, part.start);
						this.replacement = 'undefined';
						return;
					}

					keypath.parts.shift();
				}

				if (keypath.parts.length) {
					_get$21(MemberExpression.prototype.__proto__ || Object.getPrototypeOf(MemberExpression.prototype), 'bind', this).call(this);
					return; // not a namespaced declaration
				}

				this.variable = variable;

				if (variable.isExternal) {
					variable.module.suggestName(keypath.root.name);
				}
			} else {
				_get$21(MemberExpression.prototype.__proto__ || Object.getPrototypeOf(MemberExpression.prototype), 'bind', this).call(this);
			}
		}
	}, {
		key: 'reassignPath',
		value: function reassignPath(path$$1, options) {
			if (!this._bound) this.bind();
			if (this.variable) {
				this.variable.reassignPath(path$$1, options);
			} else {
				this.object.reassignPath([this._getPathSegment()].concat(_toConsumableArray$2(path$$1)), options);
			}
		}
	}, {
		key: 'forEachReturnExpressionWhenCalledAtPath',
		value: function forEachReturnExpressionWhenCalledAtPath(path$$1, callOptions, callback, options) {
			if (!this._bound) this.bind();
			if (this.variable) {
				this.variable.forEachReturnExpressionWhenCalledAtPath(path$$1, callOptions, callback, options);
			} else {
				this.object.forEachReturnExpressionWhenCalledAtPath([this._getPathSegment()].concat(_toConsumableArray$2(path$$1)), callOptions, callback, options);
			}
		}
	}, {
		key: 'hasEffects',
		value: function hasEffects(options) {
			return _get$21(MemberExpression.prototype.__proto__ || Object.getPrototypeOf(MemberExpression.prototype), 'hasEffects', this).call(this, options) || this._checkPropertyReadSideEffects && this.object.hasEffectsWhenAccessedAtPath([this._getPathSegment()], options);
		}
	}, {
		key: 'hasEffectsWhenAccessedAtPath',
		value: function hasEffectsWhenAccessedAtPath(path$$1, options) {
			if (path$$1.length === 0) {
				return false;
			}
			if (this.variable) {
				return this.variable.hasEffectsWhenAccessedAtPath(path$$1, options);
			}
			return this.object.hasEffectsWhenAccessedAtPath([this._getPathSegment()].concat(_toConsumableArray$2(path$$1)), options);
		}
	}, {
		key: 'hasEffectsWhenAssignedAtPath',
		value: function hasEffectsWhenAssignedAtPath(path$$1, options) {
			if (this.variable) {
				return this.variable.hasEffectsWhenAssignedAtPath(path$$1, options);
			}
			return this.object.hasEffectsWhenAssignedAtPath([this._getPathSegment()].concat(_toConsumableArray$2(path$$1)), options);
		}
	}, {
		key: 'hasEffectsWhenCalledAtPath',
		value: function hasEffectsWhenCalledAtPath(path$$1, callOptions, options) {
			if (this.variable) {
				return this.variable.hasEffectsWhenCalledAtPath(path$$1, callOptions, options);
			}
			return this._getPathSegment() === UNKNOWN_KEY || this.object.hasEffectsWhenCalledAtPath([this._getPathSegment()].concat(_toConsumableArray$2(path$$1)), callOptions, options);
		}
	}, {
		key: 'includeInBundle',
		value: function includeInBundle() {
			var addedNewNodes = _get$21(MemberExpression.prototype.__proto__ || Object.getPrototypeOf(MemberExpression.prototype), 'includeInBundle', this).call(this);
			if (this.variable && !this.variable.included) {
				this.variable.includeVariable();
				addedNewNodes = true;
			}
			return addedNewNodes;
		}
	}, {
		key: 'initialiseNode',
		value: function initialiseNode() {
			this._checkPropertyReadSideEffects = this.module.bundle.treeshake && this.module.bundle.treeshakingOptions.propertyReadSideEffects;
		}
	}, {
		key: 'render',
		value: function render(code, es) {
			if (this.variable) {
				var name = this.variable.getName(es);
				if (name !== this.name) code.overwrite(this.start, this.end, name, { storeName: true, contentOnly: false });
			} else if (this.replacement) {
				code.overwrite(this.start, this.end, this.replacement, { storeName: true, contentOnly: false });
			}

			_get$21(MemberExpression.prototype.__proto__ || Object.getPrototypeOf(MemberExpression.prototype), 'render', this).call(this, code, es);
		}
	}, {
		key: 'someReturnExpressionWhenCalledAtPath',
		value: function someReturnExpressionWhenCalledAtPath(path$$1, callOptions, predicateFunction, options) {
			if (this.variable) {
				return this.variable.someReturnExpressionWhenCalledAtPath(path$$1, callOptions, predicateFunction, options);
			}
			return this._getPathSegment() === UNKNOWN_KEY || this.object.someReturnExpressionWhenCalledAtPath([this._getPathSegment()].concat(_toConsumableArray$2(path$$1)), callOptions, predicateFunction, options);
		}
	}, {
		key: '_getPathSegment',
		value: function _getPathSegment() {
			if (this.computed) {
				return this.property.type === 'Literal' ? String(this.property.value) : UNKNOWN_KEY;
			}
			return this.property.name;
		}
	}]);

	return MemberExpression;
}(Node);

var _createClass$56 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$57(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$49(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$49(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var MethodDefinition = function (_Node) {
	_inherits$49(MethodDefinition, _Node);

	function MethodDefinition() {
		_classCallCheck$57(this, MethodDefinition);

		return _possibleConstructorReturn$49(this, (MethodDefinition.__proto__ || Object.getPrototypeOf(MethodDefinition)).apply(this, arguments));
	}

	_createClass$56(MethodDefinition, [{
		key: 'hasEffects',
		value: function hasEffects(options) {
			return this.key.hasEffects(options);
		}
	}, {
		key: 'hasEffectsWhenCalledAtPath',
		value: function hasEffectsWhenCalledAtPath(path$$1, callOptions, options) {
			return path$$1.length > 0 || this.value.hasEffectsWhenCalledAtPath([], callOptions, options);
		}
	}]);

	return MethodDefinition;
}(Node);

var _createClass$57 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$58(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$50(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$50(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var NewExpression = function (_Node) {
	_inherits$50(NewExpression, _Node);

	function NewExpression() {
		_classCallCheck$58(this, NewExpression);

		return _possibleConstructorReturn$50(this, (NewExpression.__proto__ || Object.getPrototypeOf(NewExpression)).apply(this, arguments));
	}

	_createClass$57(NewExpression, [{
		key: 'hasEffects',
		value: function hasEffects(options) {
			return this.arguments.some(function (child) {
				return child.hasEffects(options);
			}) || this.callee.hasEffectsWhenCalledAtPath([], this._callOptions, options.getHasEffectsWhenCalledOptions());
		}
	}, {
		key: 'hasEffectsWhenAccessedAtPath',
		value: function hasEffectsWhenAccessedAtPath(path$$1) {
			return path$$1.length > 1;
		}
	}, {
		key: 'initialiseNode',
		value: function initialiseNode() {
			this._callOptions = CallOptions.create({ withNew: true, args: this.arguments, caller: this });
		}
	}]);

	return NewExpression;
}(Node);

var _createClass$58 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$59(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$51(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$51(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var PROPERTY_KINDS_READ = ['init', 'get'];
var PROPERTY_KINDS_WRITE = ['init', 'set'];

var ObjectExpression = function (_Node) {
	_inherits$51(ObjectExpression, _Node);

	function ObjectExpression() {
		_classCallCheck$59(this, ObjectExpression);

		return _possibleConstructorReturn$51(this, (ObjectExpression.__proto__ || Object.getPrototypeOf(ObjectExpression)).apply(this, arguments));
	}

	_createClass$58(ObjectExpression, [{
		key: 'reassignPath',
		value: function reassignPath(path$$1, options) {
			if (path$$1.length === 0) return;

			var _getPossiblePropertie = this._getPossiblePropertiesWithName(path$$1[0], path$$1.length === 1 ? PROPERTY_KINDS_WRITE : PROPERTY_KINDS_READ),
			    properties = _getPossiblePropertie.properties,
			    hasCertainHit = _getPossiblePropertie.hasCertainHit;

			(path$$1.length === 1 || hasCertainHit) && properties.forEach(function (property) {
				return (path$$1.length > 1 || property.kind === 'set') && property.reassignPath(path$$1.slice(1), options);
			});
		}
	}, {
		key: 'forEachReturnExpressionWhenCalledAtPath',
		value: function forEachReturnExpressionWhenCalledAtPath(path$$1, callOptions, callback, options) {
			if (path$$1.length === 0) return;

			var _getPossiblePropertie2 = this._getPossiblePropertiesWithName(path$$1[0], PROPERTY_KINDS_READ),
			    properties = _getPossiblePropertie2.properties,
			    hasCertainHit = _getPossiblePropertie2.hasCertainHit;

			hasCertainHit && properties.forEach(function (property) {
				return property.forEachReturnExpressionWhenCalledAtPath(path$$1.slice(1), callOptions, callback, options);
			});
		}
	}, {
		key: '_getPossiblePropertiesWithName',
		value: function _getPossiblePropertiesWithName(name, kinds) {
			if (name === UNKNOWN_KEY) {
				return { properties: this.properties, hasCertainHit: false };
			}
			var properties = [];
			var hasCertainHit = false;

			for (var index = this.properties.length - 1; index >= 0; index--) {
				var property = this.properties[index];
				if (kinds.indexOf(property.kind) < 0) continue;
				if (property.computed) {
					properties.push(property);
				} else if (property.key.name === name) {
					properties.push(property);
					hasCertainHit = true;
					break;
				}
			}
			return { properties: properties, hasCertainHit: hasCertainHit };
		}
	}, {
		key: 'hasEffectsWhenAccessedAtPath',
		value: function hasEffectsWhenAccessedAtPath(path$$1, options) {
			if (path$$1.length === 0) return false;

			var _getPossiblePropertie3 = this._getPossiblePropertiesWithName(path$$1[0], PROPERTY_KINDS_READ),
			    properties = _getPossiblePropertie3.properties,
			    hasCertainHit = _getPossiblePropertie3.hasCertainHit;

			return path$$1.length > 1 && !hasCertainHit || properties.some(function (property) {
				return property.hasEffectsWhenAccessedAtPath(path$$1.slice(1), options);
			});
		}
	}, {
		key: 'hasEffectsWhenAssignedAtPath',
		value: function hasEffectsWhenAssignedAtPath(path$$1, options) {
			if (path$$1.length === 0) return false;

			var _getPossiblePropertie4 = this._getPossiblePropertiesWithName(path$$1[0], path$$1.length === 1 ? PROPERTY_KINDS_WRITE : PROPERTY_KINDS_READ),
			    properties = _getPossiblePropertie4.properties,
			    hasCertainHit = _getPossiblePropertie4.hasCertainHit;

			return path$$1.length > 1 && !hasCertainHit || properties.some(function (property) {
				return (path$$1.length > 1 || property.kind === 'set') && property.hasEffectsWhenAssignedAtPath(path$$1.slice(1), options);
			});
		}
	}, {
		key: 'hasEffectsWhenCalledAtPath',
		value: function hasEffectsWhenCalledAtPath(path$$1, callOptions, options) {
			if (path$$1.length === 0) return true;

			var _getPossiblePropertie5 = this._getPossiblePropertiesWithName(path$$1[0], PROPERTY_KINDS_READ),
			    properties = _getPossiblePropertie5.properties,
			    hasCertainHit = _getPossiblePropertie5.hasCertainHit;

			return !hasCertainHit || properties.some(function (property) {
				return property.hasEffectsWhenCalledAtPath(path$$1.slice(1), callOptions, options);
			});
		}
	}, {
		key: 'someReturnExpressionWhenCalledAtPath',
		value: function someReturnExpressionWhenCalledAtPath(path$$1, callOptions, predicateFunction, options) {
			if (path$$1.length === 0) return true;

			var _getPossiblePropertie6 = this._getPossiblePropertiesWithName(path$$1[0], PROPERTY_KINDS_READ),
			    properties = _getPossiblePropertie6.properties,
			    hasCertainHit = _getPossiblePropertie6.hasCertainHit;

			return !hasCertainHit || properties.some(function (property) {
				return property.someReturnExpressionWhenCalledAtPath(path$$1.slice(1), callOptions, predicateFunction, options);
			});
		}
	}]);

	return ObjectExpression;
}(Node);

var _createClass$59 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$60(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$52(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$52(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ObjectPattern = function (_Node) {
	_inherits$52(ObjectPattern, _Node);

	function ObjectPattern() {
		_classCallCheck$60(this, ObjectPattern);

		return _possibleConstructorReturn$52(this, (ObjectPattern.__proto__ || Object.getPrototypeOf(ObjectPattern)).apply(this, arguments));
	}

	_createClass$59(ObjectPattern, [{
		key: 'reassignPath',
		value: function reassignPath(path$$1, options) {
			path$$1.length === 0 && this.properties.forEach(function (child) {
				return child.reassignPath(path$$1, options);
			});
		}
	}, {
		key: 'hasEffectsWhenAssignedAtPath',
		value: function hasEffectsWhenAssignedAtPath(path$$1, options) {
			return path$$1.length > 0 || this.someChild(function (child) {
				return child.hasEffectsWhenAssignedAtPath([], options);
			});
		}
	}, {
		key: 'initialiseAndDeclare',
		value: function initialiseAndDeclare(parentScope, kind, init) {
			this.initialiseScope(parentScope);
			this.properties.forEach(function (child) {
				return child.initialiseAndDeclare(parentScope, kind, init);
			});
		}
	}]);

	return ObjectPattern;
}(Node);

var _createClass$60 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$61(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$53(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$53(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var Property = function (_Node) {
	_inherits$53(Property, _Node);

	function Property() {
		_classCallCheck$61(this, Property);

		return _possibleConstructorReturn$53(this, (Property.__proto__ || Object.getPrototypeOf(Property)).apply(this, arguments));
	}

	_createClass$60(Property, [{
		key: 'reassignPath',
		value: function reassignPath(path$$1, options) {
			var _this2 = this;

			if (this.kind === 'get') {
				path$$1.length > 0 && this.value.forEachReturnExpressionWhenCalledAtPath([], this._accessorCallOptions, function (innerOptions) {
					return function (node) {
						return node.reassignPath(path$$1, innerOptions.addAssignedReturnExpressionAtPath(path$$1, _this2));
					};
				}, options);
			} else if (this.kind !== 'set') {
				this.value.reassignPath(path$$1, options);
			}
		}
	}, {
		key: 'forEachReturnExpressionWhenCalledAtPath',
		value: function forEachReturnExpressionWhenCalledAtPath(path$$1, callOptions, callback, options) {
			if (this.kind === 'get') {
				this.value.forEachReturnExpressionWhenCalledAtPath([], this._accessorCallOptions, function (innerOptions) {
					return function (node) {
						return node.forEachReturnExpressionWhenCalledAtPath(path$$1, callOptions, callback, innerOptions);
					};
				}, options);
			} else {
				this.value.forEachReturnExpressionWhenCalledAtPath(path$$1, callOptions, callback, options);
			}
		}
	}, {
		key: 'hasEffects',
		value: function hasEffects(options) {
			return this.key.hasEffects(options) || this.value.hasEffects(options);
		}
	}, {
		key: 'hasEffectsWhenAccessedAtPath',
		value: function hasEffectsWhenAccessedAtPath(path$$1, options) {
			var _this3 = this;

			if (this.kind === 'get') {
				return this.value.hasEffectsWhenCalledAtPath([], this._accessorCallOptions, options.getHasEffectsWhenCalledOptions()) || !options.hasReturnExpressionBeenAccessedAtPath(path$$1, this) && this.value.someReturnExpressionWhenCalledAtPath([], this._accessorCallOptions, function (innerOptions) {
					return function (node) {
						return node.hasEffectsWhenAccessedAtPath(path$$1, innerOptions.addAccessedReturnExpressionAtPath(path$$1, _this3));
					};
				}, options);
			}
			return this.value.hasEffectsWhenAccessedAtPath(path$$1, options);
		}
	}, {
		key: 'hasEffectsWhenAssignedAtPath',
		value: function hasEffectsWhenAssignedAtPath(path$$1, options) {
			var _this4 = this;

			if (this.kind === 'get') {
				return path$$1.length === 0 || this.value.someReturnExpressionWhenCalledAtPath([], this._accessorCallOptions, function (innerOptions) {
					return function (node) {
						return node.hasEffectsWhenAssignedAtPath(path$$1, innerOptions.addAssignedReturnExpressionAtPath(path$$1, _this4));
					};
				}, options);
			}
			if (this.kind === 'set') {
				return path$$1.length > 0 || this.value.hasEffectsWhenCalledAtPath([], this._accessorCallOptions, options.getHasEffectsWhenCalledOptions());
			}
			return this.value.hasEffectsWhenAssignedAtPath(path$$1, options);
		}
	}, {
		key: 'hasEffectsWhenCalledAtPath',
		value: function hasEffectsWhenCalledAtPath(path$$1, callOptions, options) {
			var _this5 = this;

			if (this.kind === 'get') {
				return this.value.hasEffectsWhenCalledAtPath([], this._accessorCallOptions, options.getHasEffectsWhenCalledOptions()) || !options.hasReturnExpressionBeenCalledAtPath(path$$1, this) && this.value.someReturnExpressionWhenCalledAtPath([], this._accessorCallOptions, function (innerOptions) {
					return function (node) {
						return node.hasEffectsWhenCalledAtPath(path$$1, callOptions, innerOptions.addCalledReturnExpressionAtPath(path$$1, _this5));
					};
				}, options);
			}
			return this.value.hasEffectsWhenCalledAtPath(path$$1, callOptions, options);
		}
	}, {
		key: 'initialiseAndDeclare',
		value: function initialiseAndDeclare(parentScope, kind) {
			this.initialiseScope(parentScope);
			this.initialiseNode(parentScope);
			this.key.initialise(parentScope);
			this.value.initialiseAndDeclare(parentScope, kind, UNKNOWN_ASSIGNMENT);
		}
	}, {
		key: 'initialiseNode',
		value: function initialiseNode() {
			this._accessorCallOptions = CallOptions.create({ withNew: false, caller: this });
		}
	}, {
		key: 'render',
		value: function render(code, es) {
			if (!this.shorthand) {
				this.key.render(code, es);
			}
			this.value.render(code, es);
		}
	}, {
		key: 'someReturnExpressionWhenCalledAtPath',
		value: function someReturnExpressionWhenCalledAtPath(path$$1, callOptions, predicateFunction, options) {
			if (this.kind === 'get') {
				return this.value.hasEffectsWhenCalledAtPath([], this._accessorCallOptions, options.getHasEffectsWhenCalledOptions()) || this.value.someReturnExpressionWhenCalledAtPath([], this._accessorCallOptions, function (innerOptions) {
					return function (node) {
						return node.someReturnExpressionWhenCalledAtPath(path$$1, callOptions, predicateFunction, innerOptions);
					};
				}, options);
			}
			return this.value.someReturnExpressionWhenCalledAtPath(path$$1, callOptions, predicateFunction, options);
		}
	}]);

	return Property;
}(Node);

var _createClass$61 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$62(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$54(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$54(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var RestElement = function (_Node) {
	_inherits$54(RestElement, _Node);

	function RestElement() {
		_classCallCheck$62(this, RestElement);

		return _possibleConstructorReturn$54(this, (RestElement.__proto__ || Object.getPrototypeOf(RestElement)).apply(this, arguments));
	}

	_createClass$61(RestElement, [{
		key: 'reassignPath',
		value: function reassignPath(path$$1, options) {
			path$$1.length === 0 && this.argument.reassignPath([], options);
		}
	}, {
		key: 'hasEffectsWhenAssignedAtPath',
		value: function hasEffectsWhenAssignedAtPath(path$$1, options) {
			return path$$1.length > 0 || this.argument.hasEffectsWhenAssignedAtPath([], options);
		}
	}, {
		key: 'initialiseAndDeclare',
		value: function initialiseAndDeclare(parentScope, kind) {
			this.initialiseScope(parentScope);
			this.argument.initialiseAndDeclare(parentScope, kind, UNKNOWN_ASSIGNMENT);
		}
	}]);

	return RestElement;
}(Node);

var _createClass$62 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$22 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$63(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$55(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$55(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ReturnStatement = function (_Statement) {
	_inherits$55(ReturnStatement, _Statement);

	function ReturnStatement() {
		_classCallCheck$63(this, ReturnStatement);

		return _possibleConstructorReturn$55(this, (ReturnStatement.__proto__ || Object.getPrototypeOf(ReturnStatement)).apply(this, arguments));
	}

	_createClass$62(ReturnStatement, [{
		key: 'hasEffects',
		value: function hasEffects(options) {
			return _get$22(ReturnStatement.prototype.__proto__ || Object.getPrototypeOf(ReturnStatement.prototype), 'hasEffects', this).call(this, options) || !options.ignoreReturnAwaitYield();
		}
	}, {
		key: 'initialiseNode',
		value: function initialiseNode() {
			this.scope.addReturnExpression(this.argument || UNDEFINED_ASSIGNMENT);
		}
	}]);

	return ReturnStatement;
}(Statement);

var _createClass$63 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$23 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$64(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$56(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$56(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var SequenceExpression = function (_Node) {
	_inherits$56(SequenceExpression, _Node);

	function SequenceExpression() {
		_classCallCheck$64(this, SequenceExpression);

		return _possibleConstructorReturn$56(this, (SequenceExpression.__proto__ || Object.getPrototypeOf(SequenceExpression)).apply(this, arguments));
	}

	_createClass$63(SequenceExpression, [{
		key: 'getValue',
		value: function getValue() {
			return this.expressions[this.expressions.length - 1].getValue();
		}
	}, {
		key: 'hasEffects',
		value: function hasEffects(options) {
			return this.expressions.some(function (expression) {
				return expression.hasEffects(options);
			});
		}
	}, {
		key: 'includeInBundle',
		value: function includeInBundle() {
			var addedNewNodes = !this.included;
			this.included = true;
			if (this.expressions[this.expressions.length - 1].includeInBundle()) {
				addedNewNodes = true;
			}
			this.expressions.forEach(function (node) {
				if (node.shouldBeIncluded()) {
					if (node.includeInBundle()) {
						addedNewNodes = true;
					}
				}
			});
			return addedNewNodes;
		}
	}, {
		key: 'render',
		value: function render(code, es) {
			if (!this.module.bundle.treeshake) {
				_get$23(SequenceExpression.prototype.__proto__ || Object.getPrototypeOf(SequenceExpression.prototype), 'render', this).call(this, code, es);
			} else {
				var last = this.expressions[this.expressions.length - 1];
				last.render(code, es);

				if (this.parent.type === 'CallExpression' && last.type === 'MemberExpression' && this.expressions.length > 1) {
					this.expressions[0].included = true;
				}

				var included = this.expressions.slice(0, this.expressions.length - 1).filter(function (expression) {
					return expression.included;
				});
				if (included.length === 0) {
					code.remove(this.start, last.start);
					code.remove(last.end, this.end);
				} else {
					var previousEnd = this.start;
					var _iteratorNormalCompletion = true;
					var _didIteratorError = false;
					var _iteratorError = undefined;

					try {
						for (var _iterator = included[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
							var expression = _step.value;

							expression.render(code, es);
							code.remove(previousEnd, expression.start);
							code.appendLeft(expression.end, ', ');
							previousEnd = expression.end;
						}
					} catch (err) {
						_didIteratorError = true;
						_iteratorError = err;
					} finally {
						try {
							if (!_iteratorNormalCompletion && _iterator.return) {
								_iterator.return();
							}
						} finally {
							if (_didIteratorError) {
								throw _iteratorError;
							}
						}
					}

					code.remove(previousEnd, last.start);
					code.remove(last.end, this.end);
				}
			}
		}
	}]);

	return SequenceExpression;
}(Node);

var _createClass$64 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$65(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$57(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$57(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var SwitchCase = function (_Node) {
	_inherits$57(SwitchCase, _Node);

	function SwitchCase() {
		_classCallCheck$65(this, SwitchCase);

		return _possibleConstructorReturn$57(this, (SwitchCase.__proto__ || Object.getPrototypeOf(SwitchCase)).apply(this, arguments));
	}

	_createClass$64(SwitchCase, [{
		key: 'includeInBundle',
		value: function includeInBundle() {
			var addedNewNodes = !this.included;
			this.included = true;
			if (this.test && this.test.includeInBundle()) {
				addedNewNodes = true;
			}
			this.consequent.forEach(function (node) {
				if (node.shouldBeIncluded()) {
					if (node.includeInBundle()) {
						addedNewNodes = true;
					}
				}
			});
			return addedNewNodes;
		}
	}]);

	return SwitchCase;
}(Node);

var _createClass$65 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$24 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$66(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$58(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$58(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var SwitchStatement = function (_Statement) {
	_inherits$58(SwitchStatement, _Statement);

	function SwitchStatement() {
		_classCallCheck$66(this, SwitchStatement);

		return _possibleConstructorReturn$58(this, (SwitchStatement.__proto__ || Object.getPrototypeOf(SwitchStatement)).apply(this, arguments));
	}

	_createClass$65(SwitchStatement, [{
		key: 'hasEffects',
		value: function hasEffects(options) {
			return _get$24(SwitchStatement.prototype.__proto__ || Object.getPrototypeOf(SwitchStatement.prototype), 'hasEffects', this).call(this, options.setIgnoreBreakStatements());
		}
	}, {
		key: 'initialiseScope',
		value: function initialiseScope(parentScope) {
			this.scope = new BlockScope({ parent: parentScope });
		}
	}]);

	return SwitchStatement;
}(Statement);

var _createClass$66 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$25 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$67(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$59(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$59(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var TaggedTemplateExpression = function (_Node) {
	_inherits$59(TaggedTemplateExpression, _Node);

	function TaggedTemplateExpression() {
		_classCallCheck$67(this, TaggedTemplateExpression);

		return _possibleConstructorReturn$59(this, (TaggedTemplateExpression.__proto__ || Object.getPrototypeOf(TaggedTemplateExpression)).apply(this, arguments));
	}

	_createClass$66(TaggedTemplateExpression, [{
		key: 'bindNode',
		value: function bindNode() {
			if (this.tag.type === 'Identifier') {
				var variable = this.scope.findVariable(this.tag.name);

				if (variable.isNamespace) {
					this.module.error({
						code: 'CANNOT_CALL_NAMESPACE',
						message: 'Cannot call a namespace (\'' + this.tag.name + '\')'
					}, this.start);
				}

				if (this.tag.name === 'eval' && variable.isGlobal) {
					this.module.warn({
						code: 'EVAL',
						message: 'Use of eval is strongly discouraged, as it poses security risks and may cause issues with minification',
						url: 'https://github.com/rollup/rollup/wiki/Troubleshooting#avoiding-eval'
					}, this.start);
				}
			}
		}
	}, {
		key: 'hasEffects',
		value: function hasEffects(options) {
			return _get$25(TaggedTemplateExpression.prototype.__proto__ || Object.getPrototypeOf(TaggedTemplateExpression.prototype), 'hasEffects', this).call(this, options) || this.tag.hasEffectsWhenCalledAtPath([], this._callOptions, options.getHasEffectsWhenCalledOptions());
		}
	}, {
		key: 'initialiseNode',
		value: function initialiseNode() {
			this._callOptions = CallOptions.create({ withNew: false, caller: this });
		}
	}]);

	return TaggedTemplateExpression;
}(Node);

var _createClass$67 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$68(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$60(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$60(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var TemplateElement = function (_Node) {
	_inherits$60(TemplateElement, _Node);

	function TemplateElement() {
		_classCallCheck$68(this, TemplateElement);

		return _possibleConstructorReturn$60(this, (TemplateElement.__proto__ || Object.getPrototypeOf(TemplateElement)).apply(this, arguments));
	}

	_createClass$67(TemplateElement, [{
		key: 'hasEffects',
		value: function hasEffects() {
			return false;
		}
	}]);

	return TemplateElement;
}(Node);

var _createClass$68 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$26 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$69(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$61(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$61(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var TemplateLiteral = function (_Node) {
	_inherits$61(TemplateLiteral, _Node);

	function TemplateLiteral() {
		_classCallCheck$69(this, TemplateLiteral);

		return _possibleConstructorReturn$61(this, (TemplateLiteral.__proto__ || Object.getPrototypeOf(TemplateLiteral)).apply(this, arguments));
	}

	_createClass$68(TemplateLiteral, [{
		key: 'render',
		value: function render(code, es) {
			code.indentExclusionRanges.push([this.start, this.end]);
			_get$26(TemplateLiteral.prototype.__proto__ || Object.getPrototypeOf(TemplateLiteral.prototype), 'render', this).call(this, code, es);
		}
	}]);

	return TemplateLiteral;
}(Node);

var _createClass$69 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$70(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$62(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$62(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ThisExpression = function (_Node) {
	_inherits$62(ThisExpression, _Node);

	function ThisExpression() {
		_classCallCheck$70(this, ThisExpression);

		return _possibleConstructorReturn$62(this, (ThisExpression.__proto__ || Object.getPrototypeOf(ThisExpression)).apply(this, arguments));
	}

	_createClass$69(ThisExpression, [{
		key: 'initialiseNode',
		value: function initialiseNode() {
			var lexicalBoundary = this.scope.findLexicalBoundary();

			if (lexicalBoundary.isModuleScope) {
				this.alias = this.module.context;
				if (this.alias === 'undefined') {
					this.module.warn({
						code: 'THIS_IS_UNDEFINED',
						message: 'The \'this\' keyword is equivalent to \'undefined\' at the top level of an ES module, and has been rewritten',
						url: 'https://github.com/rollup/rollup/wiki/Troubleshooting#this-is-undefined'
					}, this.start);
				}
			}
		}
	}, {
		key: 'bindNode',
		value: function bindNode() {
			this.variable = this.scope.findVariable('this');
		}
	}, {
		key: 'hasEffectsWhenAccessedAtPath',
		value: function hasEffectsWhenAccessedAtPath(path$$1, options) {
			return path$$1.length > 0 && this.variable.hasEffectsWhenAccessedAtPath(path$$1, options);
		}
	}, {
		key: 'hasEffectsWhenAssignedAtPath',
		value: function hasEffectsWhenAssignedAtPath(path$$1, options) {
			return this.variable.hasEffectsWhenAssignedAtPath(path$$1, options);
		}
	}, {
		key: 'render',
		value: function render(code) {
			if (this.alias) {
				code.overwrite(this.start, this.end, this.alias, { storeName: true, contentOnly: false });
			}
		}
	}]);

	return ThisExpression;
}(Node);

var _createClass$70 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$71(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$63(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$63(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ThrowStatement = function (_Node) {
	_inherits$63(ThrowStatement, _Node);

	function ThrowStatement() {
		_classCallCheck$71(this, ThrowStatement);

		return _possibleConstructorReturn$63(this, (ThrowStatement.__proto__ || Object.getPrototypeOf(ThrowStatement)).apply(this, arguments));
	}

	_createClass$70(ThrowStatement, [{
		key: 'hasEffects',
		value: function hasEffects() {
			return true;
		}
	}]);

	return ThrowStatement;
}(Node);

var _createClass$71 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _typeof2 = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

function _classCallCheck$72(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$64(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$64(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var operators$1 = {
	'-': function _(value) {
		return -value;
	},
	'+': function _(value) {
		return +value;
	},
	'!': function _(value) {
		return !value;
	},
	'~': function _(value) {
		return ~value;
	},
	typeof: function _typeof(value) {
		return typeof value === 'undefined' ? 'undefined' : _typeof2(value);
	},
	void: function _void() {
		return undefined;
	},
	delete: function _delete() {
		return UNKNOWN_VALUE;
	}
};

var UnaryExpression = function (_Node) {
	_inherits$64(UnaryExpression, _Node);

	function UnaryExpression() {
		_classCallCheck$72(this, UnaryExpression);

		return _possibleConstructorReturn$64(this, (UnaryExpression.__proto__ || Object.getPrototypeOf(UnaryExpression)).apply(this, arguments));
	}

	_createClass$71(UnaryExpression, [{
		key: 'bindNode',
		value: function bindNode() {
			if (this.operator === 'delete') {
				this.argument.reassignPath([], ExecutionPathOptions.create());
			}
		}
	}, {
		key: 'getValue',
		value: function getValue() {
			var argumentValue = this.argument.getValue();
			if (argumentValue === UNKNOWN_VALUE) return UNKNOWN_VALUE;

			return operators$1[this.operator](argumentValue);
		}
	}, {
		key: 'hasEffects',
		value: function hasEffects(options) {
			return this.argument.hasEffects(options) || this.operator === 'delete' && this.argument.hasEffectsWhenAssignedAtPath([], options);
		}
	}, {
		key: 'hasEffectsWhenAccessedAtPath',
		value: function hasEffectsWhenAccessedAtPath(path$$1) {
			if (this.operator === 'void') {
				return path$$1.length > 0;
			}
			return path$$1.length > 1;
		}
	}, {
		key: 'initialiseNode',
		value: function initialiseNode() {
			this.value = this.getValue();
		}
	}]);

	return UnaryExpression;
}(Node);

var _createClass$72 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$73(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$65(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$65(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var UpdateExpression = function (_Node) {
	_inherits$65(UpdateExpression, _Node);

	function UpdateExpression() {
		_classCallCheck$73(this, UpdateExpression);

		return _possibleConstructorReturn$65(this, (UpdateExpression.__proto__ || Object.getPrototypeOf(UpdateExpression)).apply(this, arguments));
	}

	_createClass$72(UpdateExpression, [{
		key: 'bindNode',
		value: function bindNode() {
			disallowIllegalReassignment(this.scope, this.argument);
			this.argument.reassignPath([], ExecutionPathOptions.create());
			if (this.argument.type === 'Identifier') {
				var variable = this.scope.findVariable(this.argument.name);
				variable.isReassigned = true;
			}
		}
	}, {
		key: 'hasEffects',
		value: function hasEffects(options) {
			return this.argument.hasEffects(options) || this.argument.hasEffectsWhenAssignedAtPath([], options);
		}
	}, {
		key: 'hasEffectsWhenAccessedAtPath',
		value: function hasEffectsWhenAccessedAtPath(path$$1) {
			return path$$1.length > 1;
		}
	}]);

	return UpdateExpression;
}(Node);

var _createClass$73 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$27 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$74(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$66(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$66(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var VariableDeclarator = function (_Node) {
	_inherits$66(VariableDeclarator, _Node);

	function VariableDeclarator() {
		_classCallCheck$74(this, VariableDeclarator);

		return _possibleConstructorReturn$66(this, (VariableDeclarator.__proto__ || Object.getPrototypeOf(VariableDeclarator)).apply(this, arguments));
	}

	_createClass$73(VariableDeclarator, [{
		key: 'reassignPath',
		value: function reassignPath(path$$1, options) {
			this.id.reassignPath(path$$1, options);
		}
	}, {
		key: 'initialiseDeclarator',
		value: function initialiseDeclarator(parentScope, kind) {
			this.initialiseScope(parentScope);
			this.init && this.init.initialise(this.scope);
			this.id.initialiseAndDeclare(this.scope, kind, this.init);
		}

		// TODO Deleting this does not break any tests. Find meaningful test or delete.

	}, {
		key: 'render',
		value: function render(code, es) {
			var _this2 = this;

			extractNames(this.id).forEach(function (name) {
				var variable = _this2.scope.findVariable(name);

				if (!es && variable.exportName && variable.isReassigned) {
					if (_this2.init) {
						code.overwrite(_this2.start, _this2.id.end, variable.getName(es));
					} else if (_this2.module.bundle.treeshake) {
						code.remove(_this2.start, _this2.end);
					}
				}
			});

			_get$27(VariableDeclarator.prototype.__proto__ || Object.getPrototypeOf(VariableDeclarator.prototype), 'render', this).call(this, code, es);
		}
	}]);

	return VariableDeclarator;
}(Node);

var _createClass$74 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$75(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$67(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$67(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function getSeparator(code, start) {
	var c = start;

	while (c > 0 && code[c - 1] !== '\n') {
		c -= 1;
		if (code[c] === ';' || code[c] === '{') return '; ';
	}

	var lineStart = code.slice(c, start).match(/^\s*/)[0];

	return ';\n' + lineStart;
}

var forStatement = /^For(?:Of|In)?Statement/;

var VariableDeclaration = function (_Node) {
	_inherits$67(VariableDeclaration, _Node);

	function VariableDeclaration() {
		_classCallCheck$75(this, VariableDeclaration);

		return _possibleConstructorReturn$67(this, (VariableDeclaration.__proto__ || Object.getPrototypeOf(VariableDeclaration)).apply(this, arguments));
	}

	_createClass$74(VariableDeclaration, [{
		key: 'reassignPath',
		value: function reassignPath() {
			this.eachChild(function (child) {
				return child.reassignPath([], ExecutionPathOptions.create());
			});
		}
	}, {
		key: 'hasEffectsWhenAssignedAtPath',
		value: function hasEffectsWhenAssignedAtPath() {
			return false;
		}
	}, {
		key: 'includeWithAllDeclarations',
		value: function includeWithAllDeclarations() {
			var addedNewNodes = !this.included;
			this.included = true;
			this.declarations.forEach(function (declarator) {
				if (declarator.includeInBundle()) {
					addedNewNodes = true;
				}
			});
			return addedNewNodes;
		}
	}, {
		key: 'includeInBundle',
		value: function includeInBundle() {
			var addedNewNodes = !this.included;
			this.included = true;
			this.declarations.forEach(function (declarator) {
				if (declarator.shouldBeIncluded()) {
					if (declarator.includeInBundle()) {
						addedNewNodes = true;
					}
				}
			});
			return addedNewNodes;
		}
	}, {
		key: 'initialiseChildren',
		value: function initialiseChildren() {
			var _this2 = this;

			this.declarations.forEach(function (child) {
				return child.initialiseDeclarator(_this2.scope, _this2.kind);
			});
		}
	}, {
		key: 'render',
		value: function render(code, es) {
			var _this3 = this;

			var treeshake = this.module.bundle.treeshake;

			var shouldSeparate = false;
			var separator = void 0;

			if (this.scope.isModuleScope && !forStatement.test(this.parent.type)) {
				shouldSeparate = true;
				separator = getSeparator(this.module.code, this.start);
			}

			var c = this.start;
			var empty = true;

			var _loop = function _loop(i) {
				var declarator = _this3.declarations[i];

				var prefix = empty ? '' : separator; // TODO indentation

				if (declarator.id.type === 'Identifier') {
					var variable = _this3.scope.findVariable(declarator.id.name);
					var isExportedAndReassigned = !es && variable.exportName && variable.isReassigned;

					if (isExportedAndReassigned) {
						if (declarator.init) {
							if (shouldSeparate) code.overwrite(c, declarator.start, prefix);
							c = declarator.end;
							empty = false;
						}
					} else if (!treeshake || variable.included) {
						if (shouldSeparate) code.overwrite(c, declarator.start, '' + prefix + _this3.kind + ' '); // TODO indentation
						c = declarator.end;
						empty = false;
					}
				} else {
					var exportAssignments = [];
					var isIncluded = false;

					extractNames(declarator.id).forEach(function (name) {
						var variable = _this3.scope.findVariable(name);
						var isExportedAndReassigned = !es && variable.exportName && variable.isReassigned;

						if (isExportedAndReassigned) {
							// code.overwrite( c, declarator.start, prefix );
							// c = declarator.end;
							// empty = false;
							exportAssignments.push('TODO');
						} else if (declarator.included) {
							isIncluded = true;
						}
					});

					if (!treeshake || isIncluded) {
						if (shouldSeparate) code.overwrite(c, declarator.start, '' + prefix + _this3.kind + ' '); // TODO indentation
						c = declarator.end;
						empty = false;
					}

					if (exportAssignments.length) {
						throw new Error('TODO');
					}
				}

				declarator.render(code, es);
			};

			for (var i = 0; i < this.declarations.length; i += 1) {
				_loop(i);
			}

			if (treeshake && empty) {
				code.remove(this.leadingCommentStart || this.start, this.next || this.end);
			} else {
				// always include a semi-colon (https://github.com/rollup/rollup/pull/1013),
				// unless it's a var declaration in a loop head
				var needsSemicolon = !forStatement.test(this.parent.type) || this === this.parent.body;

				if (this.end > c) {
					code.overwrite(c, this.end, needsSemicolon ? ';' : '');
				} else if (needsSemicolon) {
					this.insertSemicolon(code);
				}
			}
		}
	}]);

	return VariableDeclaration;
}(Node);

var _createClass$75 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$76(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$68(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$68(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var WhileStatement = function (_Statement) {
	_inherits$68(WhileStatement, _Statement);

	function WhileStatement() {
		_classCallCheck$76(this, WhileStatement);

		return _possibleConstructorReturn$68(this, (WhileStatement.__proto__ || Object.getPrototypeOf(WhileStatement)).apply(this, arguments));
	}

	_createClass$75(WhileStatement, [{
		key: 'hasEffects',
		value: function hasEffects(options) {
			return this.test.hasEffects(options) || this.body.hasEffects(options.setIgnoreBreakStatements());
		}
	}]);

	return WhileStatement;
}(Statement);

var _createClass$76 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$28 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$77(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$69(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$69(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var YieldExpression = function (_Node) {
	_inherits$69(YieldExpression, _Node);

	function YieldExpression() {
		_classCallCheck$77(this, YieldExpression);

		return _possibleConstructorReturn$69(this, (YieldExpression.__proto__ || Object.getPrototypeOf(YieldExpression)).apply(this, arguments));
	}

	_createClass$76(YieldExpression, [{
		key: 'hasEffects',
		value: function hasEffects(options) {
			return _get$28(YieldExpression.prototype.__proto__ || Object.getPrototypeOf(YieldExpression.prototype), 'hasEffects', this).call(this, options) || !options.ignoreReturnAwaitYield();
		}
	}]);

	return YieldExpression;
}(Node);

var nodes = {
	ArrayExpression: ArrayExpression,
	ArrayPattern: ArrayPattern,
	ArrowFunctionExpression: ArrowFunctionExpression,
	AssignmentExpression: AssignmentExpression,
	AssignmentPattern: AssignmentPattern,
	AwaitExpression: AwaitExpression,
	BinaryExpression: BinaryExpression,
	BlockStatement: BlockStatement,
	BreakStatement: BreakStatement,
	CallExpression: CallExpression,
	CatchClause: CatchClause,
	ClassBody: ClassBody,
	ClassDeclaration: ClassDeclaration,
	ClassExpression: ClassExpression,
	ConditionalExpression: ConditionalExpression,
	DoWhileStatement: DoWhileStatement,
	EmptyStatement: EmptyStatement,
	ExportAllDeclaration: ExportAllDeclaration,
	ExportDefaultDeclaration: ExportDefaultDeclaration,
	ExportNamedDeclaration: ExportNamedDeclaration,
	ExpressionStatement: ExpressionStatement,
	ForStatement: ForStatement,
	ForInStatement: ForInStatement,
	ForOfStatement: ForOfStatement,
	FunctionDeclaration: FunctionDeclaration,
	FunctionExpression: FunctionExpression,
	Identifier: Identifier,
	IfStatement: IfStatement,
	ImportDeclaration: ImportDeclaration,
	LabeledStatement: LabeledStatement,
	Literal: Literal,
	LogicalExpression: LogicalExpression,
	MemberExpression: MemberExpression,
	MethodDefinition: MethodDefinition,
	NewExpression: NewExpression,
	ObjectExpression: ObjectExpression,
	ObjectPattern: ObjectPattern,
	Property: Property,
	RestElement: RestElement,
	ReturnStatement: ReturnStatement,
	SequenceExpression: SequenceExpression,
	SwitchCase: SwitchCase,
	SwitchStatement: SwitchStatement,
	TaggedTemplateExpression: TaggedTemplateExpression,
	TemplateElement: TemplateElement,
	TemplateLiteral: TemplateLiteral,
	ThisExpression: ThisExpression,
	ThrowStatement: ThrowStatement,
	TryStatement: Statement,
	UnaryExpression: UnaryExpression,
	UpdateExpression: UpdateExpression,
	VariableDeclarator: VariableDeclarator,
	VariableDeclaration: VariableDeclaration,
	WhileStatement: WhileStatement,
	YieldExpression: YieldExpression
};

var _createClass$77 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$78(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$70(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$70(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var UnknownNode = function (_Node) {
	_inherits$70(UnknownNode, _Node);

	function UnknownNode() {
		_classCallCheck$78(this, UnknownNode);

		return _possibleConstructorReturn$70(this, (UnknownNode.__proto__ || Object.getPrototypeOf(UnknownNode)).apply(this, arguments));
	}

	_createClass$77(UnknownNode, [{
		key: 'hasEffects',
		value: function hasEffects() {
			return true;
		}
	}]);

	return UnknownNode;
}(Node);

var keys$1 = {
	Program: ['body'],
	Literal: []
};

var _typeof$2 = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var newline = /\n/;

function enhance(ast, module, comments) {
	enhanceNode(ast, module, module, module.magicString);

	var comment = comments.shift();

	var _iteratorNormalCompletion = true;
	var _didIteratorError = false;
	var _iteratorError = undefined;

	try {
		for (var _iterator = ast.body[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
			var node = _step.value;

			if (comment && comment.start < node.start) {
				node.leadingCommentStart = comment.start;
			}

			while (comment && comment.end < node.end) {
				comment = comments.shift();
			} // if the next comment is on the same line as the end of the node,
			// treat is as a trailing comment
			if (comment && !newline.test(module.code.slice(node.end, comment.start))) {
				node.trailingCommentEnd = comment.end; // TODO is node.trailingCommentEnd used anywhere?
				comment = comments.shift();
			}

			node.initialise(module.scope);
		}
	} catch (err) {
		_didIteratorError = true;
		_iteratorError = err;
	} finally {
		try {
			if (!_iteratorNormalCompletion && _iterator.return) {
				_iterator.return();
			}
		} finally {
			if (_didIteratorError) {
				throw _iteratorError;
			}
		}
	}
}

function enhanceNode(raw, parent, module, code) {
	if (!raw) return;

	if ('length' in raw) {
		for (var i = 0; i < raw.length; i += 1) {
			enhanceNode(raw[i], parent, module, code);
		}

		return;
	}

	// with e.g. shorthand properties, key and value are
	// the same node. We don't want to enhance an object twice
	if (raw.__enhanced) return;
	raw.__enhanced = true;

	if (!keys$1[raw.type]) {
		keys$1[raw.type] = Object.keys(raw).filter(function (key) {
			return _typeof$2(raw[key]) === 'object';
		});
	}

	raw.parent = parent;
	raw.module = module;
	raw.keys = keys$1[raw.type];

	code.addSourcemapLocation(raw.start);
	code.addSourcemapLocation(raw.end);

	var _iteratorNormalCompletion2 = true;
	var _didIteratorError2 = false;
	var _iteratorError2 = undefined;

	try {
		for (var _iterator2 = keys$1[raw.type][Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
			var key = _step2.value;

			enhanceNode(raw[key], raw, module, code);
		}
	} catch (err) {
		_didIteratorError2 = true;
		_iteratorError2 = err;
	} finally {
		try {
			if (!_iteratorNormalCompletion2 && _iterator2.return) {
				_iterator2.return();
			}
		} finally {
			if (_didIteratorError2) {
				throw _iteratorError2;
			}
		}
	}

	var type = nodes[raw.type] || UnknownNode;
	raw.__proto__ = type.prototype;
}

var _typeof$3 = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

function clone(node) {
	if (!node) return node;
	if ((typeof node === 'undefined' ? 'undefined' : _typeof$3(node)) !== 'object') return node;

	if (Array.isArray(node)) {
		var _cloned = new Array(node.length);
		for (var i = 0; i < node.length; i += 1) {
			_cloned[i] = clone(node[i]);
		}return _cloned;
	}

	var cloned = {};
	for (var key in node) {
		cloned[key] = clone(node[key]);
	}

	return cloned;
}

var _createClass$78 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$29 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$79(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$71(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$71(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ModuleScope = function (_Scope) {
	_inherits$71(ModuleScope, _Scope);

	function ModuleScope(module) {
		_classCallCheck$79(this, ModuleScope);

		var _this = _possibleConstructorReturn$71(this, (ModuleScope.__proto__ || Object.getPrototypeOf(ModuleScope)).call(this, {
			isModuleScope: true,
			parent: module.bundle.scope
		}));

		_this.module = module;
		_this.variables.this = new LocalVariable('this', null, UNDEFINED_ASSIGNMENT);
		return _this;
	}

	_createClass$78(ModuleScope, [{
		key: 'deshadow',
		value: function deshadow(names) {
			var _this2 = this;

			names = new Set(names);

			forOwn(this.module.imports, function (specifier) {
				if (specifier.module.isExternal) return;

				var addDeclaration = function addDeclaration(declaration) {
					if (declaration.isNamespace && !declaration.isExternal) {
						declaration.module.getExports().forEach(function (name) {
							addDeclaration(declaration.module.traceExport(name));
						});
					}

					names.add(declaration.name);
				};

				specifier.module.getExports().forEach(function (name) {
					addDeclaration(specifier.module.traceExport(name));
				});

				if (specifier.name !== '*') {
					var declaration = specifier.module.traceExport(specifier.name);
					if (!declaration) {
						_this2.module.warn({
							code: 'NON_EXISTENT_EXPORT',
							name: specifier.name,
							source: specifier.module.id,
							message: 'Non-existent export \'' + specifier.name + '\' is imported from ' + relativeId(specifier.module.id)
						}, specifier.specifier.start);
						return;
					}

					var name = declaration.getName(true);
					if (name !== specifier.name) {
						names.add(declaration.getName(true));
					}

					if (specifier.name !== 'default' && specifier.specifier.imported.name !== specifier.specifier.local.name) {
						names.add(specifier.specifier.imported.name);
					}
				}
			});

			_get$29(ModuleScope.prototype.__proto__ || Object.getPrototypeOf(ModuleScope.prototype), 'deshadow', this).call(this, names);
		}
	}, {
		key: 'findLexicalBoundary',
		value: function findLexicalBoundary() {
			return this;
		}
	}, {
		key: 'findVariable',
		value: function findVariable(name) {
			if (this.variables[name]) {
				return this.variables[name];
			}

			return this.module.trace(name) || this.parent.findVariable(name);
		}
	}]);

	return ModuleScope;
}(Scope);

var _createClass$1 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$1(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var setModuleDynamicImportsReturnBinding = wrapDynamicImportPlugin(acorn);

function tryParse(module, acornOptions) {
	try {
		return acorn.parse(module.code, assign({
			ecmaVersion: 8,
			sourceType: 'module',
			onComment: function onComment(block, text, start, end) {
				return module.comments.push({ block: block, text: text, start: start, end: end });
			},
			preserveParens: false
		}, acornOptions));
	} catch (err) {
		module.error({
			code: 'PARSE_ERROR',
			message: err.message.replace(/ \(\d+:\d+\)$/, '')
		}, err.pos);
	}
}

function includeFully(node) {
	node.included = true;
	if (node.variable && !node.variable.included) {
		node.variable.includeVariable();
	}
	node.eachChild(includeFully);
}

var Module = function () {
	function Module(_ref) {
		var _this = this;

		var id = _ref.id,
		    code = _ref.code,
		    originalCode = _ref.originalCode,
		    originalSourcemap = _ref.originalSourcemap,
		    ast = _ref.ast,
		    sourcemapChain = _ref.sourcemapChain,
		    resolvedIds = _ref.resolvedIds,
		    resolvedExternalIds = _ref.resolvedExternalIds,
		    bundle = _ref.bundle;

		_classCallCheck$1(this, Module);

		this.code = code;
		this.id = id;
		this.bundle = bundle;
		this.originalCode = originalCode;
		this.originalSourcemap = originalSourcemap;
		this.sourcemapChain = sourcemapChain;

		this.comments = [];
		this.dynamicImports = [];

		timeStart('ast');

		if (ast) {
			// prevent mutating the provided AST, as it may be reused on
			// subsequent incremental rebuilds
			this.ast = clone(ast);
			this.astClone = ast;
		} else {
			// We bind the dynamic imports array to the plugin binding above, to get the nodes added
			// to this array during parsing itself. This is faster than having to do a separate walk.
			if (bundle.dynamicImport) setModuleDynamicImportsReturnBinding(this.dynamicImports);
			this.ast = tryParse(this, bundle.acornOptions); // TODO what happens to comments if AST is provided?
			if (bundle.dynamicImport) setModuleDynamicImportsReturnBinding(undefined);
			this.astClone = clone(this.ast);
		}

		timeEnd('ast');

		this.excludeFromSourcemap = /\0/.test(id);
		this.context = bundle.getModuleContext(id);

		// all dependencies
		this.sources = [];
		this.dependencies = [];
		this.resolvedIds = resolvedIds || blank();
		this.resolvedExternalIds = resolvedExternalIds || blank();

		// imports and exports, indexed by local name
		this.imports = blank();
		this.exports = blank();
		this.exportsAll = blank();
		this.reexports = blank();

		this.exportAllSources = [];
		this.exportAllModules = null;

		// By default, `id` is the filename. Custom resolvers and loaders
		// can change that, but it makes sense to use it for the source filename
		this.magicString = new MagicString__default(code, {
			filename: this.excludeFromSourcemap ? null : id, // don't include plugin helpers in sourcemap
			indentExclusionRanges: []
		});

		// remove existing sourceMappingURL comments
		this.comments = this.comments.filter(function (comment) {
			//only one line comment can contain source maps
			var isSourceMapComment = !comment.block && SOURCEMAPPING_URL_RE.test(comment.text);
			if (isSourceMapComment) {
				_this.magicString.remove(comment.start, comment.end);
			}
			return !isSourceMapComment;
		});

		this.declarations = blank();
		this.type = 'Module'; // TODO only necessary so that Scope knows this should be treated as a function scope... messy
		this.scope = new ModuleScope(this);

		timeStart('analyse');

		this.analyse();

		timeEnd('analyse');

		this.strongDependencies = [];
	}

	_createClass$1(Module, [{
		key: 'addExport',
		value: function addExport(node) {
			var _this2 = this;

			var source = node.source && node.source.value;

			// export { name } from './other.js'
			if (source) {
				if (!~this.sources.indexOf(source)) this.sources.push(source);

				if (node.type === 'ExportAllDeclaration') {
					// Store `export * from '...'` statements in an array of delegates.
					// When an unknown import is encountered, we see if one of them can satisfy it.
					this.exportAllSources.push(source);
				} else {
					node.specifiers.forEach(function (specifier) {
						var name = specifier.exported.name;

						if (_this2.exports[name] || _this2.reexports[name]) {
							_this2.error({
								code: 'DUPLICATE_EXPORT',
								message: 'A module cannot have multiple exports with the same name (\'' + name + '\')'
							}, specifier.start);
						}

						_this2.reexports[name] = {
							start: specifier.start,
							source: source,
							localName: specifier.local.name,
							module: null // filled in later
						};
					});
				}
			}

			// export default function foo () {}
			// export default foo;
			// export default 42;
			else if (node.type === 'ExportDefaultDeclaration') {
					var identifier = node.declaration.id && node.declaration.id.name || node.declaration.name;

					if (this.exports.default) {
						this.error({
							code: 'DUPLICATE_EXPORT',
							message: 'A module can only have one default export'
						}, node.start);
					}

					this.exports.default = {
						localName: 'default',
						identifier: identifier
					};
				}

				// export var { foo, bar } = ...
				// export var foo = 42;
				// export var a = 1, b = 2, c = 3;
				// export function foo () {}
				else if (node.declaration) {
						var declaration = node.declaration;

						if (declaration.type === 'VariableDeclaration') {
							declaration.declarations.forEach(function (decl) {
								extractNames(decl.id).forEach(function (localName) {
									_this2.exports[localName] = { localName: localName };
								});
							});
						} else {
							// export function foo () {}
							var localName = declaration.id.name;
							this.exports[localName] = { localName: localName };
						}
					}

					// export { foo, bar, baz }
					else {
							node.specifiers.forEach(function (specifier) {
								var localName = specifier.local.name;
								var exportedName = specifier.exported.name;

								if (_this2.exports[exportedName] || _this2.reexports[exportedName]) {
									_this2.error({
										code: 'DUPLICATE_EXPORT',
										message: 'A module cannot have multiple exports with the same name (\'' + exportedName + '\')'
									}, specifier.start);
								}

								_this2.exports[exportedName] = { localName: localName };
							});
						}
		}
	}, {
		key: 'addImport',
		value: function addImport(node) {
			var _this3 = this;

			var source = node.source.value;

			if (!~this.sources.indexOf(source)) this.sources.push(source);

			node.specifiers.forEach(function (specifier) {
				var localName = specifier.local.name;

				if (_this3.imports[localName]) {
					_this3.error({
						code: 'DUPLICATE_IMPORT',
						message: 'Duplicated import \'' + localName + '\''
					}, specifier.start);
				}

				var isDefault = specifier.type === 'ImportDefaultSpecifier';
				var isNamespace = specifier.type === 'ImportNamespaceSpecifier';

				var name = isDefault ? 'default' : isNamespace ? '*' : specifier.imported.name;
				_this3.imports[localName] = { source: source, specifier: specifier, name: name, module: null };
			});
		}
	}, {
		key: 'analyse',
		value: function analyse() {
			enhance(this.ast, this, this.comments);

			// discover this module's imports and exports
			var lastNode = void 0;

			var _iteratorNormalCompletion = true;
			var _didIteratorError = false;
			var _iteratorError = undefined;

			try {
				for (var _iterator = this.ast.body[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
					var node = _step.value;

					if (node.isImportDeclaration) {
						this.addImport(node);
					} else if (node.isExportDeclaration) {
						this.addExport(node);
					}

					if (lastNode) lastNode.next = node.leadingCommentStart || node.start;
					lastNode = node;
				}
			} catch (err) {
				_didIteratorError = true;
				_iteratorError = err;
			} finally {
				try {
					if (!_iteratorNormalCompletion && _iterator.return) {
						_iterator.return();
					}
				} finally {
					if (_didIteratorError) {
						throw _iteratorError;
					}
				}
			}
		}
	}, {
		key: 'basename',
		value: function basename$$1() {
			var base = path.basename(this.id);
			var ext = path.extname(this.id);

			return makeLegal(ext ? base.slice(0, -ext.length) : base);
		}
	}, {
		key: 'bindImportSpecifiers',
		value: function bindImportSpecifiers() {
			var _this4 = this;

			[this.imports, this.reexports].forEach(function (specifiers) {
				keys(specifiers).forEach(function (name) {
					var specifier = specifiers[name];

					var id = _this4.resolvedIds[specifier.source] || _this4.resolvedExternalIds[specifier.source];
					specifier.module = _this4.bundle.moduleById.get(id);
				});
			});

			this.exportAllModules = this.exportAllSources.map(function (source) {
				var id = _this4.resolvedIds[source] || _this4.resolvedExternalIds[source];
				return _this4.bundle.moduleById.get(id);
			});

			this.sources.forEach(function (source) {
				var id = _this4.resolvedIds[source];

				if (id) {
					var module = _this4.bundle.moduleById.get(id);
					_this4.dependencies.push(module);
				}
			});
		}
	}, {
		key: 'bindReferences',
		value: function bindReferences() {
			var _iteratorNormalCompletion2 = true;
			var _didIteratorError2 = false;
			var _iteratorError2 = undefined;

			try {
				for (var _iterator2 = this.ast.body[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
					var node = _step2.value;

					node.bind();
				}
			} catch (err) {
				_didIteratorError2 = true;
				_iteratorError2 = err;
			} finally {
				try {
					if (!_iteratorNormalCompletion2 && _iterator2.return) {
						_iterator2.return();
					}
				} finally {
					if (_didIteratorError2) {
						throw _iteratorError2;
					}
				}
			}
		}
	}, {
		key: 'getOriginalLocation',
		value: function getOriginalLocation(sourcemapChain, line, column) {
			var location = {
				line: line,
				column: column
			};
			var filteredSourcemapChain = sourcemapChain.filter(function (sourcemap) {
				return sourcemap.mappings;
			}).map(function (sourcemap) {
				var encodedSourcemap = sourcemap;
				if (sourcemap.mappings) {
					encodedSourcemap.mappings = sourcemapCodec.encode(encodedSourcemap.mappings);
				}
				return encodedSourcemap;
			});
			while (filteredSourcemapChain.length > 0) {
				var sourcemap = filteredSourcemapChain.pop();
				var smc = new sourceMap.SourceMapConsumer(sourcemap);
				location = smc.originalPositionFor({
					line: location.line,
					column: location.column
				});
			}
			return location;
		}
	}, {
		key: 'error',
		value: function error$$1(props, pos) {
			if (pos !== undefined) {
				props.pos = pos;

				var _locate = locateCharacter.locate(this.code, pos, { offsetLine: 1 }),
				    line = _locate.line,
				    column = _locate.column; // TODO trace sourcemaps

				var location = this.getOriginalLocation(this.sourcemapChain, line, column);

				props.loc = { file: this.id, line: location.line, column: location.column };
				props.frame = getCodeFrame(this.originalCode, location.line, location.column);
			}

			error(props);
		}
	}, {
		key: 'getExports',
		value: function getExports() {
			return keys(this.exports);
		}
	}, {
		key: 'getReexports',
		value: function getReexports() {
			var reexports = blank();

			keys(this.reexports).forEach(function (name) {
				reexports[name] = true;
			});

			this.exportAllModules.forEach(function (module) {
				if (module.isExternal) {
					reexports['*' + module.id] = true;
					return;
				}

				module.getExports().concat(module.getReexports()).forEach(function (name) {
					if (name !== 'default') reexports[name] = true;
				});
			});

			return keys(reexports);
		}
	}, {
		key: 'includeAllInBundle',
		value: function includeAllInBundle() {
			this.ast.body.forEach(includeFully);
		}
	}, {
		key: 'includeInBundle',
		value: function includeInBundle() {
			var addedNewNodes = false;
			this.ast.body.forEach(function (node) {
				if (node.shouldBeIncluded()) {
					if (node.includeInBundle()) {
						addedNewNodes = true;
					}
				}
			});
			return addedNewNodes;
		}
	}, {
		key: 'processDynamicImports',
		value: function processDynamicImports(resolveDynamicImport) {
			var _this5 = this;

			return Promise.all(this.dynamicImports.map(function (node) {
				var importArgument = node.parent.arguments[0];
				var dynamicImportSpecifier = void 0;
				if (importArgument.type === 'TemplateLiteral') {
					if (importArgument.expressions.length === 0 && importArgument.quasis.length === 1) {
						dynamicImportSpecifier = importArgument.quasis[0].value.cooked;
					}
				} else if (importArgument.type === 'Literal') {
					if (typeof importArgument.value === 'string') {
						dynamicImportSpecifier = importArgument.value;
					}
				} else {
					dynamicImportSpecifier = importArgument;
				}

				return Promise.resolve(resolveDynamicImport(dynamicImportSpecifier, _this5.id)).then(function (replacement) {
					if (!replacement) return;

					// string specifier -> direct resolution
					if (typeof dynamicImportSpecifier === 'string') {
						// if we have the module, inline as Promise.resolve(namespace)
						// ensuring that we create a namespace import of it as well
						var replacementModule = _this5.bundle.moduleById.get(replacement);
						if (replacementModule && !replacementModule.isExternal) {
							var namespace = replacementModule.namespace();
							namespace.includeVariable();
							var identifierName = namespace.getName(true);
							_this5.magicString.overwrite(node.parent.start, node.parent.end, 'Promise.resolve( ' + identifierName + ' )');
							// otherwise treat as an external dynamic import resolution
						} else {
							_this5.magicString.overwrite(importArgument.start, importArgument.end, '"' + replacement + '"');
						}
						// AST Node -> source replacement
					} else {
						_this5.magicString.overwrite(importArgument.start, importArgument.end, replacement);
					}
				});
			}));
		}
	}, {
		key: 'namespace',
		value: function namespace() {
			if (!this.declarations['*']) {
				this.declarations['*'] = new NamespaceVariable(this);
			}

			return this.declarations['*'];
		}
	}, {
		key: 'render',
		value: function render(es, legacy, freeze) {
			var magicString = this.magicString.clone();

			var _iteratorNormalCompletion3 = true;
			var _didIteratorError3 = false;
			var _iteratorError3 = undefined;

			try {
				for (var _iterator3 = this.ast.body[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
					var node = _step3.value;

					node.render(magicString, es);
				}
			} catch (err) {
				_didIteratorError3 = true;
				_iteratorError3 = err;
			} finally {
				try {
					if (!_iteratorNormalCompletion3 && _iterator3.return) {
						_iterator3.return();
					}
				} finally {
					if (_didIteratorError3) {
						throw _iteratorError3;
					}
				}
			}

			if (this.namespace().needsNamespaceBlock) {
				magicString.append('\n\n' + this.namespace().renderBlock(es, legacy, freeze, '\t')); // TODO use correct indentation
			}

			return magicString.trim();
		}
	}, {
		key: 'toJSON',
		value: function toJSON() {
			return {
				id: this.id,
				dependencies: this.dependencies.map(function (module) {
					return module.id;
				}),
				code: this.code,
				originalCode: this.originalCode,
				originalSourcemap: this.originalSourcemap,
				ast: this.astClone,
				sourcemapChain: this.sourcemapChain,
				resolvedIds: this.resolvedIds,
				resolvedExternalIds: this.resolvedExternalIds
			};
		}
	}, {
		key: 'trace',
		value: function trace(name) {
			// TODO this is slightly circular
			if (name in this.scope.variables) {
				return this.scope.variables[name];
			}

			if (name in this.imports) {
				var importDeclaration = this.imports[name];
				var otherModule = importDeclaration.module;

				if (importDeclaration.name === '*' && !otherModule.isExternal) {
					return otherModule.namespace();
				}

				var declaration = otherModule.traceExport(importDeclaration.name);

				if (!declaration) {
					this.error({
						code: 'MISSING_EXPORT',
						message: '\'' + importDeclaration.name + '\' is not exported by ' + relativeId(otherModule.id),
						url: 'https://github.com/rollup/rollup/wiki/Troubleshooting#name-is-not-exported-by-module'
					}, importDeclaration.specifier.start);
				}

				return declaration;
			}

			return null;
		}
	}, {
		key: 'traceExport',
		value: function traceExport(name) {
			// export * from 'external'
			if (name[0] === '*') {
				var module = this.bundle.moduleById.get(name.slice(1));
				return module.traceExport('*');
			}

			// export { foo } from './other.js'
			var reexportDeclaration = this.reexports[name];
			if (reexportDeclaration) {
				var declaration = reexportDeclaration.module.traceExport(reexportDeclaration.localName);

				if (!declaration) {
					this.error({
						code: 'MISSING_EXPORT',
						message: '\'' + reexportDeclaration.localName + '\' is not exported by ' + relativeId(reexportDeclaration.module.id),
						url: 'https://github.com/rollup/rollup/wiki/Troubleshooting#name-is-not-exported-by-module'
					}, reexportDeclaration.start);
				}

				return declaration;
			}

			var exportDeclaration = this.exports[name];
			if (exportDeclaration) {
				var _name = exportDeclaration.localName;
				var _declaration = this.trace(_name);

				return _declaration || this.bundle.scope.findVariable(_name);
			}

			if (name === 'default') return;

			for (var i = 0; i < this.exportAllModules.length; i += 1) {
				var _module = this.exportAllModules[i];
				var _declaration2 = _module.traceExport(name);

				if (_declaration2) return _declaration2;
			}
		}
	}, {
		key: 'warn',
		value: function warn(warning, pos) {
			if (pos !== undefined) {
				warning.pos = pos;

				var _locate2 = locateCharacter.locate(this.code, pos, { offsetLine: 1 }),
				    line = _locate2.line,
				    column = _locate2.column; // TODO trace sourcemaps

				warning.loc = { file: this.id, line: line, column: column };
				warning.frame = getCodeFrame(this.code, line, column);
			}

			warning.id = this.id;
			this.bundle.warn(warning);
		}
	}]);

	return Module;
}();

var _createClass$80 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$81(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$72(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$72(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ExternalVariable = function (_Variable) {
	_inherits$72(ExternalVariable, _Variable);

	function ExternalVariable(module, name) {
		_classCallCheck$81(this, ExternalVariable);

		var _this = _possibleConstructorReturn$72(this, (ExternalVariable.__proto__ || Object.getPrototypeOf(ExternalVariable)).call(this, name));

		_this.module = module;
		_this.safeName = null;
		_this.isExternal = true;
		_this.isNamespace = name === '*';
		return _this;
	}

	_createClass$80(ExternalVariable, [{
		key: 'addReference',
		value: function addReference(identifier) {
			if (this.name === 'default' || this.name === '*') {
				this.module.suggestName(identifier.name);
			}
		}
	}, {
		key: 'getName',
		value: function getName(es) {
			if (this.name === '*') {
				return this.module.name;
			}

			if (this.name === 'default') {
				return this.module.exportsNamespace || !es && this.module.exportsNames ? this.module.name + '__default' : this.module.name;
			}

			return es ? this.safeName : this.module.name + '.' + this.name;
		}
	}, {
		key: 'includeVariable',
		value: function includeVariable() {
			if (this.included) {
				return false;
			}
			this.included = true;
			this.module.used = true;
			return true;
		}
	}, {
		key: 'setSafeName',
		value: function setSafeName(name) {
			this.safeName = name;
		}
	}]);

	return ExternalVariable;
}(Variable);

var _createClass$79 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$80(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var ExternalModule = function () {
	function ExternalModule(id) {
		_classCallCheck$80(this, ExternalModule);

		this.id = id;

		var parts = id.split(/[\\/]/);
		this.name = makeLegal(parts.pop());

		this.nameSuggestions = blank();
		this.mostCommonSuggestion = 0;

		this.isExternal = true;
		this.used = false;
		this.declarations = blank();

		this.exportsNames = false;
	}

	_createClass$79(ExternalModule, [{
		key: 'suggestName',
		value: function suggestName(name) {
			if (!this.nameSuggestions[name]) this.nameSuggestions[name] = 0;
			this.nameSuggestions[name] += 1;

			if (this.nameSuggestions[name] > this.mostCommonSuggestion) {
				this.mostCommonSuggestion = this.nameSuggestions[name];
				this.name = name;
			}
		}
	}, {
		key: 'traceExport',
		value: function traceExport(name) {
			if (name !== 'default' && name !== '*') this.exportsNames = true;
			if (name === '*') this.exportsNamespace = true;

			return this.declarations[name] || (this.declarations[name] = new ExternalVariable(this, name));
		}
	}]);

	return ExternalModule;
}();

function getInteropBlock(bundle, options) {
	return bundle.externalModules.map(function (module) {
		if (!module.declarations.default || options.interop === false) return null;

		if (module.exportsNamespace) {
			return bundle.varOrConst + ' ' + module.name + '__default = ' + module.name + '[\'default\'];';
		}

		if (module.exportsNames) {
			return bundle.varOrConst + ' ' + module.name + '__default = \'default\' in ' + module.name + ' ? ' + module.name + '[\'default\'] : ' + module.name + ';';
		}

		return module.name + ' = ' + module.name + ' && ' + module.name + '.hasOwnProperty(\'default\') ? ' + module.name + '[\'default\'] : ' + module.name + ';';
	}).filter(Boolean).join('\n');
}

function getExportBlock(bundle, exportMode) {
	var mechanism = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'return';

	var entryModule = bundle.entryModule;

	if (exportMode === 'default') {
		return mechanism + ' ' + entryModule.traceExport('default').getName(false) + ';';
	}

	var exports = entryModule.getExports().concat(entryModule.getReexports()).map(function (name) {
		if (name[0] === '*') {
			// export all from external
			var id = name.slice(1);
			var module = bundle.moduleById.get(id);

			return 'Object.keys(' + module.name + ').forEach(function (key) { exports[key] = ' + module.name + '[key]; });';
		}

		var prop = name === 'default' ? '[\'default\']' : '.' + name;
		var declaration = entryModule.traceExport(name);

		var lhs = 'exports' + prop;
		var rhs = declaration ? declaration.getName(false) : name; // exporting a global

		// prevent `exports.count = exports.count`
		if (lhs === rhs) return null;

		return lhs + ' = ' + rhs + ';';
	});

	return exports.filter(Boolean).join('\n');
}

var esModuleExport = "Object.defineProperty(exports, '__esModule', { value: true });";

var builtins$1 = {
	process: true,
	events: true,
	stream: true,
	util: true,
	path: true,
	buffer: true,
	querystring: true,
	url: true,
	string_decoder: true,
	punycode: true,
	http: true,
	https: true,
	os: true,
	assert: true,
	constants: true,
	timers: true,
	console: true,
	vm: true,
	zlib: true,
	tty: true,
	domain: true
};

// Creating a browser bundle that depends on Node.js built-in modules ('util'). You might need to include https://www.npmjs.com/package/rollup-plugin-node-builtins

function warnOnBuiltins(bundle) {
	var externalBuiltins = bundle.externalModules.filter(function (mod) {
		return mod.id in builtins$1;
	}).map(function (mod) {
		return mod.id;
	});

	if (!externalBuiltins.length) return;

	var detail = externalBuiltins.length === 1 ? 'module (\'' + externalBuiltins[0] + '\')' : 'modules (' + externalBuiltins.slice(0, -1).map(function (name) {
		return '\'' + name + '\'';
	}).join(', ') + ' and \'' + externalBuiltins.slice(-1) + '\')';

	bundle.warn({
		code: 'MISSING_NODE_BUILTINS',
		modules: externalBuiltins,
		message: 'Creating a browser bundle that depends on Node.js built-in ' + detail + '. You might need to include https://www.npmjs.com/package/rollup-plugin-node-builtins'
	});
}

function amd(bundle, magicString, _ref, options) {
	var exportMode = _ref.exportMode,
	    getPath = _ref.getPath,
	    indentString = _ref.indentString,
	    intro = _ref.intro,
	    outro = _ref.outro;

	warnOnBuiltins(bundle);
	var deps = bundle.externalModules.map(function (m) {
		return '\'' + getPath(m.id) + '\'';
	});
	var args = bundle.externalModules.map(function (m) {
		return m.name;
	});

	if (exportMode === 'named') {
		args.unshift('exports');
		deps.unshift('\'exports\'');
	}

	var amdOptions = options.amd || {};

	var params = (amdOptions.id ? '\'' + amdOptions.id + '\', ' : '') + (deps.length ? '[' + deps.join(', ') + '], ' : '');

	var useStrict = options.strict !== false ? ' \'use strict\';' : '';
	var define = amdOptions.define || 'define';
	var wrapperStart = define + '(' + params + 'function (' + args.join(', ') + ') {' + useStrict + '\n\n';

	// var foo__default = 'default' in foo ? foo['default'] : foo;
	var interopBlock = getInteropBlock(bundle, options);
	if (interopBlock) magicString.prepend(interopBlock + '\n\n');

	if (intro) magicString.prepend(intro);

	var exportBlock = getExportBlock(bundle, exportMode);
	if (exportBlock) magicString.append('\n\n' + exportBlock);
	if (exportMode === 'named' && options.legacy !== true) magicString.append('\n\n' + esModuleExport);
	if (outro) magicString.append(outro);

	return magicString.indent(indentString).append('\n\n});').prepend(wrapperStart);
}

function cjs(bundle, magicString, _ref, options) {
	var exportMode = _ref.exportMode,
	    getPath = _ref.getPath,
	    intro = _ref.intro,
	    outro = _ref.outro;

	intro = (options.strict === false ? intro : '\'use strict\';\n\n' + intro) + (exportMode === 'named' && options.legacy !== true ? esModuleExport + '\n\n' : '');

	var needsInterop = false;

	var varOrConst = bundle.varOrConst;
	var interop = options.interop !== false;

	// TODO handle empty imports, once they're supported
	var importBlock = bundle.externalModules.map(function (module) {
		if (interop && module.declarations.default) {
			if (module.exportsNamespace) {
				return varOrConst + ' ' + module.name + ' = require(\'' + getPath(module.id) + '\');' + ('\n' + varOrConst + ' ' + module.name + '__default = ' + module.name + '[\'default\'];');
			}

			needsInterop = true;

			if (module.exportsNames) {
				return varOrConst + ' ' + module.name + ' = require(\'' + getPath(module.id) + '\');' + ('\n' + varOrConst + ' ' + module.name + '__default = _interopDefault(' + module.name + ');');
			}

			return varOrConst + ' ' + module.name + ' = _interopDefault(require(\'' + getPath(module.id) + '\'));';
		} else {
			var includedDeclarations = Object.keys(module.declarations).filter(function (name) {
				return module.declarations[name].included;
			});

			var needsVar = includedDeclarations.length || module.reexported;

			return needsVar ? varOrConst + ' ' + module.name + ' = require(\'' + getPath(module.id) + '\');' : 'require(\'' + getPath(module.id) + '\');';
		}
	}).join('\n');

	if (needsInterop) {
		intro += 'function _interopDefault (ex) { return (ex && (typeof ex === \'object\') && \'default\' in ex) ? ex[\'default\'] : ex; }\n\n';
	}

	if (importBlock) {
		intro += importBlock + '\n\n';
	}

	magicString.prepend(intro);

	var exportBlock = getExportBlock(bundle, exportMode, 'module.exports =');
	if (exportBlock) magicString.append('\n\n' + exportBlock);
	if (outro) magicString.append(outro);

	return magicString;
}

function notDefault(name) {
	return name !== 'default';
}

function es(bundle, magicString, _ref) {
	var getPath = _ref.getPath,
	    intro = _ref.intro,
	    outro = _ref.outro;

	var importBlock = bundle.externalModules.map(function (module) {
		var specifiers = [];
		var specifiersList = [specifiers];
		var importedNames = keys(module.declarations).filter(function (name) {
			return name !== '*' && name !== 'default';
		}).filter(function (name) {
			return module.declarations[name].included;
		}).map(function (name) {
			if (name[0] === '*') {
				return '* as ' + module.name;
			}

			var declaration = module.declarations[name];

			if (declaration.name === declaration.safeName) return declaration.name;
			return declaration.name + ' as ' + declaration.safeName;
		}).filter(Boolean);

		if (module.declarations.default) {
			if (module.exportsNamespace) {
				specifiersList.push([module.name + '__default']);
			} else {
				specifiers.push(module.name);
			}
		}

		var namespaceSpecifier = module.declarations['*'] && module.declarations['*'].included ? '* as ' + module.name : null; // TODO prevent unnecessary namespace import, e.g form/external-imports
		var namedSpecifier = importedNames.length ? '{ ' + importedNames.sort().join(', ') + ' }' : null;

		if (namespaceSpecifier && namedSpecifier) {
			// Namespace and named specifiers cannot be combined.
			specifiersList.push([namespaceSpecifier]);
			specifiers.push(namedSpecifier);
		} else if (namedSpecifier) {
			specifiers.push(namedSpecifier);
		} else if (namespaceSpecifier) {
			specifiers.push(namespaceSpecifier);
		}

		return specifiersList.map(function (specifiers) {
			if (specifiers.length) {
				return 'import ' + specifiers.join(', ') + ' from \'' + getPath(module.id) + '\';';
			}

			return module.reexported ? null : 'import \'' + getPath(module.id) + '\';';
		}).filter(Boolean).join('\n');
	}).join('\n');

	if (importBlock) intro += importBlock + '\n\n';
	if (intro) magicString.prepend(intro);

	var module = bundle.entryModule;

	var exportInternalSpecifiers = [];
	var exportExternalSpecifiers = new Map();
	var exportAllDeclarations = [];

	module.getExports().filter(notDefault).forEach(function (name) {
		var declaration = module.traceExport(name);
		var rendered = declaration.getName(true);
		exportInternalSpecifiers.push(rendered === name ? name : rendered + ' as ' + name);
	});

	module.getReexports().forEach(function (name) {
		var declaration = module.traceExport(name);

		if (declaration.isExternal) {
			if (name[0] === '*') {
				// export * from 'external'
				exportAllDeclarations.push('export * from \'' + name.slice(1) + '\';');
			} else {
				if (!exportExternalSpecifiers.has(declaration.module.id)) exportExternalSpecifiers.set(declaration.module.id, []);
				var _rendered = declaration.getName(true);
				exportExternalSpecifiers.get(declaration.module.id).push(_rendered === name ? name : _rendered + ' as ' + name);
			}

			return;
		}

		var rendered = declaration.getName(true);
		exportInternalSpecifiers.push(rendered === name ? name : rendered + ' as ' + name);
	});

	var exportBlock = [];
	if (exportInternalSpecifiers.length) exportBlock.push('export { ' + exportInternalSpecifiers.join(', ') + ' };');
	if (module.exports.default) exportBlock.push('export default ' + module.traceExport('default').getName(true) + ';');
	if (exportAllDeclarations.length) exportBlock.push(exportAllDeclarations.join('\n'));
	if (exportExternalSpecifiers.size) {
		exportExternalSpecifiers.forEach(function (specifiers, id) {
			exportBlock.push('export { ' + specifiers.join(', ') + ' } from \'' + id + '\';');
		});
	}

	if (exportBlock.length) magicString.append('\n\n' + exportBlock.join('\n').trim());

	if (outro) magicString.append(outro);

	return magicString.trim();
}

function getGlobalNameMaker(globals, bundle) {
	var fallback = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;

	var fn = typeof globals === 'function' ? globals : function (id) {
		return globals[id];
	};

	return function (module) {
		var name = fn(module.id);
		if (name) return name;

		if (Object.keys(module.declarations).length > 0) {
			bundle.warn({
				code: 'MISSING_GLOBAL_NAME',
				source: module.id,
				guess: module.name,
				message: 'No name was provided for external module \'' + module.id + '\' in options.globals \u2013 guessing \'' + module.name + '\''
			});

			return module.name;
		}

		return fallback;
	};
}

// Generate strings which dereference dotted properties, but use array notation `['prop-deref']`
// if the property name isn't trivial
var shouldUseDot = /^[a-zA-Z$_][a-zA-Z0-9$_]*$/;

function property(prop) {
	return shouldUseDot.test(prop) ? '.' + prop : '[\'' + prop + '\']';
}

function keypath(keypath) {
	return keypath.split('.').map(property).join('');
}

function trimEmptyImports(modules) {
	var i = modules.length;

	while (i--) {
		var module = modules[i];
		if (Object.keys(module.declarations).length > 0) {
			return modules.slice(0, i + 1);
		}
	}

	return [];
}

function setupNamespace(name, root, forAssignment, globals) {
	var parts = name.split('.');
	if (globals) {
		parts[0] = (typeof globals === 'function' ? globals(parts[0]) : globals[parts[0]]) || parts[0];
	}

	var last = parts.pop();

	var acc = root;
	if (forAssignment) {
		return parts.map(function (part) {
			return acc += property(part), acc + ' = ' + acc + ' || {}';
		}).concat('' + acc + property(last)).join(', ');
	} else {
		return parts.map(function (part) {
			return acc += property(part), acc + ' = ' + acc + ' || {};';
		}).join('\n') + '\n';
	}
}

var thisProp = function thisProp(name) {
	return 'this' + keypath(name);
};

function iife(bundle, magicString, _ref, options) {
	var exportMode = _ref.exportMode,
	    indentString = _ref.indentString,
	    intro = _ref.intro,
	    outro = _ref.outro;

	var globalNameMaker = getGlobalNameMaker(options.globals || blank(), bundle, 'null');

	var extend = options.extend,
	    name = options.name;

	var isNamespaced = name && name.indexOf('.') !== -1;
	var possibleVariableAssignment = !extend && !isNamespaced;

	if (name && possibleVariableAssignment && !isLegal(name)) {
		error({
			code: 'ILLEGAL_IDENTIFIER_AS_NAME',
			message: 'Given name (' + name + ') is not legal JS identifier. If you need this you can try --extend option'
		});
	}

	warnOnBuiltins(bundle);

	var external = trimEmptyImports(bundle.externalModules);
	var dependencies = external.map(globalNameMaker);
	var args = external.map(function (m) {
		return m.name;
	});

	if (exportMode !== 'none' && !name) {
		error({
			code: 'INVALID_OPTION',
			message: 'You must supply options.name for IIFE bundles'
		});
	}

	if (extend) {
		dependencies.unshift('(' + thisProp(name) + ' = ' + thisProp(name) + ' || {})');
		args.unshift('exports');
	} else if (exportMode === 'named') {
		dependencies.unshift('{}');
		args.unshift('exports');
	}

	var useStrict = options.strict !== false ? indentString + '\'use strict\';\n\n' : '';

	var wrapperIntro = '(function (' + args + ') {\n' + useStrict;

	if (exportMode !== 'none' && !extend) {
		wrapperIntro = (isNamespaced ? thisProp(name) : bundle.varOrConst + ' ' + name) + (' = ' + wrapperIntro);
	}

	if (isNamespaced) {
		wrapperIntro = setupNamespace(name, 'this', false, options.globals) + wrapperIntro;
	}

	var wrapperOutro = '\n\n}(' + dependencies + '));';

	if (!extend && exportMode === 'named') {
		wrapperOutro = '\n\n' + indentString + 'return exports;' + wrapperOutro;
	}

	// var foo__default = 'default' in foo ? foo['default'] : foo;
	var interopBlock = getInteropBlock(bundle, options);
	if (interopBlock) magicString.prepend(interopBlock + '\n\n');

	if (intro) magicString.prepend(intro);

	var exportBlock = getExportBlock(bundle, exportMode);
	if (exportBlock) magicString.append('\n\n' + exportBlock);
	if (outro) magicString.append(outro);

	return magicString.indent(indentString).prepend(wrapperIntro).append(wrapperOutro);
}

function globalProp(name) {
	if (!name) return 'null';
	return 'global' + keypath(name);
}

function safeAccess(name) {
	var parts = name.split('.');

	var acc = 'global';
	return parts.map(function (part) {
		return acc += property(part), acc;
	}).join(' && ');
}

var wrapperOutro = '\n\n})));';

function umd(bundle, magicString, _ref, options) {
	var exportMode = _ref.exportMode,
	    getPath = _ref.getPath,
	    indentString = _ref.indentString,
	    intro = _ref.intro,
	    outro = _ref.outro;

	if (exportMode !== 'none' && !options.name) {
		error({
			code: 'INVALID_OPTION',
			message: 'You must supply options.name for UMD bundles'
		});
	}

	warnOnBuiltins(bundle);

	var globalNameMaker = getGlobalNameMaker(options.globals || blank(), bundle);

	var amdDeps = bundle.externalModules.map(function (m) {
		return '\'' + getPath(m.id) + '\'';
	});
	var cjsDeps = bundle.externalModules.map(function (m) {
		return 'require(\'' + getPath(m.id) + '\')';
	});

	var trimmed = trimEmptyImports(bundle.externalModules);
	var globalDeps = trimmed.map(function (module) {
		return globalProp(globalNameMaker(module));
	});
	var args = trimmed.map(function (m) {
		return m.name;
	});

	if (exportMode === 'named') {
		amdDeps.unshift('\'exports\'');
		cjsDeps.unshift('exports');
		globalDeps.unshift('(' + setupNamespace(options.name, 'global', true, options.globals) + ' = ' + (options.extend ? globalProp(options.name) + ' || ' : '') + '{})');

		args.unshift('exports');
	}

	var amdOptions = options.amd || {};

	var amdParams = (amdOptions.id ? '\'' + amdOptions.id + '\', ' : '') + (amdDeps.length ? '[' + amdDeps.join(', ') + '], ' : '');

	var define = amdOptions.define || 'define';

	var cjsExport = exportMode === 'default' ? 'module.exports = ' : '';
	var defaultExport = exportMode === 'default' ? setupNamespace(options.name, 'global', true, options.globals) + ' = ' : '';

	var useStrict = options.strict !== false ? ' \'use strict\';' : '';

	var globalExport = void 0;

	if (options.noConflict === true) {
		var factory = void 0;

		if (exportMode === 'default') {
			factory = 'var exports = factory(' + globalDeps + ');';
		} else if (exportMode === 'named') {
			var module = globalDeps.shift();
			factory = 'var exports = ' + module + ';\n\t\t\t\tfactory(' + ['exports'].concat(globalDeps) + ');';
		}
		globalExport = '(function() {\n\t\t\t\tvar current = ' + safeAccess(options.name) + ';\n\t\t\t\t' + factory + '\n\t\t\t\t' + globalProp(options.name) + ' = exports;\n\t\t\t\texports.noConflict = function() { ' + globalProp(options.name) + ' = current; return exports; };\n\t\t\t})()';
	} else {
		globalExport = '(' + defaultExport + 'factory(' + globalDeps + '))';
	}

	var wrapperIntro = ('(function (global, factory) {\n\t\t\ttypeof exports === \'object\' && typeof module !== \'undefined\' ? ' + cjsExport + 'factory(' + cjsDeps.join(', ') + ') :\n\t\t\ttypeof ' + define + ' === \'function\' && ' + define + '.amd ? ' + define + '(' + amdParams + 'factory) :\n\t\t\t' + globalExport + ';\n\t\t}(this, (function (' + args + ') {' + useStrict + '\n\n\t\t').replace(/^\t\t/gm, '').replace(/^\t/gm, indentString || '\t');

	// var foo__default = 'default' in foo ? foo['default'] : foo;
	var interopBlock = getInteropBlock(bundle, options);
	if (interopBlock) magicString.prepend(interopBlock + '\n\n');

	if (intro) magicString.prepend(intro);

	var exportBlock = getExportBlock(bundle, exportMode);
	if (exportBlock) magicString.append('\n\n' + exportBlock);
	if (exportMode === 'named' && options.legacy !== true) magicString.append('\n\n' + esModuleExport);
	if (outro) magicString.append(outro);

	return magicString.trim().indent(indentString).append(wrapperOutro).prepend(wrapperIntro);
}

var finalisers = { amd: amd, cjs: cjs, es: es, iife: iife, umd: umd };

function load(id) {
	return fs.readFileSync(id, 'utf-8');
}

function findFile(file) {
	try {
		var stats = fs.lstatSync(file);
		if (stats.isSymbolicLink()) return findFile(fs.realpathSync(file));
		if (stats.isFile()) {
			// check case
			var name = path.basename(file);
			var files = fs.readdirSync(path.dirname(file));

			if (~files.indexOf(name)) return file;
		}
	} catch (err) {
		// suppress
	}
}

function addJsExtensionIfNecessary(file) {
	return findFile(file) || findFile(file + '.js');
}

function resolveId(importee, importer) {
	if (typeof process === 'undefined') {
		error({
			code: 'MISSING_PROCESS',
			message: 'It looks like you\'re using Rollup in a non-Node.js environment. This means you must supply a plugin with custom resolveId and load functions',
			url: 'https://github.com/rollup/rollup/wiki/Plugins'
		});
	}

	// external modules (non-entry modules that start with neither '.' or '/')
	// are skipped at this stage.
	if (importer !== undefined && !isAbsolute(importee) && importee[0] !== '.') return null;

	// `resolve` processes paths from right to left, prepending them until an
	// absolute path is created. Absolute importees therefore shortcircuit the
	// resolve call and require no special handing on our part.
	// See https://nodejs.org/api/path.html#path_path_resolve_paths
	return addJsExtensionIfNecessary(path.resolve(importer ? path.dirname(importer) : path.resolve(), importee));
}

function makeOnwarn() {
	var warned = blank();

	return function (warning) {
		var str = warning.toString();
		if (str in warned) return;
		console.error(str); //eslint-disable-line no-console
		warned[str] = true;
	};
}

function badExports(option, keys$$1) {
	error({
		code: 'INVALID_EXPORT_OPTION',
		message: '\'' + option + '\' was specified for options.exports, but entry module has following exports: ' + keys$$1.join(', ')
	});
}

function getExportMode(bundle, _ref) {
	var exportMode = _ref.exports,
	    name = _ref.name,
	    format = _ref.format;

	var exportKeys = keys(bundle.entryModule.exports).concat(keys(bundle.entryModule.reexports)).concat(bundle.entryModule.exportAllSources); // not keys, but makes our job easier this way

	if (exportMode === 'default') {
		if (exportKeys.length !== 1 || exportKeys[0] !== 'default') {
			badExports('default', exportKeys);
		}
	} else if (exportMode === 'none' && exportKeys.length) {
		badExports('none', exportKeys);
	}

	if (!exportMode || exportMode === 'auto') {
		if (exportKeys.length === 0) {
			exportMode = 'none';
		} else if (exportKeys.length === 1 && exportKeys[0] === 'default') {
			exportMode = 'default';
		} else {
			if (bundle.entryModule.exports.default && format !== 'es') {
				bundle.warn({
					code: 'MIXED_EXPORTS',
					message: 'Using named and default exports together. Consumers of your bundle will have to use ' + (name || 'bundle') + '[\'default\'] to access the default export, which may not be what you want. Use `exports: \'named\'` to disable this warning',
					url: 'https://rollupjs.org/#exports'
				});
			}
			exportMode = 'named';
		}
	}

	if (!/(?:default|named|none)/.test(exportMode)) {
		error({
			code: 'INVALID_EXPORT_OPTION',
			message: 'options.exports must be \'default\', \'named\', \'none\', \'auto\', or left unspecified (defaults to \'auto\')'
		});
	}

	return exportMode;
}

function getIndentString(magicString, options) {
	if (options.indent === true) {
		return magicString.getIndentString();
	}

	return options.indent || '';
}

function transform(bundle, source, id, plugins) {
	var sourcemapChain = [];

	var originalSourcemap = typeof source.map === 'string' ? JSON.parse(source.map) : source.map;

	if (originalSourcemap && typeof originalSourcemap.mappings === 'string') {
		originalSourcemap.mappings = sourcemapCodec.decode(originalSourcemap.mappings);
	}

	var originalCode = source.code;
	var ast = source.ast;

	var promise = Promise.resolve(source.code);

	plugins.forEach(function (plugin) {
		if (!plugin.transform) return;

		promise = promise.then(function (previous) {
			function augment(object, pos, code) {
				if (typeof object === 'string') {
					object = { message: object };
				}

				if (object.code) object.pluginCode = object.code;
				object.code = code;

				if (pos !== undefined) {
					if (pos.line !== undefined && pos.column !== undefined) {
						var line = pos.line,
						    column = pos.column;

						object.loc = { file: id, line: line, column: column };
						object.frame = getCodeFrame(previous, line, column);
					} else {
						object.pos = pos;

						var _locate = locateCharacter.locate(previous, pos, { offsetLine: 1 }),
						    _line = _locate.line,
						    _column = _locate.column;

						object.loc = { file: id, line: _line, column: _column };
						object.frame = getCodeFrame(previous, _line, _column);
					}
				}

				object.plugin = plugin.name;
				object.id = id;

				return object;
			}

			var throwing = void 0;

			var context = {
				warn: function warn(warning, pos) {
					warning = augment(warning, pos, 'PLUGIN_WARNING');
					bundle.warn(warning);
				},

				error: function error$$1(err, pos) {
					err = augment(err, pos, 'PLUGIN_ERROR');
					throwing = true;
					error(err);
				}
			};

			var transformed = void 0;

			try {
				transformed = plugin.transform.call(context, previous, id);
			} catch (err) {
				if (!throwing) context.error(err);
				error(err);
			}

			return Promise.resolve(transformed).then(function (result) {
				if (result == null) return previous;

				if (typeof result === 'string') {
					result = {
						code: result,
						ast: undefined,
						map: undefined
					};
				}

				// `result.map` can only be a string if `result` isn't
				else if (typeof result.map === 'string') {
						result.map = JSON.parse(result.map);
					}

				if (result.map && typeof result.map.mappings === 'string') {
					result.map.mappings = sourcemapCodec.decode(result.map.mappings);
				}

				// strict null check allows 'null' maps to not be pushed to the chain, while 'undefined' gets the missing map warning
				if (result.map !== null) {
					sourcemapChain.push(result.map || { missing: true, plugin: plugin.name });
				}

				ast = result.ast;

				return result.code;
			}).catch(function (err) {
				err = augment(err, undefined, 'PLUGIN_ERROR');
				error(err);
			});
		});
	});

	return promise.then(function (code) {
		return { code: code, originalCode: originalCode, originalSourcemap: originalSourcemap, ast: ast, sourcemapChain: sourcemapChain };
	});
}

function transformBundle(code, plugins, sourcemapChain, options) {
	return plugins.reduce(function (promise, plugin) {
		if (!plugin.transformBundle) return promise;

		return promise.then(function (code) {
			return Promise.resolve().then(function () {
				return plugin.transformBundle(code, { format: options.format });
			}).then(function (result) {
				if (result == null) return code;

				if (typeof result === 'string') {
					result = {
						code: result,
						map: undefined
					};
				}

				var map = typeof result.map === 'string' ? JSON.parse(result.map) : result.map;
				if (map && typeof map.mappings === 'string') {
					map.mappings = sourcemapCodec.decode(map.mappings);
				}

				// strict null check allows 'null' maps to not be pushed to the chain, while 'undefined' gets the missing map warning
				if (map !== null) {
					sourcemapChain.push(map || { missing: true, plugin: plugin.name });
				}

				return result.code;
			}).catch(function (err) {
				error({
					code: 'BAD_BUNDLE_TRANSFORMER',
					message: 'Error transforming bundle' + (plugin.name ? ' with \'' + plugin.name + '\' plugin' : '') + ': ' + err.message,
					plugin: plugin.name
				});
			});
		});
	}, Promise.resolve(code));
}

var _createClass$81 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$82(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Source = function () {
	function Source(filename, content) {
		_classCallCheck$82(this, Source);

		this.isOriginal = true;
		this.filename = filename;
		this.content = content;
	}

	_createClass$81(Source, [{
		key: 'traceSegment',
		value: function traceSegment(line, column, name) {
			return { line: line, column: column, name: name, source: this };
		}
	}]);

	return Source;
}();

var Link = function () {
	function Link(map, sources) {
		_classCallCheck$82(this, Link);

		this.sources = sources;
		this.names = map.names;
		this.mappings = map.mappings;
	}

	_createClass$81(Link, [{
		key: 'traceMappings',
		value: function traceMappings() {
			var _this = this;

			var sources = [];
			var sourcesContent = [];
			var names = [];

			var mappings = this.mappings.map(function (line) {
				var tracedLine = [];

				line.forEach(function (segment) {
					var source = _this.sources[segment[1]];

					if (!source) return;

					var traced = source.traceSegment(segment[2], segment[3], _this.names[segment[4]]);

					if (traced) {
						var sourceIndex = null;
						var nameIndex = null;
						segment = [segment[0], null, traced.line, traced.column];

						// newer sources are more likely to be used, so search backwards.
						sourceIndex = sources.lastIndexOf(traced.source.filename);
						if (sourceIndex === -1) {
							sourceIndex = sources.length;
							sources.push(traced.source.filename);
							sourcesContent[sourceIndex] = traced.source.content;
						} else if (sourcesContent[sourceIndex] == null) {
							sourcesContent[sourceIndex] = traced.source.content;
						} else if (traced.source.content != null && sourcesContent[sourceIndex] !== traced.source.content) {
							error({
								message: 'Multiple conflicting contents for sourcemap source ' + source.filename
							});
						}

						segment[1] = sourceIndex;

						if (traced.name) {
							nameIndex = names.indexOf(traced.name);
							if (nameIndex === -1) {
								nameIndex = names.length;
								names.push(traced.name);
							}

							segment[4] = nameIndex;
						}

						tracedLine.push(segment);
					}
				});

				return tracedLine;
			});

			return { sources: sources, sourcesContent: sourcesContent, names: names, mappings: mappings };
		}
	}, {
		key: 'traceSegment',
		value: function traceSegment(line, column, name) {
			var segments = this.mappings[line];

			if (!segments) return null;

			for (var i = 0; i < segments.length; i += 1) {
				var segment = segments[i];

				if (segment[0] > column) return null;

				if (segment[0] === column) {
					var source = this.sources[segment[1]];
					if (!source) return null;

					return source.traceSegment(segment[2], segment[3], this.names[segment[4]] || name);
				}
			}

			return null;
		}
	}]);

	return Link;
}();

function collapseSourcemaps(bundle, file, map, modules, bundleSourcemapChain) {
	var moduleSources = modules.filter(function (module) {
		return !module.excludeFromSourcemap;
	}).map(function (module) {
		var sourcemapChain = module.sourcemapChain;

		var source = void 0;
		if (module.originalSourcemap == null) {
			source = new Source(module.id, module.originalCode);
		} else {
			var _sources = module.originalSourcemap.sources;
			var _sourcesContent = module.originalSourcemap.sourcesContent || [];

			if (_sources == null || _sources.length <= 1 && _sources[0] == null) {
				source = new Source(module.id, _sourcesContent[0]);
				sourcemapChain = [module.originalSourcemap].concat(sourcemapChain);
			} else {
				// TODO indiscriminately treating IDs and sources as normal paths is probably bad.
				var directory = path.dirname(module.id) || '.';
				var sourceRoot = module.originalSourcemap.sourceRoot || '.';

				var baseSources = _sources.map(function (source, i) {
					return new Source(path.resolve(directory, sourceRoot, source), _sourcesContent[i]);
				});

				source = new Link(module.originalSourcemap, baseSources);
			}
		}

		sourcemapChain.forEach(function (map) {
			if (map.missing) {
				bundle.warn({
					code: 'SOURCEMAP_BROKEN',
					plugin: map.plugin,
					message: 'Sourcemap is likely to be incorrect: a plugin' + (map.plugin ? ' (\'' + map.plugin + '\')' : '') + ' was used to transform files, but didn\'t generate a sourcemap for the transformation. Consult the plugin documentation for help',
					url: 'https://github.com/rollup/rollup/wiki/Troubleshooting#sourcemap-is-likely-to-be-incorrect'
				});

				map = {
					names: [],
					mappings: ''
				};
			}

			source = new Link(map, [source]);
		});

		return source;
	});

	var source = new Link(map, moduleSources);

	bundleSourcemapChain.forEach(function (map) {
		source = new Link(map, [source]);
	});

	var _source$traceMappings = source.traceMappings(),
	    sources = _source$traceMappings.sources,
	    sourcesContent = _source$traceMappings.sourcesContent,
	    names = _source$traceMappings.names,
	    mappings = _source$traceMappings.mappings;

	if (file) {
		var directory = path.dirname(file);
		sources = sources.map(function (source) {
			return path.relative(directory, source);
		});

		map.file = path.basename(file);
	}

	// we re-use the `map` object because it has convenient toString/toURL methods
	map.sources = sources;
	map.sourcesContent = sourcesContent;
	map.names = names;
	map.mappings = sourcemapCodec.encode(mappings);

	return map;
}

function callIfFunction(thing) {
	return typeof thing === 'function' ? thing() : thing;
}

var pureFunctions = {};

var arrayTypes = 'Array Int8Array Uint8Array Uint8ClampedArray Int16Array Uint16Array Int32Array Uint32Array Float32Array Float64Array'.split(' ');
var simdTypes = 'Int8x16 Int16x8 Int32x4 Float32x4 Float64x2'.split(' ');
var simdMethods = 'abs add and bool check div equal extractLane fromFloat32x4 fromFloat32x4Bits fromFloat64x2 fromFloat64x2Bits fromInt16x8Bits fromInt32x4 fromInt32x4Bits fromInt8x16Bits greaterThan greaterThanOrEqual lessThan lessThanOrEqual load max maxNum min minNum mul neg not notEqual or reciprocalApproximation reciprocalSqrtApproximation replaceLane select selectBits shiftLeftByScalar shiftRightArithmeticByScalar shiftRightLogicalByScalar shuffle splat sqrt store sub swizzle xor'.split(' ');
var allSimdMethods = [];
simdTypes.forEach(function (t) {
	simdMethods.forEach(function (m) {
		allSimdMethods.push('SIMD.' + t + '.' + m);
	});
});

['Array.isArray', 'Error', 'EvalError', 'InternalError', 'RangeError', 'ReferenceError', 'SyntaxError', 'TypeError', 'URIError', 'isFinite', 'isNaN', 'parseFloat', 'parseInt', 'decodeURI', 'decodeURIComponent', 'encodeURI', 'encodeURIComponent', 'escape', 'unescape', 'Object', 'Object.create', 'Object.getNotifier', 'Object.getOwn', 'Object.getOwnPropertyDescriptor', 'Object.getOwnPropertyNames', 'Object.getOwnPropertySymbols', 'Object.getPrototypeOf', 'Object.is', 'Object.isExtensible', 'Object.isFrozen', 'Object.isSealed', 'Object.keys', 'Boolean', 'Number', 'Number.isFinite', 'Number.isInteger', 'Number.isNaN', 'Number.isSafeInteger', 'Number.parseFloat', 'Number.parseInt', 'Symbol', 'Symbol.for', 'Symbol.keyFor', 'Math.abs', 'Math.acos', 'Math.acosh', 'Math.asin', 'Math.asinh', 'Math.atan', 'Math.atan2', 'Math.atanh', 'Math.cbrt', 'Math.ceil', 'Math.clz32', 'Math.cos', 'Math.cosh', 'Math.exp', 'Math.expm1', 'Math.floor', 'Math.fround', 'Math.hypot', 'Math.imul', 'Math.log', 'Math.log10', 'Math.log1p', 'Math.log2', 'Math.max', 'Math.min', 'Math.pow', 'Math.random', 'Math.round', 'Math.sign', 'Math.sin', 'Math.sinh', 'Math.sqrt', 'Math.tan', 'Math.tanh', 'Math.trunc', 'Date', 'Date.UTC', 'Date.now', 'Date.parse', 'String', 'String.fromCharCode', 'String.fromCodePoint', 'String.raw', 'RegExp', 'Map', 'Set', 'WeakMap', 'WeakSet', 'ArrayBuffer', 'ArrayBuffer.isView', 'DataView', 'Promise.all', 'Promise.race', 'Promise.resolve', 'Intl.Collator', 'Intl.Collator.supportedLocalesOf', 'Intl.DateTimeFormat', 'Intl.DateTimeFormat.supportedLocalesOf', 'Intl.NumberFormat', 'Intl.NumberFormat.supportedLocalesOf'

// TODO properties of e.g. window...
].concat(arrayTypes, arrayTypes.map(function (t) {
	return t + '.from';
}), arrayTypes.map(function (t) {
	return t + '.of';
}), simdTypes.map(function (t) {
	return 'SIMD.' + t;
}), allSimdMethods).forEach(function (name) {
	return pureFunctions[name] = true;
});

var _createClass$83 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _toConsumableArray$3(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck$84(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$74(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$74(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var GlobalVariable = function (_Variable) {
	_inherits$74(GlobalVariable, _Variable);

	function GlobalVariable(name) {
		_classCallCheck$84(this, GlobalVariable);

		var _this = _possibleConstructorReturn$74(this, (GlobalVariable.__proto__ || Object.getPrototypeOf(GlobalVariable)).call(this, name));

		_this.isExternal = true;
		_this.isGlobal = true;
		_this.isReassigned = false;
		_this.included = true;
		return _this;
	}

	_createClass$83(GlobalVariable, [{
		key: 'addReference',
		value: function addReference(identifier) {
			if (identifier.isReassignment) this.isReassigned = true;
		}
	}, {
		key: 'hasEffectsWhenAccessedAtPath',
		value: function hasEffectsWhenAccessedAtPath(path$$1) {
			// path.length == 0 can also have an effect but we postpone this for now
			return path$$1.length > 0 && !pureFunctions[[this.name].concat(_toConsumableArray$3(path$$1)).join('.')] && !pureFunctions[[this.name].concat(_toConsumableArray$3(path$$1.slice(0, -1))).join('.')] && !(path$$1.length > 1 && pureFunctions[[this.name].concat(_toConsumableArray$3(path$$1.slice(0, -2))).join('.')] && path$$1[path$$1.length - 2] === 'prototype');
		}
	}, {
		key: 'hasEffectsWhenCalledAtPath',
		value: function hasEffectsWhenCalledAtPath(path$$1) {
			return !pureFunctions[[this.name].concat(_toConsumableArray$3(path$$1)).join('.')];
		}
	}]);

	return GlobalVariable;
}(Variable);

var _createClass$82 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$83(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$73(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$73(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var BundleScope = function (_Scope) {
	_inherits$73(BundleScope, _Scope);

	function BundleScope() {
		_classCallCheck$83(this, BundleScope);

		return _possibleConstructorReturn$73(this, (BundleScope.__proto__ || Object.getPrototypeOf(BundleScope)).apply(this, arguments));
	}

	_createClass$82(BundleScope, [{
		key: 'findVariable',
		value: function findVariable(name) {
			if (!this.variables[name]) {
				this.variables[name] = new GlobalVariable(name);
			}

			return this.variables[name];
		}
	}]);

	return BundleScope;
}(Scope);

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _typeof$1 = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Bundle$1 = function () {
	function Bundle$$1(options) {
		var _this = this;

		_classCallCheck(this, Bundle$$1);

		this.cachedModules = new Map();
		if (options.cache) {
			options.cache.modules.forEach(function (module) {
				_this.cachedModules.set(module.id, module);
			});
		}

		this.plugins = ensureArray(options.plugins);

		options = this.plugins.reduce(function (acc, plugin) {
			if (plugin.options) return plugin.options(acc) || acc;
			return acc;
		}, options);

		if (!options.input) {
			throw new Error('You must supply options.input to rollup');
		}

		this.entry = options.input;
		this.entryId = null;
		this.entryModule = null;

		this.treeshake = options.treeshake !== false;
		if (this.treeshake) {
			this.treeshakingOptions = {
				propertyReadSideEffects: options.treeshake ? options.treeshake.propertyReadSideEffects !== false : true,
				pureExternalModules: options.treeshake ? options.treeshake.pureExternalModules : false
			};
			if (this.treeshakingOptions.pureExternalModules === true) {
				this.isPureExternalModule = function () {
					return true;
				};
			} else if (typeof this.treeshakingOptions.pureExternalModules === 'function') {
				this.isPureExternalModule = this.treeshakingOptions.pureExternalModules;
			} else if (Array.isArray(this.treeshakingOptions.pureExternalModules)) {
				var pureExternalModules = new Set(this.treeshakingOptions.pureExternalModules);
				this.isPureExternalModule = function (id) {
					return pureExternalModules.has(id);
				};
			} else {
				this.isPureExternalModule = function () {
					return false;
				};
			}
		} else {
			this.isPureExternalModule = function () {
				return false;
			};
		}

		this.resolveId = first([function (id, parentId) {
			return _this.isExternal(id, parentId, false) ? false : null;
		}].concat(this.plugins.map(function (plugin) {
			return plugin.resolveId;
		}).filter(Boolean)).concat(resolveId));

		var loaders = this.plugins.map(function (plugin) {
			return plugin.load;
		}).filter(Boolean);
		this.hasLoaders = loaders.length !== 0;
		this.load = first(loaders.concat(load));

		this.scope = new BundleScope();
		// TODO strictly speaking, this only applies with non-ES6, non-default-only bundles
		['module', 'exports', '_interopDefault'].forEach(function (name) {
			_this.scope.findVariable(name); // creates global variable as side-effect
		});

		this.moduleById = new Map();
		this.modules = [];
		this.externalModules = [];

		this.context = String(options.context);

		var optionsModuleContext = options.moduleContext;
		if (typeof optionsModuleContext === 'function') {
			this.getModuleContext = function (id) {
				return optionsModuleContext(id) || _this.context;
			};
		} else if ((typeof optionsModuleContext === 'undefined' ? 'undefined' : _typeof$1(optionsModuleContext)) === 'object') {
			var moduleContext = new Map();
			Object.keys(optionsModuleContext).forEach(function (key) {
				moduleContext.set(path.resolve(key), optionsModuleContext[key]);
			});
			this.getModuleContext = function (id) {
				return moduleContext.get(id) || _this.context;
			};
		} else {
			this.getModuleContext = function () {
				return _this.context;
			};
		}

		if (typeof options.external === 'function') {
			this.isExternal = options.external;
		} else {
			var ids = ensureArray(options.external);
			this.isExternal = function (id) {
				return ids.indexOf(id) !== -1;
			};
		}

		this.onwarn = options.onwarn || makeOnwarn();

		this.varOrConst = options.preferConst ? 'const' : 'var';
		this.legacy = options.legacy;
		this.acornOptions = options.acorn || {};
		this.dynamicImport = typeof options.experimentalDynamicImport === 'boolean' ? options.experimentalDynamicImport : false;

		if (this.dynamicImport) {
			this.resolveDynamicImport = first(this.plugins.map(function (plugin) {
				return plugin.resolveDynamicImport;
			}).filter(Boolean));
			this.acornOptions.plugins = this.acornOptions.plugins || {};
			this.acornOptions.plugins.dynamicImport = true;
		}
	}

	_createClass(Bundle$$1, [{
		key: 'collectAddon',
		value: function collectAddon(initialAddon, addonName) {
			var sep = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : '\n';

			return runSequence([{ pluginName: 'rollup', source: initialAddon }].concat(this.plugins.map(function (plugin, idx) {
				return {
					pluginName: plugin.name || 'Plugin at pos ' + idx,
					source: plugin[addonName]
				};
			})).map(function (addon) {
				addon.source = callIfFunction(addon.source);
				return addon;
			}).filter(function (addon) {
				return addon.source;
			}).map(function (_ref) {
				var pluginName = _ref.pluginName,
				    source = _ref.source;

				return Promise.resolve(source).catch(function (err) {
					error({
						code: 'ADDON_ERROR',
						message: 'Could not retrieve ' + addonName + '. Check configuration of ' + pluginName + '.\n\tError Message: ' + err.message
					});
				});
			})).then(function (addons) {
				return addons.filter(Boolean).join(sep);
			});
		}
	}, {
		key: 'build',
		value: function build() {
			var _this2 = this;

			// Phase 1 – discovery. We load the entry module and find which
			// modules it imports, and import those, until we have all
			// of the entry module's dependencies
			return this.resolveId(this.entry, undefined).then(function (id) {
				if (id === false) {
					error({
						code: 'UNRESOLVED_ENTRY',
						message: 'Entry module cannot be external'
					});
				}

				if (id == null) {
					error({
						code: 'UNRESOLVED_ENTRY',
						message: 'Could not resolve entry (' + _this2.entry + ')'
					});
				}

				_this2.entryId = id;
				return _this2.fetchModule(id, undefined);
			}).then(function (entryModule) {

				_this2.entryModule = entryModule;

				// Phase 2 – binding. We link references to their variables
				// to generate a complete picture of the bundle

				timeStart('phase 2');

				_this2.modules.forEach(function (module) {
					return module.bindImportSpecifiers();
				});
				_this2.modules.forEach(function (module) {
					return module.bindReferences();
				});

				// hook dynamic imports
				if (_this2.dynamicImport) return Promise.all(_this2.modules.map(function (module) {
					return module.processDynamicImports(_this2.resolveDynamicImport);
				}));
			}).then(function () {

				timeEnd('phase 2');

				// Phase 3 – marking. We include all statements that should be included

				timeStart('phase 3');

				// mark all export statements
				_this2.entryModule.getExports().forEach(function (name) {
					var variable = _this2.entryModule.traceExport(name);

					variable.exportName = name;
					variable.includeVariable();

					if (variable.isNamespace) {
						variable.needsNamespaceBlock = true;
					}
				});

				_this2.entryModule.getReexports().forEach(function (name) {
					var variable = _this2.entryModule.traceExport(name);

					if (variable.isExternal) {
						variable.reexported = variable.module.reexported = true;
					} else {
						variable.exportName = name;
						variable.includeVariable();
					}
				});

				// mark statements that should appear in the bundle
				if (_this2.treeshake) {
					var addedNewNodes = void 0;
					do {
						addedNewNodes = false;
						_this2.modules.forEach(function (module) {
							if (module.includeInBundle()) {
								addedNewNodes = true;
							}
						});
					} while (addedNewNodes);
				} else {
					// Necessary to properly replace namespace imports
					_this2.modules.forEach(function (module) {
						return module.includeAllInBundle();
					});
				}

				timeEnd('phase 3');

				// Phase 4 – final preparation. We order the modules with an
				// enhanced topological sort that accounts for cycles, then
				// ensure that names are deconflicted throughout the bundle

				timeStart('phase 4');

				// while we're here, check for unused external imports
				_this2.externalModules.forEach(function (module) {
					var unused = Object.keys(module.declarations).filter(function (name) {
						return name !== '*';
					}).filter(function (name) {
						return !module.declarations[name].included && !module.declarations[name].reexported;
					});

					if (unused.length === 0) return;

					var names = unused.length === 1 ? '\'' + unused[0] + '\' is' : unused.slice(0, -1).map(function (name) {
						return '\'' + name + '\'';
					}).join(', ') + ' and \'' + unused.slice(-1) + '\' are';

					_this2.warn({
						code: 'UNUSED_EXTERNAL_IMPORT',
						source: module.id,
						names: unused,
						message: names + ' imported from external module \'' + module.id + '\' but never used'
					});
				});

				// prune unused external imports
				_this2.externalModules = _this2.externalModules.filter(function (module) {
					return module.used || !_this2.isPureExternalModule(module.id);
				});

				_this2.orderedModules = _this2.sort();
				_this2.deconflict();

				timeEnd('phase 4');
			});
		}
	}, {
		key: 'deconflict',
		value: function deconflict() {
			var used = blank();

			// ensure no conflicts with globals
			keys(this.scope.variables).forEach(function (name) {
				return used[name] = 1;
			});

			function getSafeName(name) {
				while (used[name]) {
					name += '$' + used[name]++;
				}

				used[name] = 1;
				return name;
			}

			var toDeshadow = new Set();

			this.externalModules.forEach(function (module) {
				var safeName = getSafeName(module.name);
				toDeshadow.add(safeName);
				module.name = safeName;

				// ensure we don't shadow named external imports, if
				// we're creating an ES6 bundle
				forOwn(module.declarations, function (declaration, name) {
					var safeName = getSafeName(name);
					toDeshadow.add(safeName);
					declaration.setSafeName(safeName);
				});
			});

			this.modules.forEach(function (module) {
				forOwn(module.scope.variables, function (variable) {
					if (!variable.isDefault || !variable.hasId) {
						variable.name = getSafeName(variable.name);
					}
				});

				// deconflict reified namespaces
				var namespace = module.namespace();
				if (namespace.needsNamespaceBlock) {
					namespace.name = getSafeName(namespace.name);
				}
			});

			this.scope.deshadow(toDeshadow);
		}
	}, {
		key: 'fetchModule',
		value: function fetchModule(id, importer) {
			var _this3 = this;

			// short-circuit cycles
			if (this.moduleById.has(id)) return null;
			this.moduleById.set(id, null);

			return this.load(id).catch(function (err) {
				var msg = 'Could not load ' + id;
				if (importer) msg += ' (imported by ' + importer + ')';

				msg += ': ' + err.message;
				throw new Error(msg);
			}).then(function (source) {
				if (typeof source === 'string') return source;
				if (source && (typeof source === 'undefined' ? 'undefined' : _typeof$1(source)) === 'object' && source.code) return source;

				// TODO report which plugin failed
				error({
					code: 'BAD_LOADER',
					message: 'Error loading ' + relativeId(id) + ': plugin load hook should return a string, a { code, map } object, or nothing/null'
				});
			}).then(function (source) {
				if (typeof source === 'string') {
					source = {
						code: source,
						ast: null
					};
				}

				if (_this3.cachedModules.has(id) && _this3.cachedModules.get(id).originalCode === source.code) {
					return _this3.cachedModules.get(id);
				}

				return transform(_this3, source, id, _this3.plugins);
			}).then(function (source) {
				var code = source.code,
				    originalCode = source.originalCode,
				    originalSourcemap = source.originalSourcemap,
				    ast = source.ast,
				    sourcemapChain = source.sourcemapChain,
				    resolvedIds = source.resolvedIds;


				var module = new Module({
					id: id,
					code: code,
					originalCode: originalCode,
					originalSourcemap: originalSourcemap,
					ast: ast,
					sourcemapChain: sourcemapChain,
					resolvedIds: resolvedIds,
					bundle: _this3
				});

				_this3.modules.push(module);
				_this3.moduleById.set(id, module);

				return _this3.fetchAllDependencies(module).then(function () {
					keys(module.exports).forEach(function (name) {
						if (name !== 'default') {
							module.exportsAll[name] = module.id;
						}
					});
					module.exportAllSources.forEach(function (source) {
						var id = module.resolvedIds[source] || module.resolvedExternalIds[source];
						var exportAllModule = _this3.moduleById.get(id);
						if (exportAllModule.isExternal) return;

						keys(exportAllModule.exportsAll).forEach(function (name) {
							if (name in module.exportsAll) {
								_this3.warn({
									code: 'NAMESPACE_CONFLICT',
									reexporter: module.id,
									name: name,
									sources: [module.exportsAll[name], exportAllModule.exportsAll[name]],
									message: 'Conflicting namespaces: ' + relativeId(module.id) + ' re-exports \'' + name + '\' from both ' + relativeId(module.exportsAll[name]) + ' and ' + relativeId(exportAllModule.exportsAll[name]) + ' (will be ignored)'
								});
							} else {
								module.exportsAll[name] = exportAllModule.exportsAll[name];
							}
						});
					});
					return module;
				});
			});
		}
	}, {
		key: 'fetchAllDependencies',
		value: function fetchAllDependencies(module) {
			var _this4 = this;

			return mapSequence(module.sources, function (source) {
				var resolvedId = module.resolvedIds[source];
				return (resolvedId ? Promise.resolve(resolvedId) : _this4.resolveId(source, module.id)).then(function (resolvedId) {
					var externalId = resolvedId || (isRelative(source) ? path.resolve(module.id, '..', source) : source);
					var isExternal = _this4.isExternal(externalId, module.id, true);

					if (!resolvedId && !isExternal) {
						if (isRelative(source)) {
							error({
								code: 'UNRESOLVED_IMPORT',
								message: 'Could not resolve \'' + source + '\' from ' + relativeId(module.id)
							});
						}

						_this4.warn({
							code: 'UNRESOLVED_IMPORT',
							source: source,
							importer: relativeId(module.id),
							message: '\'' + source + '\' is imported by ' + relativeId(module.id) + ', but could not be resolved \u2013 treating it as an external dependency',
							url: 'https://github.com/rollup/rollup/wiki/Troubleshooting#treating-module-as-external-dependency'
						});
						isExternal = true;
					}

					if (isExternal) {
						module.resolvedExternalIds[source] = externalId;

						if (!_this4.moduleById.has(externalId)) {
							var _module = new ExternalModule(externalId);
							_this4.externalModules.push(_module);
							_this4.moduleById.set(externalId, _module);
						}

						var externalModule = _this4.moduleById.get(externalId);

						// add external declarations so we can detect which are never used
						Object.keys(module.imports).forEach(function (name) {
							var importDeclaration = module.imports[name];
							if (importDeclaration.source !== source) return;

							externalModule.traceExport(importDeclaration.name);
						});
					} else {
						module.resolvedIds[source] = resolvedId;
						return _this4.fetchModule(resolvedId, module.id);
					}
				});
			});
		}
	}, {
		key: 'getPathRelativeToEntryDirname',
		value: function getPathRelativeToEntryDirname(resolvedId) {
			if (isRelative(resolvedId) || isAbsolute(resolvedId)) {
				var entryDirname = path.dirname(this.entryId);
				var relativeToEntry = normalize(path.relative(entryDirname, resolvedId));

				return isRelative(relativeToEntry) ? relativeToEntry : './' + relativeToEntry;
			}

			return resolvedId;
		}
	}, {
		key: 'render',
		value: function render() {
			var _this5 = this;

			var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

			return Promise.resolve().then(function () {
				return Promise.all([_this5.collectAddon(options.banner, 'banner'), _this5.collectAddon(options.footer, 'footer'), _this5.collectAddon(options.intro, 'intro', '\n\n'), _this5.collectAddon(options.outro, 'outro', '\n\n')]);
			}).then(function (_ref2) {
				var _ref3 = _slicedToArray(_ref2, 4),
				    banner = _ref3[0],
				    footer = _ref3[1],
				    intro = _ref3[2],
				    outro = _ref3[3];

				// Determine export mode - 'default', 'named', 'none'
				var exportMode = getExportMode(_this5, options);

				var magicString = new MagicString.Bundle({ separator: '\n\n' });
				var usedModules = [];

				timeStart('render modules');

				_this5.orderedModules.forEach(function (module) {
					var source = module.render(options.format === 'es', _this5.legacy, options.freeze !== false);
					if (source.toString().length) {
						magicString.addSource(source);
						usedModules.push(module);
					}
				});

				if (!magicString.toString().trim() && _this5.entryModule.getExports().length === 0 && _this5.entryModule.getReexports().length === 0) {
					_this5.warn({
						code: 'EMPTY_BUNDLE',
						message: 'Generated an empty bundle'
					});
				}

				timeEnd('render modules');

				var indentString = getIndentString(magicString, options);

				var finalise = finalisers[options.format];
				if (!finalise) {
					error({
						code: 'INVALID_OPTION',
						message: 'Invalid format: ' + options.format + ' - valid options are ' + keys(finalisers).join(', ')
					});
				}

				timeStart('render format');

				var optionsPaths = options.paths;
				var getPath = typeof optionsPaths === 'function' ? function (id) {
					return optionsPaths(id) || _this5.getPathRelativeToEntryDirname(id);
				} : optionsPaths ? function (id) {
					return optionsPaths.hasOwnProperty(id) ? optionsPaths[id] : _this5.getPathRelativeToEntryDirname(id);
				} : function (id) {
					return _this5.getPathRelativeToEntryDirname(id);
				};

				if (intro) intro += '\n\n';
				if (outro) outro = '\n\n' + outro;

				magicString = finalise(_this5, magicString.trim(), { exportMode: exportMode, getPath: getPath, indentString: indentString, intro: intro, outro: outro }, options);

				timeEnd('render format');

				if (banner) magicString.prepend(banner + '\n');
				if (footer) magicString.append('\n' + footer);

				var prevCode = magicString.toString();
				var map = null;
				var bundleSourcemapChain = [];

				return transformBundle(prevCode, _this5.plugins, bundleSourcemapChain, options).then(function (code) {
					if (options.sourcemap) {
						timeStart('sourcemap');

						var file = options.sourcemapFile || options.file;
						if (file) file = path.resolve(typeof process !== 'undefined' ? process.cwd() : '', file);

						if (_this5.hasLoaders || find(_this5.plugins, function (plugin) {
							return plugin.transform || plugin.transformBundle;
						})) {
							map = magicString.generateMap({});
							if (typeof map.mappings === 'string') {
								map.mappings = sourcemapCodec.decode(map.mappings);
							}
							map = collapseSourcemaps(_this5, file, map, usedModules, bundleSourcemapChain);
						} else {
							map = magicString.generateMap({ file: file, includeContent: true });
						}

						map.sources = map.sources.map(normalize);

						timeEnd('sourcemap');
					}

					if (code[code.length - 1] !== '\n') code += '\n';
					return { code: code, map: map };
				});
			});
		}
	}, {
		key: 'sort',
		value: function sort() {
			var _this6 = this;

			var hasCycles = void 0;
			var seen = {};
			var ordered = [];

			var stronglyDependsOn = blank();
			var dependsOn = blank();

			this.modules.forEach(function (module) {
				stronglyDependsOn[module.id] = blank();
				dependsOn[module.id] = blank();
			});

			this.modules.forEach(function (module) {
				function processStrongDependency(dependency) {
					if (dependency === module || stronglyDependsOn[module.id][dependency.id]) return;

					stronglyDependsOn[module.id][dependency.id] = true;
					dependency.strongDependencies.forEach(processStrongDependency);
				}

				function processDependency(dependency) {
					if (dependency === module || dependsOn[module.id][dependency.id]) return;

					dependsOn[module.id][dependency.id] = true;
					dependency.dependencies.forEach(processDependency);
				}

				module.strongDependencies.forEach(processStrongDependency);
				module.dependencies.forEach(processDependency);
			});

			var visit = function visit(module) {
				if (seen[module.id]) {
					hasCycles = true;
					return;
				}

				seen[module.id] = true;

				module.dependencies.forEach(visit);
				ordered.push(module);
			};

			visit(this.entryModule);

			if (hasCycles) {
				ordered.forEach(function (a, i) {
					var _loop = function _loop() {
						var b = ordered[i];

						// TODO reinstate this! it no longer works
						if (stronglyDependsOn[a.id][b.id]) {
							// somewhere, there is a module that imports b before a. Because
							// b imports a, a is placed before b. We need to find the module
							// in question, so we can provide a useful error message
							var parent = '[[unknown]]';
							var visited = {};

							var findParent = function findParent(module) {
								if (dependsOn[module.id][a.id] && dependsOn[module.id][b.id]) {
									parent = module.id;
									return true;
								}
								visited[module.id] = true;
								for (var _i = 0; _i < module.dependencies.length; _i += 1) {
									var dependency = module.dependencies[_i];
									if (!visited[dependency.id] && findParent(dependency)) return true;
								}
							};

							findParent(_this6.entryModule);

							_this6.onwarn('Module ' + a.id + ' may be unable to evaluate without ' + b.id + ', but is included first due to a cyclical dependency. Consider swapping the import statements in ' + parent + ' to ensure correct ordering');
						}
					};

					for (i += 1; i < ordered.length; i += 1) {
						_loop();
					}
				});
			}

			return ordered;
		}
	}, {
		key: 'warn',
		value: function warn(warning) {
			warning.toString = function () {
				var str = '';

				if (warning.plugin) str += '(' + warning.plugin + ' plugin) ';
				if (warning.loc) str += relativeId(warning.loc.file) + ' (' + warning.loc.line + ':' + warning.loc.column + ') ';
				str += warning.message;

				return str;
			};

			this.onwarn(warning);
		}
	}]);

	return Bundle$$1;
}();

function addDeprecations(deprecations, warn) {
	var message = 'The following options have been renamed \u2014 please update your config: ' + deprecations.map(function (option) {
		return option.old + ' -> ' + option.new;
	}).join(', ');
	warn({
		code: 'DEPRECATED_OPTIONS',
		message: message,
		deprecations: deprecations
	});
}

function checkInputOptions(options) {
	if (options.transform || options.load || options.resolveId || options.resolveExternal) {
		throw new Error('The `transform`, `load`, `resolveId` and `resolveExternal` options are deprecated in favour of a unified plugin API. See https://github.com/rollup/rollup/wiki/Plugins for details');
	}
}

function checkOutputOptions(options) {
	if (options.format === 'es6') {
		error({
			message: 'The `es6` output format is deprecated – use `es` instead',
			url: 'https://rollupjs.org/#format-f-output-format-'
		});
	}

	if (!options.format) {
		error({
			message: 'You must specify options.format, which can be one of \'amd\', \'cjs\', \'es\', \'iife\' or \'umd\'',
			url: 'https://rollupjs.org/#format-f-output-format-'
		});
	}

	if (options.moduleId) {
		if (options.amd) throw new Error('Cannot have both options.amd and options.moduleId');
	}
}

var throwAsyncGenerateError = {
	get: function get() {
		throw new Error('bundle.generate(...) now returns a Promise instead of a { code, map } object');
	}
};

function rollup(_inputOptions) {
	try {
		if (!_inputOptions) {
			throw new Error('You must supply an options object to rollup');
		}

		var _mergeOptions = mergeOptions({
			config: _inputOptions,
			deprecateConfig: { input: true }
		}),
		    inputOptions = _mergeOptions.inputOptions,
		    deprecations = _mergeOptions.deprecations,
		    optionError = _mergeOptions.optionError;

		if (optionError) throw new Error(optionError);

		if (deprecations.length) addDeprecations(deprecations, inputOptions.onwarn);
		checkInputOptions(inputOptions);
		var bundle = new Bundle$1(inputOptions);

		timeStart('--BUILD--');

		return bundle.build().then(function () {
			timeEnd('--BUILD--');

			function generate(_outputOptions) {
				if (!_outputOptions) {
					throw new Error('You must supply an options object');
				}
				// since deprecateOptions, adds the output properties
				// to `inputOptions` so adding that lastly
				var consolidatedOutputOptions = Object.assign({}, {
					output: Object.assign({}, _outputOptions, _outputOptions.output, inputOptions.output)
				});
				var mergedOptions = mergeOptions({
					// just for backward compatiblity to fallback on root
					// if the option isn't present in `output`
					config: consolidatedOutputOptions,
					deprecateConfig: { output: true }
				});

				// check for errors
				if (mergedOptions.optionError) throw new Error(mergedOptions.optionError);

				// now outputOptions is an array, but rollup.rollup API doesn't support arrays
				var outputOptions = mergedOptions.outputOptions[0];
				var deprecations = mergedOptions.deprecations;

				if (deprecations.length) addDeprecations(deprecations, inputOptions.onwarn);
				checkOutputOptions(outputOptions);

				timeStart('--GENERATE--');

				var promise = Promise.resolve().then(function () {
					return bundle.render(outputOptions);
				}).then(function (rendered) {
					timeEnd('--GENERATE--');

					bundle.plugins.forEach(function (plugin) {
						if (plugin.ongenerate) {
							plugin.ongenerate(assign({
								bundle: result
							}, outputOptions), rendered);
						}
					});

					flushTime();

					return rendered;
				});

				Object.defineProperty(promise, 'code', throwAsyncGenerateError);
				Object.defineProperty(promise, 'map', throwAsyncGenerateError);

				return promise;
			}

			var result = {
				imports: bundle.externalModules.map(function (module) {
					return module.id;
				}),
				exports: keys(bundle.entryModule.exports),
				modules: bundle.orderedModules.map(function (module) {
					return module.toJSON();
				}),

				generate: generate,
				write: function write(outputOptions) {
					if (!outputOptions || !outputOptions.file && !outputOptions.dest) {
						error({
							code: 'MISSING_OPTION',
							message: 'You must specify output.file'
						});
					}

					return generate(outputOptions).then(function (result) {
						var file = outputOptions.file;
						var code = result.code,
						    map = result.map;


						var promises = [];

						if (outputOptions.sourcemap) {
							var url = void 0;

							if (outputOptions.sourcemap === 'inline') {
								url = map.toUrl();
							} else {
								url = path.basename(file) + '.map';
								promises.push(writeFile$1(file + '.map', map.toString()));
							}

							code += '//# ' + SOURCEMAPPING_URL + '=' + url + '\n';
						}

						promises.push(writeFile$1(file, code));
						return Promise.all(promises).then(function () {
							return mapSequence(bundle.plugins.filter(function (plugin) {
								return plugin.onwrite;
							}), function (plugin) {
								return Promise.resolve(plugin.onwrite(assign({
									bundle: result
								}, outputOptions), result));
							});
						});
					});
				}
			};

			return result;
		});
	} catch (err) {
		return Promise.reject(err);
	}
}

var chokidar = void 0;

try {
	chokidar = relative$1('chokidar', process.cwd());
} catch (err) {
	chokidar = null;
}

var chokidar$1 = chokidar;

var _createClass$85 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$86(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var opts = { encoding: 'utf-8', persistent: true };

var watchers = new Map();

function addTask(id, task, chokidarOptions, chokidarOptionsHash) {
	if (!watchers.has(chokidarOptionsHash)) watchers.set(chokidarOptionsHash, new Map());
	var group = watchers.get(chokidarOptionsHash);

	if (!group.has(id)) {
		var watcher = new FileWatcher(id, chokidarOptions, function () {
			group.delete(id);
		});

		if (watcher.fileExists) {
			group.set(id, watcher);
		} else {
			return;
		}
	}

	group.get(id).tasks.add(task);
}

function deleteTask(id, target, chokidarOptionsHash) {
	var group = watchers.get(chokidarOptionsHash);

	var watcher = group.get(id);
	if (watcher) {
		watcher.tasks.delete(target);

		if (watcher.tasks.size === 0) {
			watcher.close();
			group.delete(id);
		}
	}
}

var FileWatcher = function () {
	function FileWatcher(id, chokidarOptions, dispose) {
		var _this = this;

		_classCallCheck$86(this, FileWatcher);

		this.tasks = new Set();

		var data = void 0;

		try {
			fs.statSync(id);
			this.fileExists = true;
		} catch (err) {
			if (err.code === 'ENOENT') {
				// can't watch files that don't exist (e.g. injected
				// by plugins somehow)
				this.fileExists = false;
				return;
			} else {
				throw err;
			}
		}

		var handleWatchEvent = function handleWatchEvent(event) {
			if (event === 'rename' || event === 'unlink') {
				_this.fsWatcher.close();
				_this.trigger();
				dispose();
			} else {
				// this is necessary because we get duplicate events...
				var contents = fs.readFileSync(id, 'utf-8');
				if (contents !== data) {
					data = contents;
					_this.trigger();
				}
			}
		};

		if (chokidarOptions) {
			this.fsWatcher = chokidar$1.watch(id, chokidarOptions).on('all', handleWatchEvent);
		} else {
			this.fsWatcher = fs.watch(id, opts, handleWatchEvent);
		}
	}

	_createClass$85(FileWatcher, [{
		key: 'close',
		value: function close() {
			this.fsWatcher.close();
		}
	}, {
		key: 'trigger',
		value: function trigger() {
			this.tasks.forEach(function (task) {
				task.makeDirty();
			});
		}
	}]);

	return FileWatcher;
}();

var _createClass$84 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _toConsumableArray$4(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck$85(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$75(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$75(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var DELAY = 100;

var Watcher = function (_EventEmitter) {
	_inherits$75(Watcher, _EventEmitter);

	function Watcher(configs) {
		_classCallCheck$85(this, Watcher);

		var _this = _possibleConstructorReturn$75(this, (Watcher.__proto__ || Object.getPrototypeOf(Watcher)).call(this));

		_this.dirty = true;
		_this.running = false;
		_this.tasks = ensureArray(configs).map(function (config) {
			return new Task(_this, config);
		});
		_this.succeeded = false;

		process.nextTick(function () {
			_this._run();
		});
		return _this;
	}

	_createClass$84(Watcher, [{
		key: 'close',
		value: function close() {
			this.tasks.forEach(function (task) {
				task.close();
			});

			this.removeAllListeners();
		}
	}, {
		key: '_makeDirty',
		value: function _makeDirty() {
			var _this2 = this;

			if (this.dirty) return;
			this.dirty = true;

			if (!this.running) {
				setTimeout(function () {
					_this2._run();
				}, DELAY);
			}
		}
	}, {
		key: '_run',
		value: function _run() {
			var _this3 = this;

			this.running = true;
			this.dirty = false;

			this.emit('event', {
				code: 'START'
			});

			mapSequence(this.tasks, function (task) {
				return task.run();
			}).then(function () {
				_this3.succeeded = true;

				_this3.emit('event', {
					code: 'END'
				});
			}).catch(function (error) {
				_this3.emit('event', {
					code: _this3.succeeded ? 'ERROR' : 'FATAL',
					error: error
				});
			}).then(function () {
				_this3.running = false;

				if (_this3.dirty) {
					_this3._run();
				}
			});
		}
	}]);

	return Watcher;
}(EventEmitter);

var Task = function () {
	function Task(watcher, config) {
		_classCallCheck$85(this, Task);

		this.cache = null;
		this.watcher = watcher;

		this.dirty = true;
		this.closed = false;
		this.watched = new Set();

		var _mergeOptions = mergeOptions({ config: config }),
		    inputOptions = _mergeOptions.inputOptions,
		    outputOptions = _mergeOptions.outputOptions,
		    deprecations = _mergeOptions.deprecations;

		this.inputOptions = inputOptions;

		this.outputs = outputOptions;
		this.outputFiles = this.outputs.map(function (output) {
			return path__default.resolve(output.file);
		});

		var watchOptions = inputOptions.watch || {};
		if ('useChokidar' in watchOptions) watchOptions.chokidar = watchOptions.useChokidar;
		var chokidarOptions = 'chokidar' in watchOptions ? watchOptions.chokidar : !!chokidar$1;
		if (chokidarOptions) {
			chokidarOptions = Object.assign(chokidarOptions === true ? {} : chokidarOptions, {
				ignoreInitial: true
			});
		}

		if (chokidarOptions && !chokidar$1) {
			throw new Error('options.watch.chokidar was provided, but chokidar could not be found. Have you installed it?');
		}

		this.chokidarOptions = chokidarOptions;
		this.chokidarOptionsHash = JSON.stringify(chokidarOptions);

		this.filter = createFilter(watchOptions.include, watchOptions.exclude);
		this.deprecations = [].concat(_toConsumableArray$4(deprecations), _toConsumableArray$4(watchOptions._deprecations || []));
	}

	_createClass$84(Task, [{
		key: 'close',
		value: function close() {
			var _this4 = this;

			this.closed = true;
			this.watched.forEach(function (id) {
				deleteTask(id, _this4, _this4.chokidarOptionsHash);
			});
		}
	}, {
		key: 'makeDirty',
		value: function makeDirty() {
			if (!this.dirty) {
				this.dirty = true;
				this.watcher._makeDirty();
			}
		}
	}, {
		key: 'run',
		value: function run() {
			var _this5 = this;

			if (!this.dirty) return;
			this.dirty = false;

			var options = Object.assign(this.inputOptions, {
				cache: this.cache
			});

			var start = Date.now();

			this.watcher.emit('event', {
				code: 'BUNDLE_START',
				input: this.inputOptions.input,
				output: this.outputFiles
			});

			if (this.deprecations.length) {
				this.inputOptions.onwarn({
					code: 'DEPRECATED_OPTIONS',
					deprecations: this.deprecations,
					message: 'The following options have been renamed \u2014 please update your config: ' + this.deprecations.map(function (option) {
						return option.old + ' -> ' + option.new;
					}).join(', ')
				});
			}

			return rollup(options).then(function (bundle) {
				if (_this5.closed) return;

				_this5.cache = bundle;

				var watched = new Set();

				bundle.modules.forEach(function (module) {
					watched.add(module.id);
					_this5.watchFile(module.id);
				});

				_this5.watched.forEach(function (id) {
					if (!watched.has(id)) deleteTask(id, _this5, _this5.chokidarOptionsHash);
				});

				_this5.watched = watched;

				return Promise.all(_this5.outputs.map(function (output) {
					return bundle.write(output);
				}));
			}).then(function () {
				_this5.watcher.emit('event', {
					code: 'BUNDLE_END',
					input: _this5.inputOptions.input,
					output: _this5.outputFiles,
					duration: Date.now() - start
				});
			}).catch(function (error) {
				if (_this5.closed) return;

				if (_this5.cache) {
					_this5.cache.modules.forEach(function (module) {
						// this is necessary to ensure that any 'renamed' files
						// continue to be watched following an error
						_this5.watchFile(module.id);
					});
				}
				throw error;
			});
		}
	}, {
		key: 'watchFile',
		value: function watchFile(id) {
			if (!this.filter(id)) return;

			if (this.outputFiles.some(function (file) {
				return file === id;
			})) {
				throw new Error('Cannot import the generated bundle');
			}

			// this is necessary to ensure that any 'renamed' files
			// continue to be watched following an error
			addTask(id, this, this.chokidarOptions, this.chokidarOptionsHash);
		}
	}]);

	return Task;
}();

function watch$1(configs) {
	return new Watcher(configs);
}

var version = "0.53.0";

exports.rollup = rollup;
exports.watch = watch$1;
exports.VERSION = version;
//# sourceMappingURL=rollup.js.map
